import string
from TagObject import TagObject

TAG_NAME = "cluster"

CLUSTER_NAME_ALIAS = "Cluster Name: %s"
ACTUAL_CLUSTER_NAME = "Actual Cluster Name: %s"
ACTUAL_CLUSTER_NAME_DESC = "The Actual Cluster Name should be used for storage configuration, such as making a clustered file system"
CLUSTER_POPULATION = "Number of Members: %d"
DLM_TYPE = "Locking Type: Distributed"
GULM_TYPE = "Locking Type: GULM"
LOCKSERVER = "Lock Server:"
CONFIG_VERSION = "Config Version"
MCAST_MODE = "Cluster Manager using Multicast Mode. \n   Multicast Address: %s"

#Quorum disk attributes
QD_INTERVAL=("Interval: %s")
QD_TKO=("TKO Value: %s")
QD_VOTES=("Votes: %s")
QD_MIN_SCORE=("Minimum Score: %s")
QD_DEVICE=("Device: %s")
QD_LABEL=("Label: %s")
 
class Cluster(TagObject):
  def __init__(self):
    TagObject.__init__(self)
    self.TAG_NAME = TAG_NAME
    self.is_cfg_version_dirty = False

  def getProperties(self):
    stringbuf = ""
    special_kid = None
    kidname = ""
    numkids = 0
    dlm_locking = True
    gulm_ptr = None
    isMulticast = False
    mcast_address = ""

    stringbuf = CLUSTER_NAME_ALIAS % self.getAlias() + "\n"

    stringbuf = stringbuf + ACTUAL_CLUSTER_NAME % self.getAttribute("name") + "\n"
    stringbuf = stringbuf + ACTUAL_CLUSTER_NAME_DESC + "\n"

    stringbuf = stringbuf + CONFIG_VERSION + ": " + self.getConfigVersion() + "\n"

    kids = self.getChildren()
    for kid in kids:
      try:
        kidname = kid.getTagName()
        if kidname.strip() == "clusternodes":
          special_kid = kid
          break
      except KeyError, e:
        kidname = ""
        continue
   
    if special_kid != None:
      cnodes = special_kid.getChildren() #Should be clusternodes...
      numkids = len(cnodes) #numkids == number of nodes in cluster

    ###Here we walk through list of children an inefficient second time,
    ###But there will only be a few children here, and searching for
    ###both tag types in one loop will just make the loop code messy, imho
    kids = self.getChildren()
    for kid in kids:
      try:
        kidname = kid.getTagName()
        if kidname.strip() == "cman":
          dlm_locking = True
          gulm_ptr = None 
          cman_kids = kid.getChildren()
          for cman_kid in cman_kids:
            if cman_kid.getTagName() == "multicast":
              isMulticast = True
              mcast_address = cman_kid.getAttribute("addr")
              break
          break
        if kidname.strip() == "gulm":
          dlm_locking = False
          gulm_ptr = kid          
          break
      except KeyError, e:
        kidname = ""
        continue

    stringbuf = stringbuf + CLUSTER_POPULATION % numkids + "\n\n"

    if dlm_locking == True:
      stringbuf = stringbuf + DLM_TYPE + "\n"
      if isMulticast == True:
        stringbuf = stringbuf + (MCAST_MODE % mcast_address)
    else:
      stringbuf = stringbuf + GULM_TYPE + "\n"
      lockservers = gulm_ptr.getChildren()
      for server in lockservers:
        stringbuf = stringbuf + "  " + LOCKSERVER + "  " + server.getName() + "\n"

    return stringbuf

  def getConfigVersion(self):
    version = self.getAttribute("config_version")
    return version

  def setConfigVersion(self, version):
    current_version = self.getConfigVersion()
    if current_version == version:
      return

    self.addAttribute("config_version", version)
    self.is_cfg_version_dirty = True

  def incrementConfigVersion(self):
    version = self.getAttribute("config_version")
    intversion = int(version)
    intversion = intversion + 1
    self.addAttribute("config_version", str(intversion))
    #self.is_cfg_version_dirty = True

  def doesClusterUseQuorumDisk(self):
    kids = self.getChildren()
    for kid in kids:
      if kid.getTagName().strip() == "quorumd":
        return True
                                                                                
    return False

  def getQuorumdPtr(self):
    kids = self.getChildren()
    for kid in kids:
      if kid.getTagName().strip() == "quorumd":
        return kid
                                                                                
    return None



  #def addAttribute(self, name, value):
  #  if name == "config_version":
  #    cfg = self.getAttribute("config_version")
  #    if cfg != None:
  #      if int(value) != int(cfg):
  #        self.is_cfg_version_dirty = TRUE
  # 
  #  self.attr_hash[name] = value


  def generateXML(self, doc, parent=None):
    if self.is_cfg_version_dirty == False:
      self.incrementConfigVersion()
    else:
      self.is_cfg_version_dirty = False
    tag = doc.createElement(self.TAG_NAME)
    if parent != None:
      parent.appendChild(tag)
    else:
      doc.appendChild(tag)
    #tag = parent.createChildElement(TAG_NAME)
    self.exportAttributes(tag)
    #parent.appendChild(tag)
    if len(self.children) > 0:
      for child in self.children:
        if child == None:
          continue
        child.generateXML(doc, tag)

  def getNameAlias(self):
    return self.getAlias()
                                                                                
  def getAlias(self):
    #returns None if no alias set
    alias = self.getAttribute("alias")
    if alias == None:
      alias = self.getName()
      self.addAttribute('alias',alias)
    return alias
