import re
import os
from AccessControl import getSecurityManager
import cgi

from conga_constants import PLONE_ROOT, CLUSTER_NODE_NEED_AUTH, \
							HOMEBASE_ADD_CLUSTER, HOMEBASE_ADD_CLUSTER_INITIAL, \
							HOMEBASE_ADD_SYSTEM, HOMEBASE_ADD_USER, \
							HOMEBASE_DEL_SYSTEM, HOMEBASE_DEL_USER, HOMEBASE_PERMS, \
							STORAGE_FOLDER_PATH, CLUSTER_FOLDER_PATH

from ricci_bridge import getClusterConf
from ricci_communicator import RicciCommunicator, CERTS_DIR_PATH
from clusterOS import resolveOSType
from LuciSyslog import LuciSyslog

try:
	luci_log = LuciSyslog()
except:
	pass

def siteIsSetup(self):
	try:
		if os.path.isfile(CERTS_DIR_PATH + 'privkey.pem') and os.path.isfile(CERTS_DIR_PATH + 'cacert.pem'):
			return True
	except: pass
	return False

def strFilter(regex, replaceChar, arg):
	return re.sub(regex, replaceChar, arg)

def validateDelSystem(self, request):
	errors = list()
	messages = list()

	if '__SYSTEM' in request.form:
		sysNames = request.form['__SYSTEM']
		for i in sysNames:
			if i == '':
				continue
			dsResult = delSystem(self, i)
			if dsResult:
				errors.append(dsResult)
			else:
				messages.append('Removed storage system \"%s\" successfully' % i)

	if '__CLUSTER' in request.form:
		cluNames = request.form['__CLUSTER']
		for i in cluNames:
			if i == '':
				continue
			dcResult = delCluster(self, i)
			if dcResult:
				errors.append(dcResult)
			else:
				messages.append('Removed cluster \"%s\" successfully' % i)

	if len(errors) > 0:
		retCode = False
	else:
		retCode = True

	return (retCode, {'messages': messages, 'errors': errors })

def validateDelUser(self, request):
	errors = list()
	messages = list()

	if not 'deluserId' in request.form:
		return (False, {'errors': [ 'No User Id given' ] })
	
	userId = request.form['deluserId']

	try:
		user = self.portal_membership.getMemberById(userId)
		if not user:
			raise Exception, 'user %s does not exist' % userId
	except:
		return (False, {'errors': [ 'No such user: \"' + userId + '\"' ] })

	for i in getClusters(self):
		try:
			i[1].manage_delLocalRoles([userId])
		except:
			errors.append('Error deleting roles from cluster \"' + i[0] + '\" for user \"' + userId + '\"')

	for i in getStorage(self):
		try:
			i[1].manage_delLocalRoles([userId])
		except:
			errors.append('Error deleting roles from storage system \"' + i[0] + '\" for user \"' + userId + '\"')

	try:
		self.acl_users.userFolderDelUsers([userId])
	except:
		errors.append('Unable to delete user \"' + userId + '\"')
		return (False, {'errors': errors })

	messages.append('User \"' + userId + '\" has been deleted')
	return (True, {'errors': errors, 'messages': messages })

def validateAddUser(self, request):
	messages = list()

	if not 'newUserName' in request.form:
		return (False, { 'errors': ['No user name given'] })
	if not 'newPassword' in request.form:
		return (False, { 'errors': ['No password given'] })
	if not 'newPasswordConfirm' in request.form:
		return (False, {'errors': ['You didn\'t confirm the password'] })

	user = request.form['newUserName']

	if self.portal_membership.getMemberById(user):
		return (False, {'errors': ['The user \"' + user + '\" already exists']})

	passwd = request.form['newPassword']
	pwconfirm = request.form['newPasswordConfirm']

	if passwd != pwconfirm:
		return (False, {'errors': ['The passwords do not match']})

	try:
		self.portal_registration.addMember(user, passwd, properties = { 'username': user, 'password': passwd, 'confirm': passwd, 'roles': ['Member'], 'domains':[], 'email': user + '@example.com' })
	except:
		return (False, {'errors': [ 'Unable to add new user \"' + user + '\"' ] })

	if not self.portal_membership.getMemberById(user):
		return (False, {'errors': [ 'Unable to add new user \"' + user + '\"'] })

	messages.append('Added new user \"' + user + '\" successfully')
	return (True, {'messages': messages, 'params': { 'user': user }})

def validateAddClusterInitial(self, request):
	errors = list()
	messages = list()

	try:
		request.SESSION.delete('add_cluster_initial')
		request.SESSION.delete('add_cluster')
	except:
		pass

	cur_host = None
	try:
		sysData = request.form['__SYSTEM0']
		if not sysData or len(sysData) < 1:
			raise Exception, 'no node was given'
		cur_host = sysData[0]
	except Exception, e:
		luci_log.debug_verbose('vACI0: %s' % str(e))
		return (False, { 'errors': [ 'You must provide the address of at least one node in the cluster you wish to add.' ]})

	cur_entry = { 'host': cur_host }
	try:
		if len(sysData) < 2 or not sysData[1]:
			raise Exception, 'no password'
		cur_pass = sysData[1]
		cur_entry['passwd'] = cur_pass
	except:
		luci_log.debug_verbose('vACI1: %s no password given')
		request.SESSION.set('add_cluster_initial', cur_entry)
		return (False, { 'errors': [ 'No password was given for %s' % cur_host ] })

	check_certs = False
	try:
		check_certs = request.form.has_key('check_certs')
	except:
		check_certs = False

	cur_host_trusted = False
	try:
		cur_host_trusted = request.form.has_key('host_is_trusted')
	except:
		cur_host_trusted = False

	trust_shown = False
	try:
		trust_shown = request.form.has_key('trust_shown')
	except:
		trust_shown = False

	cur_host_fp = None
	try:
		cur_host_fp = request.form['host_fingerprint'].strip()
		if not cur_host_fp:
			cur_host_fp = None
	except:
		cur_host_fp = None

	try:
		rc = RicciCommunicator(cur_host, enforce_trust=False)
		if not rc:
			raise Exception, 'rc is None'
		cur_fp = rc.fingerprint()
		if cur_host_fp is not None:
			cur_entry['fp'] = cur_host_fp
		else:
			cur_entry['fp'] = cur_fp[1]
	except Exception, e:
		luci_log.debug_verbose('vACI2: %s: %s' % (cur_host, str(e)))
		request.SESSION.set('add_cluster_initial', cur_entry)
		return (False, { 'errors': [ 'Unable to establish a connection to the ricci agent on %s: %s' \
			% (cur_host, str(e)) ] })

	if not check_certs or cur_host_trusted:
		try:
			if cur_host_fp is not None and cur_host_fp != cur_fp[1]:
				errmsg = 'The key fingerprint for %s has changed from under us. It was \"%s\" and is now \"%s\".' \
					% (cur_host, cur_host_fp, cur_fp[1])
				request.SESSION.set('add_cluster_initial', cur_entry)
				luci_log.info('SECURITY: %s' % errmsg)
				return (False, { 'errors': [ errmsg ] })
			if trust_shown is True and cur_host_trusted is False:
				errmsg = 'You must elect to trust \"%s\" or abort the addition of the cluster to Luci.' % cur_host
				request.SESSION.set('add_cluster_initial', cur_entry)
				return (False, { 'errors': [ errmsg ] })
			rc.trust()
		except Exception, e:
			luci_log.debug_verbose('vACI3: %s %s' % (cur_host, str(e)))
			request.SESSION.set('add_cluster_initial', cur_entry)
			return (False, { 'errors': [ 'Unable to establish trust for host %s' % (cur_host, str(e)) ] })
	elif check_certs:
		if not rc.trusted():
			msg = '%s has %s fingerprint %s' \
				% (cur_host, cur_fp[0], cur_fp[1])
		else:
			cur_host_trusted = True
			cur_entry['trusted'] = True
			msg = 'Host %s %s fingerprint %s is already trusted.' \
				% (cur_host, cur_fp[0], cur_fp[1])
		request.SESSION.set('add_cluster_initial', cur_entry)
		messages.append(msg)
		return (True, { 'messages': [ msg ] })

	try:
		request.SESSION.delete('add_cluster_initial')
	except:
		pass

	rc = None
	try:
		rc = RicciCommunicator(cur_host, enforce_trust=True)
		if not rc:
			raise Exception, 'rc is None'
		cur_entry['trusted'] = rc.trusted()
	except Exception, e:
		luci_log.debug_verbose('vACI4: %s %s' % (cur_host, str(e)))
		request.SESSION.set('add_cluster_initial', cur_entry)
		return (False, { 'errors': [ 'Unable to connect to the ricci agent on %s' % cur_host ] })

	prev_auth = rc.authed()
	if not prev_auth:
		try:
			rc.auth(cur_pass)
			if not rc.authed():
				raise Exception, 'authentication failed'
		except Exception, e:
			errmsg = 'Unable to authenticate to the ricci agent on %s: %s' % (cur_host, str(e))
			luci_log.debug_verbose('vACI5: %s: %s' % (cur_host, str(e)))
			request.SESSION.set('add_cluster_initial', cur_entry)
			return (False, { 'errors': [ 'Unable to authenticate to the ricci agent on \"%s\"' % cur_host ] })

	del cur_entry

	try:
		cluster_info = rc.cluster_info()
	except:
		cluster_info = None

	if not cluster_info or not cluster_info[0]:
		if not prev_auth:
			try:
				rc.unauth()
			except:
				pass

		if not cluster_info:
			errmsg = 'An error occurred while attempting to retrieve the cluster.conf file from \"%s\"' % cur_host
		else:
			errmsg = '\"%s\" reports is not a member of any cluster.' % cur_host
		return (False, { 'errors': [ errmsg ] })

	cluster_name = cluster_info[0]

	cluster_os = resolveOSType(rc.os())
	luci_log.debug_verbose('vACI5a: cluster os is %s (%s)' \
		% (cluster_os, rc.os()))

	try:
		cluster_conf = getClusterConf(rc)
	except:
		cluster_conf = None

	if cluster_conf:
		try:
			node_list = getClusterConfNodes(cluster_conf)
		except:
			node_list = None

	# Make sure a cluster with this name is not already managed before
	# going any further.
	try:
		dummy = self.restrictedTraverse(CLUSTER_FOLDER_PATH + cluster_name)
		if not dummy:
			raise Exception, 'no existing cluster'
		errors.append('A cluster named \"%s\" is already managed.')
		if not prev_auth:
			try:
				rc.unauth()
			except:
				pass
		return (False, { 'errors': errors })
	except:
		pass

	if not cluster_conf or not node_list or len(node_list) < 1:
		if not prev_auth:
			try:
				rc.unauth()
			except:
				pass
		return (False, { 'errors': [ 'Error retrieving the nodes list for cluster \"%s\" from node \"%s\"' % (cluster_name, cur_host) ] })

	same_node_passwds = False
	try:
		same_node_passwds = 'allSameCheckBox' in request.form
	except:
		same_node_passwds = False

	add_cluster = { 'name': cluster_name,
					'nodes': {},
					'cluster_os':cluster_os,
					'pass': 0,
					'identical_passwds': same_node_passwds,
					'check_certs': check_certs }

	for i in node_list:
		cur_node = { 'host': i }
		if same_node_passwds:
			cur_node['passwd'] = cur_pass
		add_cluster['nodes'][i] = cur_node
	request.SESSION.set('add_cluster', add_cluster)
	request.response.redirect('/luci/homebase/index_html?pagetype=%s' % HOMEBASE_ADD_CLUSTER)

def parseHostForm(request, check_certs):
	errors = list()
	messages = list()
	system_list = {}

	try:
		num_storage = int(request.form['numStorage'].strip())
	except Exception, e:
		luci_log.debug_verbose('PHF1: numStorage field missing: %s' % str(e))
		errors.append('The number of systems entered could not be determined.')

	trust_shown = False
	try:
		trust_shown = request.form.has_key('trust_shown')
	except:
		trust_shown = False

	incomplete = False
	i = 0
	while i < num_storage:
		try:
			sysData = request.form['__SYSTEM%d' % i]
			if len(sysData) < 1 or not sysData[0]:
				raise Exception, 'no hostname'
			cur_host = sysData[0]
			if cur_host in system_list:
				errors.append('You have added \"%s\" more than once.' % cur_host)
				raise Exception, '%s added more than once' % cur_host
		except:
			i += 1
			continue

		cur_system = { 'host': cur_host }

		if len(sysData) < 2 or not sysData[1]:
			errors.append('No password for %s (entry %d).' % (cur_host, i))
			cur_passwd = None
		else:
			cur_passwd = sysData[1]
			cur_system['passwd'] = cur_passwd

		try:
			cur_fp = request.form['__SYSTEM%dFingerprint' % i].strip()
			if not cur_fp:
				raise Exception, 'fingerprint is blank'
			cur_system['fp'] = cur_fp
		except:
			cur_fp = None

		try:
			cur_set_trust = request.form.has_key('__SYSTEM%dTrusted' % i)
		except:
			cur_set_trust = False

		if check_certs or (cur_fp is not None and cur_set_trust is True):
			try:
				rc = RicciCommunicator(cur_host, enforce_trust=False)
				if not rc:
					raise Exception, 'rc is None'
				cur_system['prev_auth'] = rc.authed()
				fp = rc.fingerprint()

				if cur_set_trust is True and cur_fp is not None:
					cur_system['fp'] = cur_fp
					if cur_fp != fp[1]:
						errmsg = '1The key fingerprint for %s has changed from under us. It was \"%s\" and is now \"%s\".' % (cur_host, cur_fp, fp[1])
						errors.append(errmsg)
						luci_log.info('SECURITY: %s' % errmsg)
						cur_system['error'] = True
						incomplete = True
					else:
						rc.trust()
						cur_system['trusted'] = True
				else:
					cur_system['fp'] = fp[1]

				if not rc.trusted():
					incomplete = True
					msg = '%s has %s fingerprint %s' % (cur_host, fp[0], fp[1])
				else:
					cur_system['trusted'] = True
					msg = '%s %s fingerprint %s is already trusted.' % (cur_host, fp[0], fp[1])

				if check_certs:
					messages.append(msg)
			except Exception, e:
				cur_system['error'] = True
				try:
					del cur_system['trusted']
				except:
					pass
				errors.append('Unable to retrieve the SSL fingerprint for node %s: %s' % (cur_host, str(e)))
				luci_log.debug_verbose('PHF2: %s: %s' \
					% (cur_host, str(e)))
		else:
			try:
				rc = RicciCommunicator(cur_host, enforce_trust=False)
				if not rc:
					raise Exception, 'rc is None'

				if not rc.trusted() and (trust_shown is True and cur_set_trust is False):
					incomplete = True
					cur_system['error'] = True
					errors.append('You must either trust \"%s\" or remove it.' % cur_host)
				else:
					# The user doesn't care. Trust the system.
					rc.trust()
					cur_system['trusted'] = True
					cur_system['prev_auth'] = rc.authed()
			except Exception, e:
				incomplete = True
				cur_system['error'] = True
				try:
					if not 'prev_auth' in cur_system:
						del cur_system['trusted']
						rc.untrust()
				except:
					pass		
				errors.append('Unable to add the key for node %s to the trusted keys list.' % cur_host)
				luci_log.debug_verbose('PHF3: %s: %s' % (cur_host, str(e)))
		system_list[cur_host] = cur_system
		i += 1

	return system_list, incomplete, errors, messages

def validateAddCluster(self, request):
	errors = list()

	try:
		request.SESSION.delete('add_cluster')
		request.SESSION.delete('add_cluster_initial')
	except:
		pass

	try:
		cluster_name = request.form['clusterName'].strip()
	except:
		luci_log.debug_verbose('VAC0: no cluster name')
		errors.append('No cluster name was given.')

	try:
		cluster_os = request.form['cluster_os'].strip()
	except:
		luci_log.debug_verbose('VAC1: no cluster os')
		errors.append('Unable to determine the version of cluster %s.' % cluster_name)

	check_certs = False
	try:
		check_certs = 'check_certs' in request.form
	except:
		check_certs = False

	try:
		pass_num = int(request.form['pass'].strip()) + 1
	except:
		pass_num = 1

	same_node_passwds = False
	try:
		same_node_passwds = 'allSameCheckBox' in request.form
	except:
		same_node_passwds = False

	add_cluster = { 'name': cluster_name,
					'pass': pass_num,
					'cluster_os': cluster_os,
					'identical_passwds': same_node_passwds,
					'check_certs': check_certs }

	system_list, incomplete, new_errors, messages = parseHostForm(request, check_certs)
	errors.extend(new_errors)
	add_cluster['nodes'] = system_list

	for i in system_list:
		cur_system = system_list[i]

		cur_host_trusted = 'trusted' in cur_system
		cur_host = cur_system['host']
		prev_auth = False
		try:
			cur_passwd = cur_system['passwd']
		except:
			cur_passwd = None

		if (cur_host_trusted or not check_certs) and cur_passwd:
			try:
				rc = RicciCommunicator(cur_host)
				prev_auth = rc.authed()
			except Exception, e:
				errors.append('Unable to connect to the ricci agent on %s: %s' \
					% (cur_host, str(e)))
				incomplete = True
				cur_system['errors'] = True
				luci_log.debug_verbose('VAC2: %s: %s' \
					% (cur_host, str(e)))
				continue

			try:
				rc.auth(cur_passwd)
				if not rc.authed():
					raise Exception, 'authentication failed'
			except Exception, e:
				errors.append('Unable to authenticate to the ricci agent on %s: %s' \
					% (cur_host, str(e)))
				incomplete = True
				cur_system['errors'] = True
				luci_log.debug_verbose('VAC3: %s: %s' % (cur_host, str(e)))
				continue

			cluster_info = rc.cluster_info()
			if cluster_info[0] != cluster_name and cluster_info[1] != cluster_name:
				incomplete = True
				cur_system['errors'] = True

				if cluster_info[0]:
					cur_cluster_name = cluster_info[0]
				else:
					cur_cluster_name = cluster_info[1]

				if cur_cluster_name:
					err_msg = 'Node %s reports it is in cluster \"%s\" and we expect \"%s\"' \
						% (cur_host, cur_cluster_name % cluster_name)
				else:
					err_msg = 'Node %s reports it is not a member of any cluster' % cur_host

				if not prev_auth:
					try:
						rc.unauth()
					except Exception, e:
						luci_log.debug_verbose('VAC4: %s: %s' % (cur_host, str(e)))

				errors.append(err_msg)
				luci_log.debug_verbose('VAC5: %s' % err_msg)
				continue

			cur_os = resolveOSType(rc.os())
			if cur_os != cluster_os:
				luci_log.debug_verbose('VAC5a: \"%s\" / \"%s\" -> \"%s\"' \
					% (cluster_os, rc.os(), cur_os))
				incomplete = True
				cur_system['errors'] = True

				if not prev_auth:
					try:
						rc.unauth()
					except Exception, e:
						luci_log.debug_verbose('VAC6: %s: %s' % (cur_host, str(e)))

				err_msg = 'Node %s reports its cluster version is %s and we expect %s' \
					% (cur_host, cur_os, cluster_os)

				errors.append(err_msg)
				luci_log.debug_verbose('VAC7: %s' % err_msg)
				continue
		else:
			incomplete = True

	if len(errors) > 0:
		incomplete = True

	if not incomplete or request.form.has_key('asis'):
		err_msg = manageCluster(self, cluster_name, system_list, cluster_os)
		if err_msg:
			incomplete = True
			errors.append('An error occurred while creating the database objects for cluster %s: %s' \
				% (cluster_name, err_msg))
			luci_log.debug_verbose('VAC7: error adding cluster DB objects for %s: %s' \
				% (cluster_name, err_msg))
		else:
			messages.append('Cluster %s is now managed by Luci' % cluster_name)
			incomplete = False

	if incomplete:
		add_cluster['incomplete'] = True
		request.SESSION.set('add_cluster', add_cluster)
		return_code = False
	else:
		return_code = True
	
	return (return_code, {'errors': errors, 'messages': messages })

def validateAddSystem(self, request):
	try:
		request.SESSION.delete('add_systems')
	except:
		pass

	check_certs = False
	try:
		check_certs = 'check_certs' in request.form
	except:
		check_certs = False

	add_systems, incomplete, errors, messages = parseHostForm(request, check_certs)
	delete_keys = list()
	for i in add_systems:
		cur_system = add_systems[i]

		cur_host_trusted = 'trusted' in cur_system
		cur_host = cur_system['host']

		try:
			cur_passwd = cur_system['passwd']
		except:
			cur_passwd = None

		if (cur_host_trusted or not check_certs) and cur_passwd:
			csResult = createSystem(self, cur_host, cur_passwd)
			if csResult:
				incomplete = True
				cur_system['error'] = True
				errors.append(csResult)
			else:
				delete_keys.append(i)
				messages.append('Added storage system \"%s\" successfully' \
					% cur_host)

	for i in delete_keys:
		try:
			del add_systems[i]
		except:
			pass

	if len(errors) > 0:
		return_code = False
	else:
		return_code = True

	if incomplete:
		try:
			request.SESSION.set('add_systems', add_systems)
		except Exception, e:
			luci_log.debug_verbose('validateSA2: %s' % str(e))
		return_code = False
	else:
		try:
			request.SESSION.delete('add_systems')
			del add_systems
		except:
			pass

	return (return_code, { 'errors': errors, 'messages': messages})

def validatePerms(self, request):
	userId = None
	messages = list()
	errors = list()

	try:
		userId = request.form['userList']
	except:
		return (False, {'errors': [ 'No user specified' ], 'params': { 'user': userId }})

	user = self.portal_membership.getMemberById(userId)
	if not user:
		return (False, {'errors': [ 'Invalid user specified' ], 'params': { 'user': userId }})

	userId = user.getUserId()

	clusters = self.restrictedTraverse(PLONE_ROOT + '/systems/cluster/objectItems')('Folder')
	if not '__CLUSTER' in request.form:
		for i in clusters:
			try:
				if user.has_role('View', i[1]):
					roles = list(i[1].get_local_roles_for_userid(userId))
					roles.remove('View')

					if roles:
						i[1].manage_setLocalRoles(userId, roles)
					else:
						i[1].manage_delLocalRoles([userId])
					messages.append('Removed permission for ' + userId + ' for cluster ' + i[0])
			except:
				errors.append('Failed to remove permission for ' + userId + ' for cluster ' + i[0])
	else:
		for i in clusters:
			if i[0] in request.form['__CLUSTER']:
				try:
					if not user.has_role('View', i[1]):
						roles = list(i[1].get_local_roles_for_userid(userId))
						roles.append('View')
						i[1].manage_setLocalRoles(userId, roles)
						messages.append('Added permission for ' + userId + ' for cluster ' + i[0])
				except:
					errors.append('Failed to add permission for ' + userId + ' for cluster ' + i[0])
			else:
				try:
					if user.has_role('View', i[1]):
						roles = list(i[1].get_local_roles_for_userid(userId))
						roles.remove('View')

						if roles:
							i[1].manage_setLocalRoles(userId, roles)
						else:
							i[1].manage_delLocalRoles([userId])

						messages.append('Removed permission for ' + userId + ' for cluster ' + i[0])
				except:
					errors.append('Failed to remove permission for ' + userId + ' for cluster ' + i[0])

	storage = self.restrictedTraverse(PLONE_ROOT + '/systems/storage/objectItems')('Folder')
	if not '__SYSTEM' in request.form:
		for i in storage:
			try:
				if user.has_role('View', i[1]):
					roles = list(i[1].get_local_roles_for_userid(userId))
					roles.remove('View')

					if roles:
						i[1].manage_setLocalRoles(userId, roles)
					else:
						i[1].manage_delLocalRoles([userId])
					messages.append('Removed permission for ' + userId + ' for ' + i[0])
			except:
				errors.append('Failed to remove permission for ' + userId + ' for ' + i[0])
	else:
		for i in storage:
			if i[0] in request.form['__SYSTEM']:
				try:
					if not user.has_role('View', i[1]):
						roles = list(i[1].get_local_roles_for_userid(userId))
						roles.append('View')
						i[1].manage_setLocalRoles(userId, roles)
						messages.append('Added permission for ' + userId + ' for system ' + i[0])
				except:
					errors.append('Failed to add permission for ' + userId + ' for system ' + i[0])
			else:
				try:
					if user.has_role('View', i[1]):
						roles = list(i[1].get_local_roles_for_userid(userId))
						roles.remove('View')

						if roles:
							i[1].manage_setLocalRoles(userId, roles)
						else:
							i[1].manage_delLocalRoles([userId])

						messages.append('Removed permission for ' + userId + ' for system ' + i[0])
				except:
					errors.append('Failed to remove permission for ' + userId + ' for system ' + i[0])

	if len(errors) > 0:
		returnCode = False
	else:
		returnCode = True

	return (returnCode, {'errors': errors, 'messages': messages, 'params': {'user': userId }})

def validateAuthenticate(self, request):
	try:
		request.SESSION.delete('auth_systems')
		request.SESSION.delete('auth_status')
	except:
		pass

	check_certs = False
	try:
		check_certs = 'check_certs' in request.form
	except:
		check_certs = False

	system_list, incomplete, errors, messages = parseHostForm(request, check_certs)
	delete_keys = list()
	for i in system_list:
		cur_system = system_list[i]

		cur_host_trusted = 'trusted' in cur_system
		cur_host = cur_system['host']

		try:
			cur_passwd = cur_system['passwd']
		except:
			cur_passwd = None

		if (cur_host_trusted or not check_certs) and cur_passwd:
			try:
				rc = RicciCommunicator(cur_host, enforce_trust=True)
				if not rc:
					raise Exception, 'connection failed'
			except Exception, e:
				luci_log.debug_verbose('validateAuth0: %s: %s' % (cur_host, str(e)))
				errors.append('Unable to communicate with the ricci agent on %s: %s' \
					% (cur_host, str(e)))
				incomplete = True
				cur_system['error'] = True
				continue

			try:
				if rc.authed():
					messages.append('%s is already authenticated.' % cur_host)
				else:
					rc.auth(cur_passwd)
					if not rc.authed():
						raise Exception, 'authentication failed'
					messages.append('Authenticated to %s successfully' \
						% cur_host)
				delete_keys.append(i)
				try:
					delNodeFlag(self, getStorageNode(self, cur_host), CLUSTER_NODE_NEED_AUTH)
				except:
					pass

				try:
					delNodeFlag(self, getClusterNode(self, cur_host, rc.cluster_info()[0]), CLUSTER_NODE_NEED_AUTH)
				except:
					pass
			except Exception, e:
				errors.append('Unable to authenticate to %s: %s' % (cur_host, str(e)))
				luci_log.debug_verbose('validateAuth1: %s: %s' % (cur_host, str(e)))
				incomplete = True
				cur_system['error'] = True

	for i in delete_keys:
		try:
			del system_list[i]
		except:
			pass

	if len(errors) > 0:
		return_code = False
	else:
		return_code = True

	if incomplete:
		try:
			request.SESSION.set('auth_systems', system_list)
		except Exception, e:
			luci_log.debug_verbose('validateAuthenticate2: %s' % str(e))
		return_code = False
	else:
		try:
			request.SESSION.delete('auth_systems')
		except:
			pass

	auth_msgs = {}
	if len(errors) > 0:
		auth_msgs['errors'] = errors
	if len(messages) > 0:
		auth_msgs['messages'] = messages
	request.SESSION.set('auth_status', auth_msgs)
	request.response.redirect('/luci/homebase/index_html?pagetype=5')

formValidators = [
	validateAddUser,
	validateAddSystem,
	validatePerms,
	validateDelUser,
	validateDelSystem,
	validateAddCluster,
	validateAddClusterInitial,
	validateAuthenticate
]

def userAuthenticated(self):
	try:
		if (isAdmin(self) or getSecurityManager().getUser().has_role('Authenticated', self.restrictedTraverse(PLONE_ROOT))):
			return True
	except Exception, e:
		luci_log.debug_verbose('UA0: %s' % str(e)) 
	return False

def isAdmin(self):
	try:
		return getSecurityManager().getUser().has_role('Owner', self.restrictedTraverse(PLONE_ROOT))
	except Exception, e:
		luci_log.debug_verbose('IA0: %s' % str(e)) 
	return False

def userIsAdmin(self, userId):
	try:
		return self.portal_membership.getMemberById(userId).has_role('Owner', self.restrictedTraverse(PLONE_ROOT))
	except Exception, e:
		luci_log.debug_verbose('UIA0: %s: %s' % (userId, str(e)))
	return False

def homebaseControlPost(self, request):
	if 'ACTUAL_URL' in request:
		url = request['ACTUAL_URL']
	else:
		url = '.'

	if 'pagetype' in request.form:
		pagetype = int(request.form['pagetype'])
	else:
		try:
			request.SESSION.set('checkRet', {})
		except:
			pass
		return homebasePortal(self, request, '.', '0')

	try:
		validatorFn = formValidators[pagetype - 1]
	except:
		try:
			request.SESSION.set('checkRet', {})
		except:
			pass
		return homebasePortal(self, request, '.', '0')

	ret = validatorFn(self, request)
	params = None

	if ret and len(ret) > 1 and ret[1]:
		if 'params' in ret[1]:
			params = ret[1]['params']
		request.SESSION.set('checkRet', ret[1])

	return homebasePortal(self, request, url, pagetype, params)

def homebaseControl(self, request):
	if request.REQUEST_METHOD == 'POST':
		return homebaseControlPost(self, request)

	try:
		request.SESSION.set('checkRet', {})
	except:
		pass

	if 'ACTUAL_URL' in request:
		url = request['ACTUAL_URL']
	else:
		url = '.'

	if 'pagetype' in request:
		pagetype = request['pagetype']
	else:
		pagetype = '0'

	return homebasePortal(self, request, url, pagetype)

def homebasePortal(self, request=None, url=None, pagetype=None, params=None):
	ret = {}
	temp = list()
	index = 0
	cur = None

	pagetype = str(pagetype)

#
# The Add System page
#

	try:
		if havePermAddStorage(self):
			addSystem = {}
			addSystem['Title'] = 'Add a System'
			addSystem['absolute_url'] = url + '?pagetype=' + HOMEBASE_ADD_SYSTEM
			addSystem['Description'] = 'Add a system to the Luci storage management interface.'
			if pagetype == HOMEBASE_ADD_SYSTEM:
				cur = addSystem
				addSystem['currentItem'] = True
				ret['curIndex'] = index
			else:
				addSystem['currentItem'] = False
			index += 1
			temp.append(addSystem)
	except: pass

	# Initial add cluster page
	try:
		if havePermAddCluster(self):
			addCluster = {}
			addCluster['Title'] = 'Add an Existing Cluster'
			addCluster['absolute_url'] = url + '?pagetype=' + HOMEBASE_ADD_CLUSTER_INITIAL
			addCluster['Description'] = 'Add an existing cluster to the Luci cluster management interface.'
			if pagetype == HOMEBASE_ADD_CLUSTER_INITIAL or pagetype == HOMEBASE_ADD_CLUSTER:
				addCluster['currentItem'] = True
				ret['curIndex'] = index
				cur = addCluster
			else:
				addCluster['currentItem'] = False
			index += 1
			temp.append(addCluster)
	except: pass

#
# The Remove Storage System page
#

	try:
		if (havePermRemStorage(self) and havePermRemCluster(self) and (getStorage(self) or getClusters(self))):
			remSystem = {}
			remSystem['Title'] = 'Manage Systems'
			remSystem['absolute_url'] = url + '?pagetype=' + HOMEBASE_DEL_SYSTEM
			remSystem['Description'] = 'Update or remove storage systems and clusters.'
			if pagetype == HOMEBASE_DEL_SYSTEM:
				remSystem['currentItem'] = True
				ret['curIndex'] = index
				cur = remSystem
			else:
				remSystem['currentItem'] = False
			index += 1
			temp.append(remSystem)
	except: pass

#
# Add a Luci user.
#

	try:
		if havePermAddUser(self):
			addUser = {}
			addUser['Title'] = 'Add a User'
			addUser['absolute_url'] = url + '?pagetype=' + HOMEBASE_ADD_USER
			addUser['Description'] = 'Add a user to the Luci interface.'
			if pagetype == HOMEBASE_ADD_USER:
				addUser['currentItem'] = True
				ret['curIndex'] = index
				cur = addUser
			else:
				addUser['currentItem'] = False
			index += 1
			temp.append(addUser)
	except: pass

#
# Delete a Luci user
#

	try:
		if (self.portal_membership.listMembers() and havePermDelUser(self)):
			delUser = {}
			delUser['Title'] = 'Delete a User'
			delUser['absolute_url'] = url + '?pagetype=' + HOMEBASE_DEL_USER
			delUser['Description'] = 'Delete a Luci user.'
			if pagetype == HOMEBASE_DEL_USER:
				delUser['currentItem'] = True
				ret['curIndex'] = index
				cur = delUser
			else:
				delUser['currentItem'] = False
			index += 1
			temp.append(delUser)
	except: pass

#
# The User Permissions Page
#
	try:
		if (havePermEditPerms(self) and self.portal_membership.listMembers() and (getStorage(self) or getClusters(self))):
			userPerm = {}
			userPerm['Title'] = 'User Permissions'
			userPerm['absolute_url'] = url + '?pagetype=' + HOMEBASE_PERMS
			userPerm['Description'] = 'Set permissions for Luci users.'
			if pagetype == HOMEBASE_PERMS:
				userPerm['currentItem'] = True
				ret['curIndex'] = index
				cur = userPerm
			else:
				userPerm['currentItem'] = False
			index += 1
			temp.append(userPerm)
	except: pass

	if not 'curIndex' in ret:
		ret['curIndex'] = 0

	if cur and 'absolute_url' in cur and params:
		cur['base_url'] = cur['absolute_url']
		for i in params:
			cur['absolute_url'] += '&' + cgi.escape(i) + '=' + cgi.escape(params[i])
	elif cur and 'absolute_url' in cur:
		cur['base_url'] = cur['absolute_url']
	else:
		cur = {}
		cur['base_url'] = '#'

	ret['children'] = temp
	return ret

def getClusterSystems(self, clusterName):
	if isAdmin(self):
		try:
			return self.restrictedTraverse(CLUSTER_FOLDER_PATH + clusterName + '/objectItems')('Folder')
		except Exception, e:
			luci_log.debug_verbose('GCSy0: %s: %s' % (clusterName, str(e)))
			return None

	try:
		i = getSecurityManager().getUser()
		if not i:
			raise Exception, 'security manager says no user'
	except Exception, e:
		luci_log.debug_verbose('GCSy1: %s: %s' % (clusterName, str(e)))
		return None

	try:
		csystems = self.restrictedTraverse(CLUSTER_FOLDER_PATH + clusterName + '/objectItems')('Folder')
		if not csystems or len(csystems) < 1:
			return None
	except Exception, e:
		luci_log.debug_verbose('GCSy2: %s: %s' % (clusterName, str(e)))
		return None

	allowedCSystems = list()
	for c in csystems:
		try:
			if i.has_role('View', c[1]):
				allowedCSystems.append(c)
		except Exception, e:
			luci_log.debug_verbose('GCSy3: %s: %s: %s' \
				% (clusterName, c[0], str(e)))

	return allowedCSystems

def getClusters(self):
	if isAdmin(self):
		try:
			return self.restrictedTraverse(PLONE_ROOT + '/systems/cluster/objectItems')('Folder')
		except Exception, e:
			luci_log.debug_verbose('GC0: %s' % str(e))
			return None
	try:
		i = getSecurityManager().getUser()
		if not i:
			raise Exception, 'GSMGU failed'
	except Exception, e:
		luci_log.debug_verbose('GC1: %s' % str(e))
		return None

	try:
		clusters = self.restrictedTraverse(PLONE_ROOT + '/systems/cluster/objectItems')('Folder')
		if not clusters or len(clusters) < 1:
			return None
	except Exception, e:
		luci_log.debug_verbose('GC2: %s' % str(e))
		return None

	allowedClusters = list()
	for c in clusters:
		try:
			if i.has_role('View', c[1]):
				allowedClusters.append(c)
		except Exception, e:
			luci_log.debug_verbose('GC3: %s: %s' % (c[0], str(e)))

	return allowedClusters

def getStorage(self):
	if isAdmin(self):
		try:
			return self.restrictedTraverse(PLONE_ROOT + '/systems/storage/objectItems')('Folder')
		except Exception, e:
			luci_log.debug_verbose('GS0: %s' % str(e))
			return None

	try:
		i = getSecurityManager().getUser()
		if not i:
			raise Exception, 'GSMGU failed'
	except Exception, e:
		luci_log.debug_verbose('GS1: %s' % str(e))
		return None

	try:
		storage = self.restrictedTraverse(PLONE_ROOT + '/systems/storage/objectItems')('Folder')
		if not storage or len(storage) < 1:
			return None
	except Exception, e:
		luci_log.debug_verbose('GS2: %s' % str(e))
		return None

	allowedStorage = list()
	for s in storage:
		try:
			if i.has_role('View', s[1]):
				allowedStorage.append(s)
		except Exception, e:
			luci_log.debug_verbose('GS3: %s' % str(e))

	return allowedStorage

def createSystem(self, host, passwd):
	try:
		dummy = self.restrictedTraverse(str(STORAGE_FOLDER_PATH + host)).objectItems()
		luci_log.debug_verbose('CS0: %s already exists' % host)
		return 'Storage system %s is already managed' % host
	except:
		pass

	try:
		rc = RicciCommunicator(host, enforce_trust=True)
		if rc is None:
			raise Exception, 'rc is None'
	except Exception, e:
		luci_log.debug_verbose('CS1: %s: %s' % (host, str(e)))
		return 'Unable to establish a secure connection to the ricci agent on %s: %s' % (host, str(e))

	try:
		if not rc.authed():
			rc.auth(passwd)
	except Exception, e:
		luci_log.debug_verbose('CS2: %s: %s' % (host, str(e)))
		return 'Unable to communicate with the ricci agent on %s for authentication' % host

	try:
		i = rc.authed()
	except Exception, e:
		luci_log.debug_verbose('CS3 %s: %s' % (host, str(e)))
		return 'Unable to authenticate to the ricci agent on %s' % host

	if i != True:
		return 'Authentication for storage system %s failed' % host

	try:
		dummy = self.restrictedTraverse(str(STORAGE_FOLDER_PATH + host)).objectItems()
		luci_log.debug_verbose('CS4 %s already exists' % host)
		return 'Storage system %s is already managed' % host
	except:
		pass

	try:
		ssystem = self.restrictedTraverse(STORAGE_FOLDER_PATH)
	except Exception, e:
		luci_log.debug_verbose('CS5 %s: %s' % (host, str(e)))
		return 'Unable to create storage system %s: %s' % host

	try:
		ssystem.manage_addFolder(host, '__luci__:system')
		newSystem = self.restrictedTraverse(STORAGE_FOLDER_PATH + host)
	except Exception, e:
		luci_log.debug_verbose('CS6 %s: %s' % (host, str(e)))
		return 'Unable to create DB entry for storage system %s' % host

	try:
		newSystem.manage_acquiredPermissions([])
		newSystem.manage_role('View', ['Access contents information', 'View'])
	except Exception, e:
		luci_log.debug_verbose('CS7 %s: %s' % (host, str(e)))
		return 'Unable to set permissions on storage system %s' % host

	return None

def abortManageCluster(self, request):
	pass

def manageCluster(self, clusterName, node_list, cluster_os):
	clusterName = str(clusterName)

	try:
		clusters = self.restrictedTraverse(CLUSTER_FOLDER_PATH)
		if not clusters:
			raise Exception, 'cannot find the cluster entry in the DB'
	except Exception, e:
		luci_log.debug_verbose('MC0: %s: %s' % (clusterName, str(e)))
		return 'Unable to create cluster %s: the cluster directory is missing.' % clusterName

	try:
		newCluster = self.restrictedTraverse(CLUSTER_FOLDER_PATH + clusterName)
		if newCluster:
			luci_log.debug_verbose('MC1: cluster %s: already exists' % clusterName)
			return 'A cluster named %s is already managed by Luci' % clusterName
	except:
		pass

	try:
		clusters.manage_addFolder(clusterName, '__luci__:cluster')
		newCluster = self.restrictedTraverse(CLUSTER_FOLDER_PATH + clusterName)
		if not newCluster:
			raise Exception, 'unable to create the cluster DB entry for %s' % clusterName
	except Exception, e:
		luci_log.debug_verbose('MC2: %s: %s' % (clusterName, str(e)))
		return 'Unable to create cluster %s: %s' % (clusterName, str(e))

	try:
		newCluster.manage_acquiredPermissions([])
		newCluster.manage_role('View', ['Access Contents Information', 'View'])
	except Exception, e:
		luci_log.debug_verbose('MC3: %s: %s' % (clusterName, str(e)))
		try:
			clusters.manage_delObjects([clusterName])
		except Exception, e:
			luci_log.debug_verbose('MC4: %s: %s' % (clusterName, str(e)))
		return 'Unable to set permissions on new cluster: %s: %s' % (clusterName, str(e))

	try:
		newCluster.manage_addProperty('cluster_os', cluster_os, 'string')
	except Exception, e:
		luci_log.debug_verbose('MC5: %s: %s: %s' \
			% (clusterName, cluster_os, str(e)))

	for i in node_list:
		host = node_list[i]['host']

		try:
			newCluster.manage_addFolder(host, '__luci__:csystem:' + clusterName)
			newSystem = self.restrictedTraverse(str(CLUSTER_FOLDER_PATH + clusterName + '/' + host))
			if not newSystem:
				raise Exception, 'unable to create cluster system DB entry for node %s' % host
			newSystem.manage_acquiredPermissions([])
			newSystem.manage_role('View', [ 'Access contents information' , 'View' ])
		except Exception, e:
			try:
				clusters.manage_delObjects([clusterName])
			except Exception, e:
				luci_log.debug_verbose('MC6: %s: %s: %s' \
					% (clusterName, host, str(e)))

			luci_log.debug_verbose('MC7: %s: %s: %s' \
				% (clusterName, host, str(e)))
			return 'Unable to create cluster node %s for cluster %s: %s' \
				% (host, clusterName, str(e))

	try:
		ssystem = self.restrictedTraverse(STORAGE_FOLDER_PATH)
		if not ssystem:
			raise Exception, 'The storage DB entry is missing'
	except Exception, e:
		luci_log.debug_verbose('MC8: %s: %s: %s' % (clusterName, host, str(e)))
		return 'Error adding storage node %s: %s' % (host, str(e))

	# Only add storage systems if the cluster and cluster node DB
	# objects were added successfully.
	for i in node_list:
		host = node_list[i]['host']

		try:
			# It's already there, as a storage system, no problem.
			dummy = self.restrictedTraverse(str(STORAGE_FOLDER_PATH + host)).objectItems()
			continue
		except:
			pass

		try:
			ssystem.manage_addFolder(host, '__luci__:system')
			newSystem = self.restrictedTraverse(STORAGE_FOLDER_PATH + host)
			newSystem.manage_acquiredPermissions([])
			newSystem.manage_role('View', [ 'Access contents information' , 'View' ])
		except Exception, e:
			luci_log.debug_verbose('MC9: %s: %s: %s' % (clusterName, host, str(e)))

def createClusterSystems(self, clusterName, node_list):
	try:
		clusterObj = self.restrictedTraverse(CLUSTER_FOLDER_PATH + clusterName)
		if not clusterObj:
			raise Exception, 'cluster %s DB entry is missing' % clusterName
	except Exception, e:
		luci_log.debug_verbose('CCS0: %s: %s' % (clusterName, str(e)))
		return 'No cluster named \"%s\" is managed by Luci' % clusterName

	for x in node_list:
		i = node_list[x]
		host = str(i['host'])

		try:
			clusterObj.manage_addFolder(host, '__luci__:csystem:' + clusterName)
		except Exception, e:
			luci_log.debug_verbose('CCS0a: %s: %s: %s' % (clusterName, host, str(e)))
		try:
			newSystem = self.restrictedTraverse(CLUSTER_FOLDER_PATH + clusterName + '/' + host)
			if not newSystem:
				raise Exception, 'cluster node DB entry for %s disappeared from under us' % host
					
			newSystem.manage_acquiredPermissions([])
			newSystem.manage_role('View', [ 'Access contents information' , 'View' ])
		except Exception, e:
			luci_log.debug_verbose('CCS1: %s: %s: %s' % (clusterName, host, str(e)))
			return 'Unable to create cluster node %s for cluster %s: %s' \
				% (host, clusterName, str(e))

	try:
		ssystem = self.restrictedTraverse(STORAGE_FOLDER_PATH)
		if not ssystem:
			raise Exception, 'storage DB entry is missing'
	except Exception, e:
		# This shouldn't fail, but if it does, it's harmless right now
		luci_log.debug_verbose('CCS2: %s: %s' % (clusterName, host, str(e)))
		return None

	# Only add storage systems if the and cluster node DB
	# objects were added successfully.
	for x in node_list:
		i = node_list[x]
		host = str(i['host'])

		try:
			# It's already there, as a storage system, no problem.
			dummy = self.restrictedTraverse(str(STORAGE_FOLDER_PATH + host)).objectItems()
			continue
		except:
			pass

		try:
			ssystem.manage_addFolder(host, '__luci__:system')
			newSystem = self.restrictedTraverse(STORAGE_FOLDER_PATH + host)
			newSystem.manage_acquiredPermissions([])
			newSystem.manage_role('View', [ 'Access contents information' , 'View' ])
		except Exception, e:
			luci_log.debug_verbose('CCS3: %s: %s' % (clusterName, host, str(e)))

def delSystem(self, systemName):
	try:
		ssystem = self.restrictedTraverse(STORAGE_FOLDER_PATH)
		if not ssystem:
			raise Exception, 'storage DB entry is missing'
	except Exception, e:
		luci_log.debug_verbose('delSystem0: %s: %s' % (systemName, str(e)))
		return 'Unable to find storage system %s: %s' % (systemName, str(e))

	try:
		rc = RicciCommunicator(systemName, enforce_trust=False)
		if rc is None:
			raise Exception, 'rc is None'
	except Exception, e:
		try:
			ssystem.manage_delObjects([ systemName ])
		except Exception, e:
			luci_log.debug_verbose('delSystem1: %s: %s' % (systemName, str(e)))
			return 'Unable to delete the storage system %s' % systemName
		luci_log.debug_verbose('delSystem2: %s: %s' % (systemName, str(e)))
		return

	# Only unauthenticate if the system isn't a member of
	# a managed cluster.
	cluster_info = rc.cluster_info()
	if not cluster_info:
		cluster_name = None
	elif not cluster_info[0]:
		cluster_name = cluster_info[1]
	else:
		cluster_name = cluster_info[0]

	unauth = False
	if not cluster_name:
		# If it's a member of no cluster, unauthenticate
		unauth = True
	else:
		try:
			dummy = self.restrictedTraverse(str(CLUSTER_FOLDER_PATH + cluster_name + '/' + systemName)).objectItems()
		except Exception, e:
			# It's not a member of a managed cluster, so unauthenticate.
			unauth = True

	if unauth is True:
		try:
			rc.unauth()
		except:
			pass

	try:
		ssystem.manage_delObjects([ systemName ])
	except Exception, e:
		luci_log.debug_verbose('delSystem3: %s: %s' % (systemName, str(e)))
		return 'Unable to delete storage system %s: %s' \
			% (systemName, str(e))

def delCluster(self, clusterName):
	try:
		clusters = self.restrictedTraverse(CLUSTER_FOLDER_PATH)
		if not clusters:
			raise Exception, 'clusters DB entry is missing'
	except Exception, e:
		luci_log.debug_verbose('delCluster0: %s' % str(e))
		return 'Unable to find cluster %s' % clusterName

	err = delClusterSystems(self, clusterName)
	if err:
		return err

	try:
		clusters.manage_delObjects([ clusterName ])
	except Exception, e:
		luci_log.debug_verbose('delCluster1: %s' % str(e))
		return 'Unable to delete cluster %s' % clusterName

def delClusterSystem(self, cluster, systemName):
	try:
		dummy = self.restrictedTraverse(str(STORAGE_FOLDER_PATH + systemName)).objectItems()
	except:
		# It's not a storage system, so unauthenticate.
		try:
			rc = RicciCommunicator(systemName, enforce_trust=False)
			rc.unauth()
		except Exception, e:
			luci_log.debug_verbose('delClusterSystem0: ricci error for %s: %s' \
				% (systemName, str(e)))

	try:
		cluster.manage_delObjects([ systemName ])
	except Exception, e:
		err_str = 'Error deleting cluster object %s: %s' % (systemName, str(e))
		luci_log.debug_verbose('delClusterSystem1: %s' % err_str)
		return err_str

def delClusterSystems(self, clusterName):
	try:
		cluster = self.restrictedTraverse(CLUSTER_FOLDER_PATH + clusterName)
		if not cluster:
			raise Exception, 'cluster DB entry is missing'

		try:
			csystems = getClusterSystems(self, clusterName)
			if not csystems or len(csystems) < 1:
				return None
		except Exception, e:
			luci_log.debug_verbose('delCluSystems0: %s' % str(e))
			return None
	except Exception, er:
		luci_log.debug_verbose('delCluSystems1: error for %s: %s' \
			% (clusterName, str(er)))
		return str(er)

	errors = ''
	for i in csystems:
		err = delClusterSystem(self, cluster, i[0])
		if err:
			errors += 'Unable to delete the cluster system %s: %s\n' % (i[0], err)
			luci_log.debug_verbose('delCluSystems2: %s' % err)
	return errors

def getDefaultUser(self, request):
	try:
		user = request.form['userList']
	except KeyError, e:
		try:
			user = request['user']
		except:
			try:
				members = list()
				members.extend(self.portal_membership.listMembers())
				members.sort()
				user = members[0].getUserName()
			except Exception, e:
				luci_log.debug_verbose('getDefaultUser0: %s' % str(e))
				user = None

	if not user:
		luci_log.debug_verbose('getDefaultUser1: user is none')
	return user

def getUserPerms(self):
	perms = {}

	try:
		members = list()
		members.extend(self.portal_membership.listMembers())
		if len(members) < 1:
			raise Exception, 'no portal members exist'
		members.sort()
	except Exception, e:
		luci_log.debug_verbose('getUserPerms0: %s' % str(e))
		return {}

	for i in members:
		userName = i.getUserName()

		perms[userName] = {}
		perms[userName]['cluster'] = {}
		perms[userName]['storage'] = {}

		try:
			clusters = self.restrictedTraverse(PLONE_ROOT + '/systems/cluster/objectItems')('Folder')
			storage = self.restrictedTraverse(PLONE_ROOT + '/systems/storage/objectItems')('Folder')
		except Exception, e:
			luci_log.debug_verbose('getUserPerms1: user %s: %s' % (userName, str(e)))
			continue

		for c in clusters:
			try:
				perms[userName]['cluster'][c[0]] = i.has_role('View', c[1])
			except Exception, e:
				luci_log.debug_verbose('getUserPerms2: user %s, obj %s: %s' \
					% (userName, c[0], str(e)))
				continue
				
		for s in storage:
			try:
				perms[userName]['storage'][s[0]] = i.has_role('View', s[1])
			except Exception, e:
				luci_log.debug_verbose('getUserPerms2: user %s, obj %s: %s' \
					% (userName, s[0], str(e)))
				continue

	return perms

# In case we want to give access to non-admin users in the future

def havePermCreateCluster(self):
	return isAdmin(self)

def havePermAddStorage(self):
	return isAdmin(self)

def havePermAddCluster(self):
	return isAdmin(self)

def havePermAddUser(self):
	return isAdmin(self)

def havePermDelUser(self):
	return isAdmin(self)

def havePermRemStorage(self):
	return isAdmin(self) 

def havePermRemCluster(self):
	return isAdmin(self) 

def havePermEditPerms(self):
	return isAdmin(self) 

def getClusterConfNodes(clusterConfDom):
	cur = clusterConfDom
	clusterNodes = list()

	for i in cur.childNodes:
		cur = i
		if i.nodeName == 'clusternodes':
			for i in cur.childNodes:
				if i.nodeName == 'clusternode':
					clusterNodes.append(i.getAttribute('name'))
			return clusterNodes
	return clusterNodes

def getSystems(self):
	storage = getStorage(self)
	clusters = getClusters(self)
	storageList = list()
	ret = [{}, [], {}]

	need_auth_hash = {}
	for i in storage:
		storageList.append(i[0])
		if testNodeFlag(i[1], CLUSTER_NODE_NEED_AUTH) != False:
			need_auth_hash[i[0]] = i[1]

	chash = {}
	for i in clusters:
		csystems = getClusterSystems(self, i[0])
		cslist = list()
		for c in csystems:
			if testNodeFlag(c[1], CLUSTER_NODE_NEED_AUTH) != False:
				need_auth_hash[c[0]] = c[1]
			cslist.append(c[0])
		chash[i[0]] = cslist

	ret[0] = chash
	ret[1] = storageList
	ret[2] = need_auth_hash
	return ret

def getClusterNode(self, nodename, clustername):
	try:
		cluster_node = self.restrictedTraverse(CLUSTER_FOLDER_PATH + str(clustername) + '/' + str(nodename))
		if not cluster_node:
			raise Exception, 'cluster node is none'
		return cluster_node
	except Exception, e:
		luci_log.debug_verbose('getClusterNode0: %s %s: %s' \
			% (nodename, clustername, str(e)))
		return None

def getStorageNode(self, nodename):
	try:
		storage_node = self.restrictedTraverse(STORAGE_FOLDER_PATH + str(nodename))
		if not storage_node:
			raise Exception, 'storage node is none'
		return storage_node
	except Exception, e:
		luci_log.debug_verbose('getStorageNode0: %s: %s' % (nodename, str(e)))
		return None

def testNodeFlag(node, flag_mask):
	try:
		flags = node.getProperty('flags')
		if flags is None:
			return False
		return flags & flag_mask != 0
	except Exception, e:
		luci_log.debug_verbose('testNodeFlag0: %s' % str(e))
	return False

def setNodeFlag(node, flag_mask):
	try:
		flags = node.getProperty('flags')
		if flags is None:
			flags = 0
		node.manage_changeProperties({ 'flags': flags | flag_mask })
	except:
		try:
			node.manage_addProperty('flags', flag_mask, 'int')
		except Exception, e:
			luci_log.debug_verbose('setNodeFlag0: %s' % str(e))

def delNodeFlag(node, flag_mask):
	try:
		flags = node.getProperty('flags')
		if flags is None:
			return
		if flags & flag_mask != 0:
			node.manage_changeProperties({ 'flags': flags & ~flag_mask })
	except Exception, e:
		luci_log.debug_verbose('delNodeFlag0: %s' % str(e))
