import socket
from ModelBuilder import ModelBuilder
from xml.dom import minidom
import AccessControl
from conga_constants import *
from ricci_bridge import *
from ricci_communicator import RicciCommunicator, RicciError, batch_status, extract_module_status
import time
import Products.ManagedSystem
from Products.Archetypes.utils import make_uuid
from Ip import Ip
from Clusterfs import Clusterfs
from Fs import Fs
from FailoverDomain import FailoverDomain
from FailoverDomainNode import FailoverDomainNode
from RefObject import RefObject
from ClusterNode import ClusterNode
from NFSClient import NFSClient
from NFSExport import NFSExport
from Service import Service
from Lockserver import Lockserver
from Netfs import Netfs
from Apache import Apache
from MySQL import MySQL
from Postgres8 import Postgres8
from Tomcat5 import Tomcat5
from OpenLDAP import OpenLDAP
from Vm import Vm
from FenceXVMd import FenceXVMd
from Script import Script
from Samba import Samba
from LVM import LVM
from QuorumD import QuorumD
from Heuristic import Heuristic
from clusterOS import resolveOSType
from Fence import Fence
from Method import Method
from Totem import Totem
from Device import Device
from FenceHandler import validateNewFenceDevice, FENCE_OPTS, validateFenceDevice, validate_fenceinstance
from GeneralError import GeneralError
from homebase_adapters import manageCluster, createClusterSystems, havePermCreateCluster, setNodeFlag, delNodeFlag, userAuthenticated, getStorageNode, getClusterNode, delCluster, parseHostForm
from LuciSyslog import LuciSyslog

#Policy for showing the cluster chooser menu:
#1) If there are no clusters in the ManagedClusterSystems
#folder, then only the admin user may see this menu, and
#the configure option should not be displayed.
#2)If there are clusters in the ManagedClusterSystems,
#then only display chooser if the current user has
#permissions on at least one. If the user is admin, show ALL clusters

try:
	luci_log = LuciSyslog()
except:
	pass

def get_fsid_list(model):
	obj_list = model.searchObjectTree('fs')
	obj_list.extend(model.searchObjectTree('clusterfs'))
	return map(lambda x: x.getAttribute('fsid') and int(x.getAttribute('fsid')) or 0, obj_list)

def fsid_is_unique(model, fsid):
	fsid_list = get_fsid_list(model)
	return fsid not in fsid_list

def generate_fsid(model, name):
	import binascii
	from random import random
	fsid_list = get_fsid_list(model)

	fsid = binascii.crc32(name) & 0xffff
	dupe = fsid in fsid_list
	while dupe is True:
		fsid = (fsid + random.randrange(1, 0xfffe)) & 0xffff
		dupe = fsid in fsid_list
	return fsid

def buildClusterCreateFlags(self, batch_map, clusterName):
	path = str(CLUSTER_FOLDER_PATH + clusterName)

	try:
		clusterfolder = self.restrictedTraverse(path)
	except Exception, e:
		luci_log.debug_verbose('buildCCF0: no cluster folder at %s' % path)
		return None

	for key in batch_map.keys():
		try:
			key = str(key)
			batch_id = str(batch_map[key])
			#This suffix needed to avoid name collision
			objname = str(key + "____flag")

			clusterfolder.manage_addProduct['ManagedSystem'].addManagedSystem(objname)
			#now designate this new object properly
			objpath = str(path + "/" + objname)
			flag = self.restrictedTraverse(objpath)

			flag.manage_addProperty(BATCH_ID, batch_id, "string")
			flag.manage_addProperty(TASKTYPE, CLUSTER_ADD, "string")
			flag.manage_addProperty(FLAG_DESC, "Creating node " + key + " for cluster " + clusterName, "string")
			flag.manage_addProperty(LAST_STATUS, 0, "int")
		except Exception, e:
			luci_log.debug_verbose('buildCCF1: error creating flag for %s: %s' \
				% (key, str(e)))

def parseClusterNodes(self, request, cluster_os):
	check_certs = False
	try:
		check_certs = 'check_certs' in request.form
	except:
		check_certs = False

	download_pkgs = 1
	try:
		download_pkgs = int(request.form['download_pkgs'].strip())
	except:
		download_pkgs = 1

	clusterName = None
	try:
		clusterName = str(request.form['clusterName'])
	except Exception, e:
		luci_log.debug_verbose('PNC00: missing cluster name: %s' % str(e))
		clusterName = None

	if clusterName is None:
		luci_log.debug_verbose('PCN0: no cluster name was given')
		return (False, { 'errors': [ 'No cluster name was given.' ]})

	shared_storage = False
	try:
		shared_storage = request.form.has_key('enable_storage')
	except:
		shared_storage = False

	same_node_passwds = False
	try:
		same_node_passwds = 'allSameCheckBox' in request.form
	except:
		same_node_passwds = False

	add_cluster = { 'name': clusterName,
					'shared_storage': shared_storage,
					'download_pkgs': download_pkgs,
					'cluster_os': cluster_os,
					'identical_passwds': same_node_passwds,
					'check_certs': check_certs }

	system_list, incomplete, errors, messages = parseHostForm(request, check_certs)
	add_cluster['nodes'] = system_list

	for i in system_list:
		cur_system = system_list[i]

		cur_host_trusted = 'trusted' in cur_system
		cur_host = cur_system['host']

		try:
			cur_passwd = cur_system['passwd']
		except:
			cur_passwd = None

		if (cur_host_trusted or not check_certs) and cur_passwd:
			try:
				rc = RicciCommunicator(cur_host, enforce_trust=True)
				if not rc:
					raise Exception, 'connection failed'
			except Exception, e:
				cur_system['errors'] = True
				incomplete = True
				errors.append('Unable to connect to %s: %s' \
					% (cur_host, str(e)))
				luci_log.debug_verbose('PCN1: %s: %s' % (cur_host, str(e)))
				continue

			prev_auth = rc.authed()
			cur_system['prev_auth'] = prev_auth

			try:
				if prev_auth:
					messages.append('Host %s is already authenticated.' \
						% cur_host)
				else:
					rc.auth(cur_passwd)

				if not rc.authed():
					raise Exception, 'authentication failed'
			except Exception, e:
				cur_system['errors'] = True
				incomplete = True
				errors.append('Error authenticating to %s: %s' \
					% (cur_host, str(e)))
				luci_log.debug_verbose('PCN2: %s: %s' % (cur_host, str(e)))
				continue

			cur_cluster_info = rc.cluster_info()
			if cur_cluster_info[0] or cur_cluster_info[1]:
				cur_system['errors'] = True
				incomplete = True

				if cur_cluster_info[0]:
					cur_cluster_name = cur_cluster_info[0]
				elif cur_cluster_info[1]:
					cur_cluster_name = cur_cluster_info[1]

				try:
					if not cur_system['prev_auth']:
						rc.unauth()
						del cur_system['trusted']
				except Exception, e:
					luci_log.debug_verbose('PCN3: %s: %s' % (cur_host, str(e)))

				errors.append('%s reports it is a member of cluster \"%s\"' \
					% (cur_host, cur_cluster_name))
				luci_log.debug_verbose('PCN4: %s: already in %s cluster' \
					% (cur_host, cur_cluster_name))
				continue

			cur_host_os = resolveOSType(rc.os())
			if cluster_os is None:
				cluster_os = cur_host_os
				add_cluster['cluster_os'] = cur_host_os

			elif cluster_os != cur_host_os:
				cur_system['errors'] = True
				incomplete = True

				try:
					if not cur_system['prev_auth']:
						rc.unauth()
						del cur_system['trusted']
				except Exception, e:
					luci_log.debug_verbose('PCN5: %s: %s' % (cur_host, str(e)))

				errors.append('The cluster software version on %s (%s) does not match the software on the other cluster nodes (%s)' % (cur_host, cur_host_os, cluster_os))
				luci_log.debug_verbose('PCN6: version mismatch for %s: (%s vs. %s)' \
					% (cur_host, cur_host_os, cluster_os))
				continue

	return add_cluster, incomplete, errors, messages

def validateCreateCluster(self, request):
	try:
		request.SESSION.delete('create_cluster')
	except:
		pass

	cluster_os = None
	try:
		cluster_os = request.form['cluster_os'].strip()
		if not cluster_os:
			raise Exception, 'cluster OS is blank'
	except:
		cluster_os = None

	add_cluster, incomplete, errors, messages = parseClusterNodes(self, request, cluster_os)
	clusterName = add_cluster['name']

	if len(clusterName) > 15:
		errors.append('A cluster\'s name must be less than 16 characters long.')

	try:
		cluster_os = add_cluster['cluster_os']
	except:
		pass

	lockmanager = 'dlm'
	if cluster_os == 'rhel4':
		add_cluster['gulm_support'] = True
		if not request.form.has_key('lockmanager'):
			# The user hasn't been presented with the RHEL4
			# lock manager options yet.
			incomplete = True
		else:
			try:
				lockmanager = request.form['lockmanager'].strip()
			except:
				lockmanager = 'dlm'

	lockservers = None
	if lockmanager == 'gulm':
		add_cluster['lockmanager'] = 'gulm'
		try:
			lockservers = filter(lambda x: x.strip(), request.form['__GULM__'])
			if not lockservers or len(lockservers) < 1:
				raise Exception, 'blank'
			num_lockservers = len(lockservers)
			if not num_lockservers in (1, 3, 5):
				errors.append('You must have exactly 1, 3, or 5 GULM lock servers. You submitted %d lock servers.' % num_lockservers)
		except:
			errors.append('No lock servers were given.')

		if len(errors) > 0:
			try:
				ls_hash = {}
				for i in xrange(num_lockservers):
					ls_hash['server%d' % (i + 1)] = lockservers[i]
				add_cluster['gulm_lockservers'] = ls_hash
			except:
				pass

	if incomplete or len(errors) > 0:
		request.SESSION.set('create_cluster', add_cluster)
		return (False, { 'errors': errors, 'messages': messages })

	node_list = add_cluster['nodes'].keys()
	batchNode = createClusterBatch(add_cluster['cluster_os'],
					clusterName,
					clusterName,
					node_list,
					True,
					True,
					add_cluster['shared_storage'],
					False,
					add_cluster['download_pkgs'],
					lockservers)

	if not batchNode:
		request.SESSION.set('create_cluster', add_cluster)
		errors.append('Unable to generate cluster creation ricci command')
		return (False, { 'errors': errors, 'messages': messages })

	error = manageCluster(self, clusterName, add_cluster['nodes'], add_cluster['cluster_os'])
	if error:
		errors.append('Unable to create the cluster Luci database objects')
		request.SESSION.set('create_cluster', add_cluster)
		return (False, { 'errors': errors, 'messages': messages })

	batch_id_map = {}
	for i in node_list:
		try:
			rc = RicciCommunicator(i)
			if not rc:
				raise Exception, 'rc is None'
		except Exception, e:
			msg = 'Unable to connect to the ricci agent on %s: %s' % (i, str(e))
			errors.append(msg)
			luci_log.debug_verbose(msg)
			if len(batch_id_map) == 0:
				request.SESSION.set('create_cluster', add_cluster)
				return (False, { 'errors': errors, 'messages': messages })
			continue

		try:
			resultNode = rc.process_batch(batchNode, async=True)
			batch_id_map[i] = resultNode.getAttribute('batch_id')
		except Exception, e:
			luci_log.debug_verbose('validateCreateCluster0: %s: %s' \
				% (i, str(e)))
			errors.append('An error occurred while attempting to add cluster node \"%s\"' % i)
			if len(batch_id_map) == 0:
				request.SESSION.set('create_cluster', add_cluster)
				return (False, { 'errors': errors, 'messages': messages })
			continue

	buildClusterCreateFlags(self, batch_id_map, clusterName)
	response = request.RESPONSE
	response.redirect(request['URL'] + "?pagetype=" + CLUSTER_CONFIG + "&clustername=" + clusterName + '&busyfirst=true')

def validateAddClusterNode(self, request):
	try:
		request.SESSION.delete('add_node')
	except:
		pass

	check_certs = False
	try:
		check_certs = 'check_certs' in request.form
	except:
		check_certs = False

	download_pkgs = 1
	try:
		download_pkgs = int(request.form['download_pkgs'].strip())
	except:
		download_pkgs = 1

	cluster_os = None
	try:
		cluster_os = request.form['cluster_os'].strip()
		if not cluster_os:
			raise Exception, 'cluster OS is blank'
	except:
		cluster_os = None

	clusterName = None
	try:
		clusterName = str(request.form['clusterName'])
	except:
		clusterName = None

	if clusterName is None:
		luci_log.debug_verbose('VACN0: no cluster name was given')
		return (False, { 'errors': [ 'No cluster name was given.' ]})

	if cluster_os is None:
		cluster_folder = None
		try:
			cluster_folder = self.restrictedTraverse(str(CLUSTER_FOLDER_PATH + clusterName))
			if not cluster_folder:
				raise Exception, 'cluster DB object is missing'
		except Exception, e:
			luci_log.debug_verbose('VACN1: %s: %s' % (clusterName, str(e)))
			return (False, { 'errors': [ 'The database object for %s is missing.' % clusterName ] })

		try:
			cluster_os = cluster_folder.manage_getProperty('cluster_os')
			if not cluster_os:
				raise Exception, 'cluster os is blank'
		except Exception, e:
			luci_log.debug_verbose('VACN2: %s: %s' % (clusterName, str(e)))
			cluster_os = None

		if cluster_os is None:
			try:
				cluster_ricci = getRicciAgent(self, clusterName)
				cluster_os = resolveOSType(cluster_ricci.os())
			except Exception, e:
				luci_log.debug_verbose('VACN3: %s: %s' % (clusterName, str(e)))
				cluster_os = None

	if cluster_os is None:
		luci_log.debug_verbose('Unable to determine cluster OS for %s' % clusterName)
		return (False, { 'errors': [ 'Unable to determine the version of the cluster suite this cluster is running.' ] })

	shared_storage = False
	try:
		shared_storage = request.form.has_key('enable_storage')
	except:
		shared_storage = False

	same_node_passwds = False
	try:
		same_node_passwds = 'allSameCheckBox' in request.form
	except:
		same_node_passwds = False

	add_cluster = { 'name': clusterName,
					'shared_storage': shared_storage,
					'download_pkgs': download_pkgs,
					'cluster_os': cluster_os,
					'identical_passwds': same_node_passwds,
					'check_certs': check_certs }

	system_list, incomplete, errors, messages = parseHostForm(request, check_certs)
	add_cluster['nodes'] = system_list

	for i in system_list:
		cur_system = system_list[i]

		cur_host_trusted = 'trusted' in cur_system
		cur_host = cur_system['host']

		try:
			cur_passwd = cur_system['passwd']
		except:
			cur_passwd = None

		if (cur_host_trusted or not check_certs) and cur_passwd:
			try:
				rc = RicciCommunicator(cur_host, enforce_trust=True)
				if not rc:
					raise Exception, 'connection failed'
			except Exception, e:
				cur_system['errors'] = True
				incomplete = True
				errors.append('Unable to connect to %s: %s' \
					% (cur_host, str(e)))
				luci_log.debug_verbose('VACN4: %s: %s' % (cur_host, str(e)))
				continue

			prev_auth = rc.authed()
			cur_system['prev_auth'] = prev_auth

			try:
				if prev_auth:
					messages.append('Host %s is already authenticated.' \
						% cur_host)
				else:
					rc.auth(cur_passwd)

				if not rc.authed():
					raise Exception, 'authentication failed'
			except Exception, e:
				cur_system['errors'] = True
				incomplete = True
				errors.append('Error authenticating to %s: %s' \
					% (cur_host, str(e)))
				luci_log.debug_verbose('VACN5: %s: %s' % (cur_host, str(e)))
				continue

			cur_cluster_info = rc.cluster_info()
			if cur_cluster_info[0] or cur_cluster_info[1]:
				cur_system['errors'] = True
				incomplete = True

				if cur_cluster_info[0]:
					cur_cluster_name = cur_cluster_info[0]
				elif cur_cluster_info[1]:
					cur_cluster_name = cur_cluster_info[1]

				try:
					if not cur_system['prev_auth']:
						rc.unauth()
						del cur_system['trusted']
				except Exception, e:
					luci_log.debug_verbose('VACN6: %s: %s' % (cur_host, str(e)))

				errors.append('%s reports it is already a member of cluster \"%s\"' % (cur_host, cur_cluster_name))
				luci_log.debug_verbose('VACN7: %s: already in %s cluster' \
					% (cur_host, cur_cluster_name))
				continue

			cur_host_os = resolveOSType(rc.os())
			if cluster_os is not None and cluster_os != cur_host_os:
				cur_system['errors'] = True
				incomplete = True

				try:
					if not cur_system['prev_auth']:
						rc.unauth()
						del cur_system['trusted']
				except Exception, e:
					luci_log.debug_verbose('VACN8: %s: %s' % (cur_host, str(e)))

				errors.append('The cluster software version on %s (%s) does not match the software on the other cluster nodes (%s)' % (cur_host, cur_host_os, cluster_os))
				luci_log.debug_verbose('VACN9: version mismatch for %s: (%s vs. %s)' \
					% (cur_host, cur_host_os, cluster_os))
				continue

	if incomplete or len(errors) > 0:
		request.SESSION.set('add_node', add_cluster)
		return (False, { 'errors': errors, 'messages': messages })

	try:
		cluster_ricci = getRicciAgent(self, clusterName)
		if not cluster_ricci:
			raise Exception, 'Unable to find a ricci agent for %s' % clusterName
	except Exception, e:
		incomplete = True
		errors.append('Unable to contact a ricci agent for %s.' % clusterName)
		luci_log.debug_verbose('VACN10: %s: %s' % (clusterName, str(e)))

	if incomplete or len(errors) > 0:
		request.SESSION.set('add_node', add_cluster)
		return (False, { 'errors': errors, 'messages': messages })

	try:
		model = getModelBuilder(None, cluster_ricci, cluster_ricci.dom0())
		if not model:
			errors.append('Unable to build the cluster model for %s' \
				% clusterName)
			raise Exception, 'unable to get model for %s' % clusterName

		nodesptr = model.getClusterNodesPtr()
		used_ids = {}
		for i in model.getNodes():
			used_ids[int(i.getAttribute('nodeid'))] = 1
			node_name = str(i.getAttribute('name'))
			if node_name in system_list:
				system_list[node_name]['errors'] = True
				errors.append('%s is already a member of %s' \
					% (node_name, clusterName))
	except Exception, e:
		incomplete = True
		errors.append('Unable to build the cluster model for %s' \
			% clusterName)
		luci_log.debug_verbose('VACN11: %s' % str(e))

	if incomplete or len(errors) > 0:
		request.SESSION.set('add_node', add_cluster)
		return (False, { 'errors': errors, 'messages': messages })

	next_node_id = 1

	try:
		for x in system_list:
			i = system_list[x]

			try:
				batch_node = addClusterNodeBatch(cluster_os,
								clusterName,
								True,
								True,
								shared_storage,
								False,
								download_pkgs,
								model.GULM_ptr is not None)
				if not batch_node:
					raise Exception, 'batch is blank'
				system_list[x]['batch'] = batch_node
			except Exception, e:
				cur_system['errors'] = True
				incomplete = True

				try:
					if not cur_system['prev_auth']:
						rc.unauth()
						del cur_system['trusted']
				except Exception, e:
					luci_log.debug_verbose('VACN12: %s: %s' % (cur_host, str(e)))

				errors.append('Unable to initiate cluster join for %s' % cur_host)
				luci_log.debug_verbose('VACN13: %s: %s' % (cur_host, str(e)))
				continue

			next_node_id += 1
			new_node = ClusterNode()
			new_node.attr_hash['name'] = str(i['host'])
			new_node.attr_hash['votes'] = str(1)
			while next_node_id in used_ids:
				next_node_id += 1
			new_node.attr_hash['nodeid'] = str(next_node_id)
			nodesptr.addChild(new_node)

		if incomplete or len(errors) > 0:
			request.SESSION.set('add_node', add_cluster)
			return (False, { 'errors': errors, 'messages': messages })

		model.setModified(True)
		conf_str = str(model.exportModelAsString())
		if not conf_str:
			raise Exception, 'Unable to save the new cluster model.'

		batch_number, result = setClusterConf(cluster_ricci, conf_str)
		if not batch_number or not result:
			raise Exception, 'batch or result is None'
	except Exception, e:
		incomplete = True
		errors.append('Unable to save the new cluster model.')
		luci_log.debug_verbose('VACN14: %s' % str(e))

	# Propagate the new cluster.conf to the existing nodes
	# before having any of the new nodes join. If this fails,
	# abort the whole process.
	try:
		while True:
			batch_ret = checkBatch(cluster_ricci, batch_number)
			code = batch_ret[0]
			if code == True:
				break
			if code == -1:
				errors.append(batch_ret[1])
				raise Exception, str(batch_ret[1])
			if code == False:
				time.sleep(0.5)
	except Exception, e:
		incomplete = True
		errors.append('Unable to update the cluster node list for %s' \
			% clusterName)
		luci_log.debug_verbose('VACN15: %s' % str(e))

	if incomplete or len(errors) > 0:
		request.SESSION.set('add_node', add_cluster)
		return (False, { 'errors': errors, 'messages': messages })

	error = createClusterSystems(self, clusterName, system_list)
	if error:
		incomplete = True
		errors.append(error)
		luci_log.debug_verbose('VACN16: %s: %s' % (clusterName, error))

	if incomplete or len(errors) > 0:
		request.SESSION.set('add_node', add_cluster)
		return (False, { 'errors': errors, 'messages': messages })

	batch_id_map = {}
	for x in system_list:
		clunode = system_list[x]
		success = True

		cur_host = clunode['host']
		try:
			rc = RicciCommunicator(cur_host)
			if not rc:
				raise Exception, 'rc is None'
		except Exception, e:
			success = False
			clunode['errors'] = True
			errors.append('Unable to connect to the ricci agent on %s: %s' \
				% (cur_host, str(e)))
			luci_log.info('VACN17: Unable to connect to the ricci daemon on host %s: %s' % (clunode['host'], str(e)))

		if success:
			try:
				resultNode = rc.process_batch(clunode['batch'], async=True)
				batch_id_map[cur_host] = resultNode.getAttribute('batch_id')
			except Exception, e:
				clunode['errors'] = True
				success = False
				luci_log.debug_verbose('VACN18: %s: %s' \
					% (cur_host, str(e)))

		if not success:
			incomplete = True
			errors.append('An error occurred while attempting to add cluster node \"%s\"' % cur_host)

	if incomplete or len(errors) > 0:
		request.SESSION.set('add_node', add_cluster)
		return (False, { 'errors': errors, 'messages': messages })

	buildClusterCreateFlags(self, batch_id_map, clusterName)

	response = request.RESPONSE
	response.redirect(request['URL'] + "?pagetype=" + CLUSTER_CONFIG + "&clustername=" + clusterName + '&busyfirst=true')

def validateServiceAdd(self, request):
	errors = list()

	try:
		form_xml = request['form_xml']
		if not form_xml:
			raise KeyError, 'form_xml must not be blank'
	except Exception, e:
		luci_log.debug_verbose('vSA0: no form_xml: %s' % str(e))
		return (False, {'errors': ['No resource data was supplied for this service.']})

	try:
		model = request.SESSION.get('model')
		if not model:
			raise Exception, 'model is None'
	except Exception, e:
		luci_log.debug_verbose('vSA0a: %s' % str(e))
		return (False, {'errors': [ 'The cluster model is missing from the session object.' ]})

	try:
		doc = minidom.parseString(form_xml)
		forms = doc.getElementsByTagName('form')
		if len(forms) < 1:
			raise
	except Exception, e:
		luci_log.debug_verbose('vSA1: error: %s' % str(e))
		return (False, {'errors': ['The resource data submitted for this service is not properly formed.']})

	form_hash = {}
	form_hash['toplevel'] = {'form': None, 'kids': [] }
	for i in forms:
		form_id = i.getAttribute('id')
		form_parent = i.getAttribute('parent')
		if not form_id or not form_parent:
			continue
		ielems = i.getElementsByTagName('input')
		if not ielems or len(ielems) < 1:
			continue
		if not form_id in form_hash:
			form_hash[form_id] = {'form': i, 'kids': []}
		elif not form_hash[form_id]['form']:
			form_hash[form_id]['form'] = i
		if not form_parent in form_hash:
			form_hash[form_parent] = {'form': None, 'kids': []}
		form_hash[form_parent]['kids'].append(form_id)
		dummy_form = {}

		for i in ielems:
			try:
				input_type = str(i.getAttribute('type'))
			except:
				continue
			if not input_type or input_type == 'button':
				continue
			try:
				dummy_form[str(i.getAttribute('name'))] = str(i.getAttribute('value'))
			except Exception, e:
				luci_log.debug_verbose('vSA2: parsing XML: %s' % str(e))
				pass

		try:
			res_type = dummy_form['type'].strip()
			if not res_type:
				raise Exception, 'no resource type was given'
			if not res_type in resourceAddHandler:
				raise Exception, 'invalid resource type: %s' % res_type
		except Exception, e:
			luci_log.debug_verbose('vSA3: %s' % str(e))
			return (False, {'errors': [ 'An invalid resource type was specified' ]})

		try:
			if res_type == 'ip':
				dummy_form['resourceName'] = dummy_form['ip_address']
		except Exception, e:
			luci_log.debug_verbose('vSA3a: type is ip but no addr: %s' % str(e))
			return (False, {'errors': [ 'No IP address was given.' ]})

		try:
			if dummy_form.has_key('immutable'):
				newRes = getResource(model, dummy_form['resourceName'])
				resObj = RefObject(newRes)
				resObj.setRef(newRes.getName())
			else:
				resObj = resourceAddHandler[res_type](request, dummy_form)[0]
		except Exception, e:
			resObj = None
			luci_log.debug_verbose('vSA4: type %s: %s' % (res_type, str(e)))

		if resObj is None:
			return (False, {'errors': [ 'An error occurred while adding %s' % res_type ]})
		form_hash[form_id]['obj'] = resObj

	if len(errors) > 0:
		return (False, {'errors': errors})

	fdom = None
	try:
		fdom = request.form['domain'].strip()
		if not fdom:
			raise Exception, 'blank'
	except:
		fdom = None

	recovery = None
	try:
		recovery = request.form['recovery'].strip()
		if not recovery:
			recovery = None
		else:
			if recovery != 'restart' and recovery != 'relocate' and recovery != 'disable':
				errors.append('You entered an invalid recovery option: \"%s\" Valid options are \"restart\" \"relocate\" and \"disable\"')
	except:
		recovery = None

	try:
		service_name = request.form['svc_name'].strip()
	except Exception, e:
		luci_log.debug_verbose('vSA5: no service name: %s' % str(e))
		return (False, {'errors': [ 'No service name was given.' ]})

	autostart = "1"
	try:
		if not request.form.has_key('autostart') or request.form['autostart'] == "0":
			autostart = "0"
	except Exception, e:
		autostart = None
		luci_log.debug_verbose('vSA5a: error getting autostart: %s' % str(e))

	exclusive = "0"
	try:
		if not request.form.has_key('exclusive') or request.form['exclusive'] != "1":
			exclusive = "0"
		else:
			exclusive = "1"
	except Exception, e:
		exclusive = "0"

	try:
		cur_service = model.retrieveServiceByName(service_name)
	except GeneralError, e:
		luci_log.debug_verbose('vSA5b: no service named %s found' % service_name)
		cur_service = None
	except Exception, e:
		luci_log.debug_verbose('vSA5c: no service named %s found: %s' % (service_name, str(e)))
		cur_service = None

	try:
		if request.form['action'] == 'edit':
			if cur_service is None:
				return (False, {'errors': [ 'The service %s could not be found for editing.' % service_name ]})
			model.deleteService(service_name)
		elif request.form['action'] == 'add':
			if cur_service is not None:
				return (False, {'errors': [ 'A service with the name %s already exists.' % service_name ]})
		else:
			luci_log.debug_verbose('vSA4a: unknown action %s' % request.form['action'])
			return (False, {'errors': [ 'An unknown action was specified.' ]})
	except Exception, e:
		luci_log.debug_verbose('vSA5: no action type: %s' % str(e))

	def buildSvcTree(parent, child_id_list):
		for i in child_id_list:
			try:
				child = form_hash[i]['obj']
				if not child:
					raise Exception, 'No object for %s' % i
			except Exception, e:
				luci_log.debug_verbose('bST0: %s' % str(e))
				continue
			parent.addChild(child)
			if 'kids' in form_hash[i]:
				buildSvcTree(child, form_hash[i]['kids'])

	new_service = Service()
	new_service.addAttribute('name', service_name)
	if fdom:
		new_service.addAttribute('domain', fdom)
	if recovery:
		new_service.addAttribute('recovery', recovery)
	new_service.addAttribute('exclusive', str(exclusive))
	if autostart is not None:
		new_service.attr_hash['autostart'] = autostart

	buildSvcTree(new_service, form_hash['toplevel']['kids'])
	model.resourcemanager_ptr.addChild(new_service)

	clustername = model.getClusterName()
	if not clustername:
		luci_log.debug_verbose('vAS6: no cluname from mb')
		return (False, {'errors': [ 'Unable to determine cluster name' ]})

	try:
		model.setModified(True)
		conf = model.exportModelAsString()
		if not conf:
			raise Exception, 'model string for %s is blank' % clustername
	except Exception, e:
		luci_log.debug_verbose('vAS6a: exportModelAsString : %s' \
			% str(e))
		return (False, {'errors': [ 'An error occurred while adding this service.' ]})

	rc = getRicciAgent(self, clustername)
	if not rc:
		luci_log.debug_verbose('vAS6b: unable to find a ricci agent for cluster %s' % clustername)
		return 'Unable to find a ricci agent for the %s cluster' % clustername

	try:
		ragent = rc.hostname()
		if not ragent:
			luci_log.debug_verbose('vAS7: missing ricci hostname')
			raise Exception, 'unknown ricci agent hostname'

		batch_number, result = setClusterConf(rc, str(conf))
		if batch_number is None or result is None:
			luci_log.debug_verbose('vAS8: missing batch_number or result')
			raise Exception, 'unable to save the new cluster configuration.'
	except Exception, e:
		luci_log.debug_verbose('vAS9: %s' % str(e))
		return 'An error occurred while propagating the new cluster.conf: %s' % str(e)

	try:
		if request.form['action'] == 'edit':
			set_node_flag(self, clustername, ragent, str(batch_number), SERVICE_CONFIG, "Configuring service \'%s\'" % service_name)
		else:
			set_node_flag(self, clustername, ragent, str(batch_number), SERVICE_ADD, "Adding new service \'%s\'" % service_name)
	except Exception, e:
		luci_log.debug_verbose('vAS10: failed to set flags: %s' % str(e))

	response = request.RESPONSE
	response.redirect(request['URL'] + "?pagetype=" + SERVICES + "&clustername=" + clustername + '&busyfirst=true')

def validateResourceAdd(self, request):
	try:
		res_type = request.form['type'].strip()
		if not res_type:
			raise KeyError, 'type is blank'
	except Exception, e:
		luci_log.debug_verbose('resourceAdd: type is blank')
		return (False, {'errors': ['No resource type was given.']})

	errors = list()
	try:
		res = resourceAddHandler[res_type](request)
		if res is None or res[0] is None or res[1] is None:
			if res and res[2]:
				errors.extend(res[2])
			raise Exception, 'An error occurred while adding this resource'
		model = res[1]
		newres = res[0]
		addResource(self, request, model, newres, res_type)
	except Exception, e:
		if len(errors) < 1:
			errors.append('An error occurred while adding this resource')
		luci_log.debug_verbose('resource error: %s' % str(e))
		return (False, {'errors': errors})

	return (True, {'messages': ['Resource added successfully']})

## Cluster properties form validation routines

# rhel5 cluster version
def validateMCastConfig(model, form):
	try:
		gulm_ptr = model.getGULMPtr()
		if gulm_ptr:
			return (False, {'errors': ['Multicast cannot be used with GULM locking.']})
	except:
		pass

	errors = list()
	try:
		mcast_val = form['mcast'].strip().lower()
		if mcast_val != 'true' and mcast_val != 'false':
			raise KeyError, mcast_val
		if mcast_val == 'true':
			mcast_manual = True
		else:
			mcast_manual = False
	except KeyError, e:
		errors.append('An invalid multicast selection was made')
		return (False, {'errors': errors})

	if mcast_manual == True:
		try:
			addr_str = form['mcast_address'].strip()
			socket.inet_pton(socket.AF_INET, addr_str)
		except KeyError, e:
			addr_str = None
			errors.append('No multicast address was given')
		except socket.error, e:
			try:
				socket.inet_pton(socket.AF_INET6, addr_str)
			except socket.error, e:
				addr_str = None
				errors.append('An invalid multicast address was given: %s')
	else:
		addr_str = None

	if (addr_str is None and mcast_manual != True) or (mcast_manual == True and addr_str == model.getMcastAddr()):
		errors.append('No multicast configuration changes were made.')
		return (False, {'errors': errors})

	try:
		model.usesMulticast = True
		model.mcast_address = addr_str
	except Exception, e:
		luci_log.debug('Error updating mcast properties: %s' % str(e))
		errors.append('Unable to update cluster multicast properties')

	if len(errors) > 0:
		return (False, {'errors': errors})

	return (True, {})

def validateQDiskConfig(model, form):
	errors = list()

	try:
		qdisk_val = form['quorumd'].strip().lower()
		if qdisk_val != 'true' and qdisk_val != 'false':
			raise KeyError(qdisk_val)
		if qdisk_val == 'true':
			qdisk_val = 1
		else:
			qdisk_val = 0
	except KeyError, e:
		return (False, {'errors': ['An invalid quorum partition selection was made']})

	cp = model.getClusterPtr()
	qdp = model.getQuorumdPtr()

	if not qdisk_val:
		if qdp:
			try:
				cp.removeChild(qdp)
			except Exception, e:
				return (False, {'errors': [ 'Error disabling quorum partition: %s' % str(e) ] })
		return (True, {})

	try:
		interval = int(form['interval'])
		if interval < 0:
			raise ValueError, 'Interval must be 0 or greater'
	except KeyError, e:
		errors.append('No Interval value was given')
	except ValueError, e:
		errors.append('An invalid Interval value was given: %s' % str(e))

	try:
		votes = int(form['votes'])
		if votes < 1:
			raise ValueError, 'Votes must be greater than 0'
	except KeyError, e:
		errors.append('No Votes value was given')
	except ValueError, e:
		errors.append('An invalid Votes value was given: %s' % str(e))

	try:
		tko = int(form['tko'])
		if tko < 0:
			raise ValueError, 'TKO must be 0 or greater'
	except KeyError, e:
		errors.append('No TKO value was given')
	except ValueError, e:
		errors.append('An invalid TKO value was given: %s' % str(e))

	try:
		min_score = int(form['min_score'])
		if min_score < 1:
			raise ValueError('Minimum Score must be greater than 0')
	except KeyError, e:
		errors.append('No Minimum Score value was given')
	except ValueError, e:
		errors.append('An invalid Minimum Score value was given: %s' % str(e))

	#Either device or label must be present
	device = None
	try:
		device = form['device'].strip()
	except:
		device = None

	label = None
	try:
		label = form['label'].strip()
	except:
		label = None

	if not device and not label:
		errors.append('No Device or Label value was given')

	num_heuristics = 0
	try:
		num_heuristics = int(form['num_heuristics']) + 1
		if num_heuristics < 1:
			raise ValueError, form['num_heuristics']
	except KeyError, e:
		errors.append('No number of heuristics was given.')
	except ValueError, e:
		errors.append('An invalid number of heuristics was given: %s' % str(e))

	heuristics = list()
	for i in xrange(num_heuristics):
		try:
			h = form['heuristic%d' % i]
			if not h or len(h) != 3:
				continue
		except:
			continue

		try:
			hprog = h[0]
			if not hprog:
				raise Exception, 'no hprog'
		except Exception, e:
			errors.append('No program was given for heuristic %d' % i + 1)
		try:
			hint = int(h[1])
			if hint < 1:
				raise ValueError, 'Heuristic interval values must be greater than 0'
		except KeyError, e:
			errors.append('No interval was given for heuristic #%d' % i + 1)
		except ValueError, e:
			errors.append('An invalid interval was given for heuristic %d: %s' \
				% (i + 1, str(e)))

		try:
			hscore = int(h[2])
			if hscore < 1:
				raise ValueError, 'Heuristic scores must be greater than 0'
		except KeyError, e:
			errors.append('No score was given for heuristic %d' % i + 1)
		except ValueError, e:
			errors.append('An invalid score was given for heuristic %d: %s' \
				% (i + 1, str(e)))

		heuristics.append([ hprog, hint, hscore ])

	if len(errors) > 0:
		return (False, {'errors': errors })

	qd = QuorumD()
	qd.addAttribute('interval', str(interval))
	qd.addAttribute('votes', str(votes))
	qd.addAttribute('tko', str(tko))
	qd.addAttribute('min_score', str(min_score))

	if device:
		qd.addAttribute('device', str(device))
	else:
		qd.addAttribute('label', str(label))

	if qdp:
		try:
			cp.removeChild(qdp)
		except:
			pass
	cp.addChild(qd)

	for h in heuristics:
		new_h = Heuristic()
		new_h.addAttribute('program', str(h[0]))
		new_h.addAttribute('interval', str(h[1]))
		new_h.addAttribute('score', str(h[2]))
		qd.addChild(new_h)

	if len(errors) > 0:
		return (False, {'errors': errors })

	return (True, {})

def validateGeneralConfig(model, form):
	errors = list()

	try:
		cp = model.getClusterPtr()
		old_name = model.getClusterAlias()
		old_ver = int(cp.getConfigVersion())
	except Exception, e:
		luci_log.debug_verbose('getConfigVersion: %s' % str(e))
		errors.append('unable to determine the current configuration version')
		return (False, {'errors': errors})

	try:
		cluster_name = form['cluname'].strip()
		if not cluster_name:
			raise KeyError('cluname')
	except KeyError, e:
		errors.append('No cluster name was given.')

	if len(cluster_name) > 15:
		errors.append('A cluster\'s name must be less than 16 characters long.')

	try:
		version_num = int(form['cfgver'])
		if version_num < old_ver:
			raise ValueError, 'configuration version number must be %d or greater.' % old_ver
		if version_num != old_ver:
			# we'll increment the cluster version before propagating it.
			version_num -= 1
	except KeyError, e:
		errors.append('No cluster configuration version was given.')
	except ValueError, e:
		errors.append('An invalid configuration version was given: %s' % str(e))

	if len(errors) < 1:
		try:
			if cluster_name != old_name:
				cp.addAttribute('alias', cluster_name)
			cp.setConfigVersion(str(version_num))
		except Exception, e:
			luci_log.debug_verbose('unable to update general properties: %s' % str(e))
			errors.append('Unable to update the cluster configuration.')

	try:
		cluster_version = form['cluster_version'].strip()
		if cluster_version != 'rhel5':
			raise Exception, 'not rhel5'
	except:
		if len(errors) > 0:
			return (False, {'errors': errors})
		return (True, {})

	totem = model.getTotemPtr()
	if totem is None:
		cp = model.getClusterPtr()
		totem = Totem()
		cp.addChild(totem)

	try:
		token = form['token'].strip()
		if not token:
			raise KeyError, 'token'
		token = int(token)
		if token < 1:
			raise ValueError, '%d is an invalid value for token timeout' % token
		totem.addAttribute('token', str(token))
	except KeyError, e:
		try:
			totem.removeAttribute('token')
		except:
			pass
	except Exception, e:
		errors.append(str(e))

	try:
		token_retransmits_before_loss_const = form['token_retransmits_before_loss_const'].strip()
		if not token_retransmits_before_loss_const:
			raise KeyError, 'token_retransmits_before_loss_const'
		token_retransmits_before_loss_const = int(token_retransmits_before_loss_const)
		if token_retransmits_before_loss_const < 1:
			raise ValueError, '%d is an invalid value for number of token retransmits before loss' % token_retransmits_before_loss_const
		totem.addAttribute('token_retransmits_before_loss_const', str(token_retransmits_before_loss_const))
	except KeyError, e:
		try:
			totem.removeAttribute('token_retransmits_before_loss_const')
		except:
			pass
	except Exception, e:
		errors.append(str(e))

	try:
		join = form['join'].strip()
		if not join:
			raise KeyError, 'join'
		join = int(join)
		if join < 1:
			raise ValueError, '%d is an invalid value for join timeout' % join
		totem.addAttribute('join', str(join))
	except KeyError, e:
		try:
			totem.removeAttribute('join')
		except:
			pass
	except Exception, e:
		errors.append(str(e))

	try:
		consensus = form['consensus'].strip()
		if not consensus:
			raise KeyError, 'consensus'
		consensus = int(consensus)
		if consensus < 1:
			raise ValueError, '%d is an invalid value for consensus timeout' % consensus
		totem.addAttribute('consensus', str(consensus))
	except KeyError, e:
		try:
			totem.removeAttribute('consensus')
		except:
			pass
	except Exception, e:
		errors.append(str(e))

	if len(errors) > 0:
		return (False, {'errors': errors})
	return (True, {})

def validateFenceConfig(model, form):
	errors = list()

	if model.getGULMPtr() is not None:
		return (False, {'errors': [ 'GULM clusters do not support fenced.' ]})

	try:
		post_fail_delay = int(form['post_fail_delay'])
		if post_fail_delay < 0:
			raise ValueError('post fail delay values must be 0 or greater.')
	except KeyError, e:
		errors.append('No post fail delay was given.')
	except ValueError, e:
		errors.append('Invalid post fail delay: %s' % str(e))

	try:
		post_join_delay = int(form['post_join_delay'])
		if post_join_delay < 0:
			raise ValueError('post join delay values must be 0 or greater.')
	except KeyError, e:
		errors.append('No post join delay was given.')
	except ValueError, e:
		errors.append('Invalid post join delay: %s' % str(e))

	run_xvmd = False
	try:
		run_xvmd = form.has_key('run_xvmd')
	except:
		pass

	if run_xvmd is True and not model.hasFenceXVM():
		fenceXVMd = FenceXVMd()
		model.addFenceXVM(fenceXVMd)
	elif not run_xvmd:
		model.delFenceXVM()

	try:
		fd = model.getFenceDaemonPtr()
		old_pj_delay = fd.getPostJoinDelay()
		old_pf_delay = fd.getPostFailDelay()

		if post_join_delay == old_pj_delay and post_fail_delay == old_pf_delay:
			errors.append('No fence daemon properties were changed.')
		else:
			fd.setPostJoinDelay(str(post_join_delay))
			fd.setPostFailDelay(str(post_fail_delay))
	except Exception, e:
		luci_log.debug_verbose('Unable to update fence daemon properties: %s' % str(e))
		errors.append('An error occurred while attempting to update fence daemon properties.')

	if len(errors) > 0:
		return (False, {'errors': errors })

	return (True, {})

def validateGULMConfig(model, form):
	gulm_ptr = model.getGULMPtr()
	if not gulm_ptr:
		return (False, {'errors': [ 'This cluster appears not to be using GULM locking.' ]})

	node_list = map(lambda x: x.getName(), gulm_ptr.getChildren())
	for i in map(lambda x: x.getName(), model.getNodes()):
		if not i in node_list:
			node_list.append(i)

	gulm_lockservers = list()
	for node in node_list:
		if form.has_key(node) and form[node] == 'on':
			ls = Lockserver()
			ls.addAttribute('name', node)
			gulm_lockservers.append(ls)

	try:
		xlockservers = filter(lambda x: x.strip(), form['__GULM__'])
	except:
		xlockservers = list()

	for i in xlockservers:
		if not i in node_list:
			ls = Lockserver()
			ls.addAttribute('name', i)
			gulm_lockservers.append(ls)

	num_ls = len(gulm_lockservers)
	if not num_ls in (1, 3, 5):
		return (False, {'errors': [ 'You must have exactly 1, 3, or 5 GULM lock servers. You submitted %d lock servers.' % num_ls ]})

	model.GULM_ptr.children = gulm_lockservers
	return (True, {})

configFormValidators = {
	'general': validateGeneralConfig,
	'mcast': validateMCastConfig,
	'fence': validateFenceConfig,
	'qdisk': validateQDiskConfig,
	'gulm': validateGULMConfig
}

def validateConfigCluster(self, request):
  errors = list()
  messages = list()
  rc = None

  try:
    model = request.SESSION.get('model')
    if not model:
      raise Exception, 'model is none'
  except Exception, e:
    model = None
    try:
      cluname = request.form['clustername']
    except:
      try:
        cluname = request['clustername']
      except:
        luci_log.debug_verbose('VCC0a: no model, no cluster name')
        return (False, {'errors': ['No cluster model was found.']})

    try:
      model = getModelForCluster(self, cluname)
    except:
      model = None

    if model is None:
      luci_log.debug_verbose('VCC0: unable to get model from session')
      return (False, {'errors': ['No cluster model was found.']})
  try:
    if not 'configtype' in request.form:
      luci_log.debug_verbose('VCC2: no configtype')
      raise Exception, 'no config type'
  except Exception, e:
    luci_log.debug_verbose('VCC2a: %s' % str(e))
    return (False, {'errors': ['No configuration type was submitted.']})

  if not request.form['configtype'] in configFormValidators:
    luci_log.debug_verbose('VCC3: invalid config type: %s' % request.form['configtype'])
    return (False, {'errors': ['An invalid configuration type was submitted.']})

  try:
    cp = model.getClusterPtr()
  except:
    luci_log.debug_verbose('VCC3a: getClusterPtr failed')
    return (False, {'errors': ['No cluster model was found.']})

  config_validator = configFormValidators[request.form['configtype']]
  ret = config_validator(model, request.form)

  retcode = ret[0]
  if 'errors' in ret[1]:
    errors.extend(ret[1]['errors'])

  if 'messages' in ret[1]:
    messages.extend(ret[1]['messages'])

  if retcode == True:
    try:
      config_ver = int(cp.getConfigVersion()) + 1
      # always increment the configuration version
      cp.setConfigVersion(str(config_ver))
      model.setModified(True)
      conf_str = model.exportModelAsString()
      if not conf_str:
        raise Exception, 'conf_str is none'
    except Exception, e:
      luci_log.debug_verbose('VCC4: export model as string failed: %s' \
 			% str(e))
      errors.append('Unable to store the new cluster configuration')

  try:
    clustername = model.getClusterName()
    if not clustername:
      raise Exception, 'cluster name from model.getClusterName() is blank'
  except Exception, e:
    luci_log.debug_verbose('VCC5: error: getClusterName: %s' % str(e))
    errors.append('Unable to determine cluster name from model')

  if len(errors) > 0:
    return (retcode, {'errors': errors, 'messages': messages})

  if not rc:
    rc = getRicciAgent(self, clustername)
    if not rc:
      luci_log.debug_verbose('VCC6: unable to find a ricci agent for the %s cluster' % clustername)
      errors.append('Unable to contact a ricci agent for cluster %s' \
      % clustername)

  if rc:
    batch_id, result = setClusterConf(rc, str(conf_str))
    if batch_id is None or result is None:
      luci_log.debug_verbose('VCC7: setCluserConf: batchid or result is None')
      errors.append('Unable to propagate the new cluster configuration for %s' \
      % clustername)
    else:
      try:
        set_node_flag(self, clustername, rc.hostname(), batch_id, CLUSTER_CONFIG, 'Updating cluster configuration')
      except:
        pass

  if len(errors) < 1:
    messages.append('The cluster properties have been updated.')
  else:
    return (retcode, {'errors': errors, 'messages': messages})

  response = request.RESPONSE
  response.redirect(request['URL'] + "?pagetype=" + CLUSTER_CONFIG + "&clustername=" + clustername + '&busyfirst=true')

def validateFenceAdd(self, request):
  errors = list()
  messages = list()
  rc = None

  try:
    model = request.SESSION.get('model')
    if not model:
      raise Exception, 'model is none'
  except Exception, e:
    model = None
    try:
      cluname = request.form['clustername']
    except:
      try:
        cluname = request['clustername']
      except:
        luci_log.debug_verbose('VFE: no model, no cluster name')
        return (False, {'errors': ['No cluster model was found.']})

    try:
      model = getModelForCluster(self, cluname)
    except:
      model = None

    if model is None:
      luci_log.debug_verbose('VFE: unable to get model from session')
      return (False, {'errors': ['No cluster model was found.']})

  form = None
  try:
    response = request.response
    form = request.form
    if not form:
      form = None
      raise Exception, 'no form was submitted'
  except:
    pass

  if form is None:
    luci_log.debug_verbose('VFE: no form was submitted')
    return (False, {'errors': ['No form was submitted']})

  error_code, retobj = validateNewFenceDevice(form, model)
  if error_code == FD_VAL_SUCCESS:
    try:
      conf_str = model.exportModelAsString()
      if not conf_str:
        raise Exception, 'conf_str is none'
    except Exception, e:
      luci_log.debug_verbose('VFE: export model as string failed: %s' \
        % str(e))
      errors.append('Unable to store the new cluster configuration')

    try:
      clustername = model.getClusterName()
      if not clustername:
        raise Exception, 'cluster name from model.getClusterName() is blank'
    except Exception, e:
      luci_log.debug_verbose('VFA: error: getClusterName: %s' % str(e))
      errors.append('Unable to determine cluster name from model')

    if not rc:
      rc = getRicciAgent(self, clustername)
      if not rc:
        luci_log.debug_verbose('VFA: unable to find a ricci agent for the %s cluster' % clustername)
        errors.append('Unable to contact a ricci agent for cluster %s' \
        % clustername)

    if rc:
      batch_id, result = setClusterConf(rc, str(conf_str))
      if batch_id is None or result is None:
        luci_log.debug_verbose('VFA: setCluserConf: batchid or result is None')
        errors.append('Unable to propagate the new cluster configuration for %s' \
        % clustername)
      else:
        try:
          set_node_flag(self, clustername, rc.hostname(), batch_id,
            CLUSTER_CONFIG, 'Adding new fence device \"%s\"' % retobj)
        except:
          pass

    response.redirect(request['URL'] + "?pagetype=" + FENCEDEV + "&clustername=" + clustername + "&fencename=" + retobj + '&busyfirst=true')
  else:
    errors.extend(retobj)
    return (False, {'errors': errors, 'messages': messages})


def validateFenceEdit(self, request):
  errors = list()
  messages = list()
  rc = None

  try:
    model = request.SESSION.get('model')
    if not model:
      raise Exception, 'model is none'
  except Exception, e:
    model = None
    try:
      cluname = request.form['clustername']
    except:
      try:
        cluname = request['clustername']
      except:
        luci_log.debug_verbose('VFE: no model, no cluster name')
        return (False, {'errors': ['No cluster model was found.']})

    try:
      model = getModelForCluster(self, cluname)
    except:
      model = None

    if model is None:
      luci_log.debug_verbose('VFE: unable to get model from session')
      return (False, {'errors': ['No cluster model was found.']})

  form = None
  try:
    response = request.response
    form = request.form
    if not form:
      form = None
      raise Exception, 'no form was submitted'
  except:
    pass

  if form is None:
    luci_log.debug_verbose('VFE: no form was submitted')
    return (False, {'errors': ['No form was submitted']})

  #This is a fence edit situation, so the model should already have an
  #entry for this fence device.
  #
  #pass form and model to validation method, then save changes if it passes.
  error_code, retobj = validateFenceDevice(form, model)
  if error_code == FD_VAL_SUCCESS:
    try:
      conf_str = model.exportModelAsString()
      if not conf_str:
        raise Exception, 'conf_str is none'
    except Exception, e:
      luci_log.debug_verbose('VFE: export model as string failed: %s' \
        % str(e))
      errors.append('Unable to store the new cluster configuration')

    try:
      clustername = model.getClusterName()
      if not clustername:
        raise Exception, 'cluster name from model.getClusterName() is blank'
    except Exception, e:
      luci_log.debug_verbose('VFA: error: getClusterName: %s' % str(e))
      errors.append('Unable to determine cluster name from model')

    if not rc:
      rc = getRicciAgent(self, clustername)
      if not rc:
        luci_log.debug_verbose('VFA: unable to find a ricci agent for the %s cluster' % clustername)
        errors.append('Unable to contact a ricci agent for cluster %s' \
          % clustername)

    if rc:
      batch_id, result = setClusterConf(rc, str(conf_str))
      if batch_id is None or result is None:
        luci_log.debug_verbose('VFA: setClusterConf: batchid or result is None')
        errors.append('Unable to propagate the new cluster configuration for %s' \
          % clustername)
      else:
        try:
          set_node_flag(self, clustername, rc.hostname(), batch_id,
            CLUSTER_CONFIG, 'Updating fence device \"%s\"' % retobj)
        except:
          pass

    response.redirect(request['URL'] + "?pagetype=" + FENCEDEV + "&clustername=" + clustername + "&fencename=" + retobj + '&busyfirst=true')
  else:
    errors.extend(retobj)
    return (False, {'errors': errors, 'messages': messages})

def validateNodeFenceConfig(self, request):
	errors = list()

	try:
		form_xml = request['fence_xml']
		if not form_xml:
			raise KeyError, 'form_xml must not be blank'
	except Exception, e:
		luci_log.debug_verbose('vNFC0: no form_xml: %s' % str(e))
		return (False, {'errors': ['No fence data was supplied.']})

	try:
		fence_level = int(request.form['fence_level'].strip())
	except Exception, e:
		luci_log.debug_verbose('vNFC1: no fence level: %s' % str(e))
		return (False, {'errors': ['No fence level was supplied.']})

	try:
		nodename = request['nodename']
		if not nodename:
			raise Exception, 'nodename is blank'
	except Exception, e:
		luci_log.debug_verbose('vNFC2: no nodename: %s' % str(e))
		return (False, {'errors': ['No node name was given.']})

	try:
		clustername = request['clustername']
		if not clustername:
			raise Exception, 'clustername is blank'
	except Exception, e:
		luci_log.debug_verbose('vNFC3: no clustername: %s' % str(e))
		return (False, {'errors': ['No cluster name was given.']})

	try:
		model = request.SESSION.get('model')
		if not model:
			raise Exception, 'model is none'
	except Exception, e:
		model = None
		try:
			model = getModelForCluster(self, clustername)
		except:
			model = None

	if model is None:
		luci_log.debug_verbose('vNFC4: unable to get model for cluster %s' % clustername)
		return (False, {'errors': ['No cluster model was found.']})

	try:
		doc = minidom.parseString(form_xml)
	except Exception, e:
		luci_log.debug_verbose('vNFC5: error: %s' % str(e))
		return (False, {'errors': ['The fence data submitted is not properly formed.']})

	try:
		node = model.retrieveNodeByName(nodename)
	except GeneralError, e:
		luci_log.debug_verbose('vNFC6: unable to find node name %s in current node list' % (str(nodename), str(e)))
		return (False, {'errors': ['Unable to find the cluster node %s in the node list.' % str(nodename) ]})

	fence_level_num = int(fence_level)
	levels = node.getFenceLevels()
	try:
		method_id = levels[fence_level_num - 1].getAttribute('name')
		if not method_id:
			raise Exception, 'No method ID'
		fence_method = Method()
		fence_method.addAttribute('name', str(method_id))
		levels[fence_level_num - 1] = fence_method
	except Exception, e:
		method_id = fence_level
		fence_method = Method()
		fence_method.addAttribute('name', str(method_id))

	forms = doc.getElementsByTagName('form')
	if len(forms) < 1:
		delete_target = None
		for l in levels:
			# delete the fence level
			if l.getAttribute('name') == method_id:
				delete_target = l
				break
		if delete_target is not None:
			try:
				node.getChildren()[0].removeChild(delete_target)
			except Exception, e:
				luci_log.debug_verbose('vNFC6a: %s: %s' % (method_id, str(e)))
				return (False, {'errors': ['An error occurred while deleting fence method %s' % method_id ]})
		else:
			return (True, {'messages': ['No changes were made.'] })

	form_hash = {}
	for i in forms:
		form_id = i.getAttribute('id')
		if not form_id:
			continue
		ielems = i.getElementsByTagName('input')
		if not ielems or len(ielems) < 1:
			continue

		dummy_form = {}

		for i in ielems:
			try:
				input_type = str(i.getAttribute('type'))
			except Exception, e:
				luci_log.debug_verbose('vNFC7: input type: %s' % str(e))
				continue

			if not input_type or input_type == 'button':
				continue

			try:
				dummy_form[str(i.getAttribute('name'))] = str(i.getAttribute('value'))
			except Exception, e:
				luci_log.debug_verbose('vNFC8: parsing XML: %s' % str(e))

		if len(dummy_form) < 1:
			continue

		if 'fence_instance' in dummy_form:
			try:
				parent = dummy_form['parent_fencedev']
			except:
				luci_log.debug_verbose('vNFC9: no parent for instance')
				return (False, {'errors': [ 'Unable to determine what device the current instance uses.' ]})

			try:
				form_hash[parent][1].append(dummy_form)
				del dummy_form['fence_instance']
			except Exception, e:
				luci_log.debug_verbose('vNFC10: no parent for instance')
				return (False, {'errors': [ 'Unable to determine what device the current instance uses.' ]})
		else:
			form_hash[form_id] = (dummy_form, list())

	fh_keys = form_hash.keys()
	fh_keys.sort()
	for i in fh_keys:
		fencedev_name = None
		fencedev_unknown = False

		try:
			fence_form, instance_list = form_hash[i]
		except Exception, e:
			luci_log.debug_verbose('vNFC11: %s' % str(e))
			continue

		try:
			fence_type = fence_form['fence_type']
			if not fence_type:
				raise Exception, 'fence type is blank'
		except Exception, e:
			luci_log.debug_verbose('vNFC12: %s %s' % (i, str(e)))
			fence_type = None

		if 'existing_device' in fence_form:
			try:
				fencedev_name = fence_form['name']
				if not fencedev_name.strip():
					raise Exception, 'no fence name'
			except Exception, e:
				errors.append('You must provide a unique name for all fence devices.')
				continue

			if fence_type is None:
				# An unknown fence device agent. Pull the data out of
				# the model and persist it and all instances.
				# All we care about is its name.
				fencedev_unknown = True
			else:
				if not 'sharable' in fence_form:
					# If it's a shared fence device that already exists, the
					# user could not have edited it (without playing dirty
					# games), so it's safe to pull the existing entry from
					# the model. All we need is the device name, and nothing
					# else needs to be done here.
					#
					# For an existing non-shared device update the device
					# in the model, since the user could have edited it.
					retcode, retmsg = validateFenceDevice(fence_form, model)
					if retcode != FD_VAL_SUCCESS:
						errors.extend(retmsg)
						continue
					else:
						fencedev_name = retmsg

					# Add back the tags under the method block
					# for the fence instance
					instance_list.append({'name': fencedev_name })
		else:
			# The user created a new fence device.
			retcode, retmsg = validateNewFenceDevice(fence_form, model)
			if retcode != FD_VAL_SUCCESS:
				errors.extend(retmsg)
				continue
			else:
				fencedev_name = retmsg

			# If it's not shared, we need to create an instance form
			# so the appropriate XML goes into the <method> block inside
			# <node><fence>. All we need for that is the device name.
			if not 'sharable' in fence_form:
				instance_list.append({'name': fencedev_name })

		if fencedev_unknown is True:
			# Save any instances for this fence device.
			# XXX FIX ME - instances must be saved.
			pass

		for inst in instance_list:
			retcode, retobj = validate_fenceinstance(inst, fencedev_name, fence_type)
			if retcode != FD_VAL_SUCCESS:
				errors.extend(retobj)
				continue
			fence_method.addChild(retobj)

		if len(node.getChildren()) > 0:
			# There's already a <fence> block
			found_target = False
			for idx in xrange(len(levels)):
				if levels[idx].getAttribute('name') == method_id:
					found_target = True
					break

			if found_target is False:
				# There's a fence block, but no relevant method
				# block
				node.getChildren()[0].addChild(fence_method)
		else:
			# There is no <fence> tag under the node yet.
			fence_node = Fence()
			fence_node.addChild(fence_method)
			node.addChild(fence_node)

	if len(errors) > 0:
		return (False, {'errors': errors })

	try:
		model.setModified(True)
		conf = str(model.exportModelAsString())
		if not conf:
			raise Exception, 'model string is blank'
		luci_log.debug_verbose('vNFC16: exported \"%s\"' % conf)
	except Exception, e:
		luci_log.debug_verbose('vNFC17: exportModelAsString failed: %s' \
			% str(e))
		return (False, {'errors': [ 'An error occurred while constructing the new cluster configuration.' ]})

	rc = getRicciAgent(self, clustername)
	if not rc:
		luci_log.debug_verbose('vNFC18: unable to find a ricci agent for cluster %s' % clustername)
		return (False, {'errors': ['Unable to find a ricci agent for the %s cluster' % clustername ]})
	ragent = rc.hostname()

	batch_number, result = setClusterConf(rc, conf)
	if batch_number is None or result is None:
		luci_log.debug_verbose('vNFC19: missing batch and/or result')
		return (False, {'errors': [ 'An error occurred while constructing the new cluster configuration.' ]})

	try:
		set_node_flag(self, clustername, ragent, str(batch_number), FENCEDEV_NODE_CONFIG, "Updating fence configuration for node \'%s\'" % nodename)
	except Exception, e:
		luci_log.debug_verbose('vNFC20: failed to set flags: %s' % str(e))

	response = request.RESPONSE
	response.redirect(request['URL'] + "?pagetype=" + NODE + "&clustername=" + clustername + '&nodename=' + nodename + '&busyfirst=true')

def deleteFenceDevice(self, request):
  errors = list()
  messages = list()
  rc = None

  try:
    model = request.SESSION.get('model')
    if not model:
      raise Exception, 'model is none'
  except Exception, e:
    model = None
    try:
      cluname = request.form['clustername']
    except:
      try:
        cluname = request['clustername']
      except:
        luci_log.debug_verbose('VFE: no model, no cluster name')
        return (False, {'errors': ['No cluster model was found.']})

    try:
      model = getModelForCluster(self, cluname)
    except:
      model = None

    if model is None:
      luci_log.debug_verbose('VFE: unable to get model from session')
      return (False, {'errors': ['No cluster model was found.']})

  form = None
  try:
    response = request.response
    form = request.form
    if not form:
      form = None
      raise Exception, 'no form was submitted'
  except:
    pass

  if form is None:
    luci_log.debug_verbose('VFE: no form was submitted')
    return (False, {'errors': ['No form was submitted']})

  #get name of fencedev
  try:
    fencedev_name = form['orig_name']
    fencedev_name = fencedev_name.strip()
  except KeyError, e:
    return (False, {'errors':['No device name in form submission']})

  fdev_to_delete = None
  #iterate thru list of current fencedevs and find one to be deleted
  fdevs = model.getFenceDevices()
  for fdev in fdevs:
    if fdev.getName().strip() == fencedev_name:
      fdev_to_delete = fdev
      break
  if fdev_to_delete == None:
    luci_log.debug_verbose('VFD: Could not find fence device name in model')
    return (False, {'errors':['Could not find fence device name in model']})

  #get fencedev ptr
  fdev_ptr = model.getFenceDevicePtr()
  #remove child
  try:
    fdev_ptr.removeChild(fdev_to_delete)
    error_code = FD_VAL_SUCCESS
    error_string = "Fence device %s successfully removed from configuration" % fencedev_name
  except:
    error_code = FD_VAL_FAIL
    error_string = "Fence device %s could not be removed from configuration" % fencedev_name

  try:
    model.removeFenceInstancesForFenceDevice(fencedev_name)
  except:
    luci_log.debug_verbose('VFD: Could not remove fence instances for')


  if error_code == FD_VAL_SUCCESS:
    messages.append(error_string)
    try:
      model.setModified(True)
      conf_str = model.exportModelAsString()
      if not conf_str:
        raise Exception, 'conf_str is none'
    except Exception, e:
      luci_log.debug_verbose('VFE: export model as string failed: %s' \
      % str(e))
      errors.append('Unable to store the new cluster configuration')

    try:
      clustername = model.getClusterName()
      if not clustername:
        raise Exception, 'cluster name from model.getClusterName() is blank'
    except Exception, e:
      luci_log.debug_verbose('VFA: error: getClusterName: %s' % str(e))
      errors.append('Unable to determine cluster name from model')

    if not rc:
      rc = getRicciAgent(self, clustername)
      if not rc:
        luci_log.debug_verbose('VFA: unable to find a ricci agent for the %s cluster' % clustername)
        errors.append('Unable to contact a ricci agent for cluster %s' \
        % clustername)

    if rc:
      batch_id, result = setClusterConf(rc, str(conf_str))
      if batch_id is None or result is None:
        luci_log.debug_verbose('VFA: setCluserConf: batchid or result is None')
        errors.append('Unable to propagate the new cluster configuration for %s' \
        % clustername)
      else:
        try:
          set_node_flag(self, clustername, rc.hostname(), batch_id,
            CLUSTER_CONFIG, 'Removing fence device \"%s\"' % fencedev_name)
        except:
          pass

    response.redirect(request['URL'] + "?pagetype=" + FENCEDEVS + "&clustername=" + clustername + '&busyfirst=true')
    return (True, {'errors': errors, 'messages': messages})
  else:
    errors.append(error_string)
    return (False, {'errors': errors, 'messages': messages})

def validateDaemonProperties(self, request):
	errors = list()

	form = None
	try:
		response = request.response
		form = request.form
		if not form:
			form = None
			raise Exception, 'no form was submitted'
	except:
		pass

	if form is None:
		luci_log.debug_verbose('VDP0: no form was submitted')
		return (False, {'errors': ['No form was submitted']})

	try:
		nodename = form['nodename'].strip()
		if not nodename:
			raise Exception, 'nodename is blank'
	except Exception, e:
		errors.append('Unable to determine the current node name')
		luci_log.debug_verbose('VDP1: no nodename: %s' % str(e))

	try:
		clustername = form['clustername'].strip()
		if not clustername:
			raise Exception, 'clustername is blank'
	except Exception, e:
		errors.append('Unable to determine the current cluster name')
		luci_log.debug_verbose('VDP2: no clustername: %s' % str(e))

	disable_list = list()
	enable_list = list()
	for i in form.items():
		try:
			if i[0][:11] == '__daemon__:':
				daemon_prop = i[1]
				if len(daemon_prop) == 2:
					if daemon_prop[1] == '1':
						disable_list.append(daemon_prop[0])
				else:
					if daemon_prop[1] == '0' and daemon_prop[2] == 'on':
						enable_list.append(daemon_prop[0])
		except Exception, e:
			luci_log.debug_verbose('VDP3: error: %s' % str(i))

	if len(enable_list) < 1 and len(disable_list) < 1:
		luci_log.debug_verbose('VDP4: no changes made')
		response.redirect(request['URL'] + "?pagetype=" + NODE + "&clustername=" + clustername + '&nodename=' + nodename)

	nodename_resolved = resolve_nodename(self, clustername, nodename)
	try:
		rc = RicciCommunicator(nodename_resolved)
		if not rc:
			raise Exception, 'rc is None'
	except Exception, e:
		luci_log.debug_verbose('VDP5: RC %s: %s' % (nodename_resolved, str(e)))
		errors.append('Unable to connect to the ricci agent on %s to update cluster daemon properties' % nodename_resolved)
		return (False, {'errors': errors})

	batch_id, result = updateServices(rc, enable_list, disable_list)
	if batch_id is None or result is None:
		luci_log.debug_verbose('VDP6: setCluserConf: batchid or result is None')
		errors.append('Unable to update the cluster daemon properties on node %s' % nodename_resolved)
		return (False, {'errors': errors})

	try:
		status_msg = 'Updating %s daemon properties:' % nodename_resolved
		if len(enable_list) > 0:
			status_msg += ' enabling %s' % str(enable_list)[1:-1]
		if len(disable_list) > 0:
			status_msg += ' disabling %s' % str(disable_list)[1:-1]
		set_node_flag(self, clustername, rc.hostname(), batch_id, CLUSTER_DAEMON, status_msg)
	except:
		pass

	if len(errors) > 0:
		return (False, {'errors': errors})

	response.redirect(request['URL'] + "?pagetype=" + NODE + "&clustername=" + clustername + '&nodename=' + nodename + '&busyfirst=true')

def validateFdom(self, request):
	errors = list()

	try:
		model = request.SESSION.get('model')
		if not model:
			raise Exception, 'no model'
	except Exception, e:
		luci_log.debug_verbose('validateFdom0: no model: %s' % str(e))
		return (False, {'errors': [ 'Unable to retrieve cluster information.' ]})

	prioritized = False
	try:
		prioritized = request.form.has_key('prioritized')
	except:
		prioritized = False

	restricted = False
	try:
		restricted = request.form.has_key('restricted')
	except:
		restricted = False

	clustername = None
	try:
		clustername = request.form['clustername'].strip()
		if not clustername:
			raise Exception, 'blank'
	except:
		try:
			clustername = model.getClusterName()
			if not clustername:
				raise Exception, 'blank'
		except:
			clustername = None

	if not clustername:
		errors.append('Unable to determine this cluster\'s name.')

	try:
		name = request.form['name'].strip()
		if not name:
			raise Exception, 'blank'
	except Exception, e:
		errors.append('No name was given for this failover domain.')
		luci_log.debug_verbose('validateFdom0: %s' % str(e))

	oldname = None
	try:
		oldname = request.form['oldname'].strip()
		if not oldname:
			raise Exception, 'blank'
	except:
		pass

	if oldname is None or oldname != name:
		if model.getFailoverDomainByName(name) is not None:
			errors.append('A failover domain named \"%s\" already exists.' % name)

	fdom = None
	if oldname is not None:
		fdom = model.getFailoverDomainByName(oldname)
		if fdom is None:
			luci_log.debug_verbose('validateFdom1: No fdom named %s exists' % oldname)
			errors.append('No failover domain named \"%s" exists.' % oldname)
		else:
			fdom.addAttribute('name', name)
			fdom.children = list()
	else:
		fdom = FailoverDomain()
		fdom.addAttribute('name', name)

	if fdom is None or len(errors) > 0:
		return (False, {'errors': errors })

	if prioritized:
		fdom.addAttribute('ordered', '1')
	else:
		fdom.addAttribute('ordered', '0')

	if restricted:
		fdom.addAttribute('restricted', '1')
	else:
		fdom.addAttribute('restricted', '0')

	cluster_nodes = map(lambda x: str(x.getName()), model.getNodes())

	for i in cluster_nodes:
		if request.form.has_key(i):
			fdn = FailoverDomainNode()
			fdn.addAttribute('name', i)
			if prioritized:
				priority = 1
				try:
					priority = int(request.form['__PRIORITY__' + i].strip())
					if priority < 1:
						priority = 1
				except Exception, e:
					priority = 1
				fdn.addAttribute('priority', str(priority))
			fdom.addChild(fdn)

	try:
		fdom_ptr = model.getFailoverDomainPtr()
		if not oldname:
			fdom_ptr.addChild(fdom)
		model.setModified(True)
		conf = str(model.exportModelAsString())
	except Exception, e:
		luci_log.debug_verbose('validateFdom2: %s' % str(e))
		errors.append('Unable to update the cluster configuration.')

	if len(errors) > 0:
		return (False, {'errors': errors })

	rc = getRicciAgent(self, clustername)
	if not rc:
		luci_log.debug_verbose('validateFdom3: unable to find a ricci agent for cluster %s' % clustername)
		return (False, {'errors': ['Unable to find a ricci agent for the %s cluster' % clustername ]})
	ragent = rc.hostname()

	batch_number, result = setClusterConf(rc, conf)
	if batch_number is None or result is None:
		luci_log.debug_verbose('validateFdom4: missing batch and/or result')
		return (False, {'errors': [ 'An error occurred while constructing the new cluster configuration.' ]})

	try:
		if oldname:
			set_node_flag(self, clustername, ragent, str(batch_number), FDOM, 'Updating failover domain \"%s\"' % oldname)
		else:
			set_node_flag(self, clustername, ragent, str(batch_number), FDOM_ADD, 'Creating failover domain \"%s\"' % name)
	except Exception, e:
		luci_log.debug_verbose('validateFdom5: failed to set flags: %s' % str(e))

	response = request.RESPONSE
	response.redirect(request['URL'] + "?pagetype=" + FDOM + "&clustername=" + clustername + '&fdomname=' + name + '&busyfirst=true')

def validateVM(self, request):
	errors = list()

	model = request.SESSION.get('model')

	try:
		vm_name = request.form['vmname'].strip()
		if not vm_name:
			raise Exception, 'blank'
	except Exception, e:
		luci_log.debug_verbose('validateVM0: no vm name: %s' % str(e))
		errors.append('No virtual machine name was given.')

	try:
		vm_path = request.form['vmpath'].strip()
		if not vm_path:
			raise Exception, 'blank'
	except Exception, e:
		luci_log.debug_verbose('validateVM1: no vm path: %s' % str(e))
		errors.append('No path to the virtual machine configuration file was given.')

	autostart = 1
	try:
		if request.form.has_key('autostart'):
			autostart = 1
		else:
			autostart = 0
	except:
		autostart = 1

	exclusive = 0
	try:
		if request.form.has_key('exclusive'):
			exclusive = 1
		else:
			exclusive = 0
	except:
		exclusive = 0

	recovery = None
	try:
		recovery = request.form['recovery'].strip()
		if not recovery:
			recovery = None
		else:
			if recovery != 'restart' and recovery != 'relocate' and recovery != 'disable':
				errors.append('You entered an invalid recovery option: \"%s\" Valid options are \"restart\" \"relocate\" and \"disable\"')
	except:
		recovery = None

	fdom = None
	try:
		fdom = request.form['domain'].strip()
		if not fdom:
			raise Exception, 'blank'
	except:
		fdom = None

	if len(errors) > 0:
		return (False, {'errors': errors })

	isNew = False
	try:
		old_name = request.form['oldname'].strip()
		if not old_name:
			raise KeyError, 'oldname'
	except KeyError, e:
		isNew = True

	delete_vm = False
	if request.form.has_key('delete'):
		try:
			xvm = model.retrieveVMsByName(old_name)
			if not xvm:
				raise Exception, 'not found'
			rmptr = model.getResourceManagerPtr()
			rmptr.removeChild(xvm)
			delete_vm = True
		except:
			return (False, {'errors': ['No virtual machine service named \"%s\" exists.' % old_name ]})
	else:
		if isNew is True:
			xvm = Vm()
			xvm.addAttribute('name', vm_name)
			xvm.addAttribute('path', vm_path)
			rmptr = model.getResourceManagerPtr()
			rmptr.addChild(xvm)
		else:
			try:
				xvm = model.retrieveVMsByName(old_name)
				if not xvm:
					raise Exception, 'not found'
			except:
				return (False, {'errors': ['No virtual machine service named \"%s\" exists.' % old_name ]})
			xvm.addAttribute('name', vm_name)
			xvm.addAttribute('path', vm_path)

	xvm.addAttribute('autostart', str(autostart))
	xvm.addAttribute('exclusive', str(exclusive))
	if fdom:
		xvm.addAttribute('domain', fdom)
	else:
		try:
			xvm.removeAttribute('domain')
		except:
			pass

	if recovery:
		xvm.addAttribute('recovery', recovery)
	else:
		try:
			xvm.removeAttribute('recovery')
		except:
			pass

	try:
		model.setModified(True)
		stringbuf = str(model.exportModelAsString())
		if not stringbuf:
			raise Exception, 'model is blank'
	except Exception, e:
		luci_log.debug_verbose('validateVM2: %s' % str(e))
		errors.append('Unable to update the cluster model')

	try:
		clustername = model.getClusterName()
		if not clustername:
			raise Exception, 'cluster name from model.getClusterName() is blank'
	except Exception, e:
		luci_log.debug_verbose('validateVM3: %s' % str(e))
		errors.append('Unable to determine the cluster name.')

	if len(errors) > 0:
		return (False, {'errors': errors })

	rc = getRicciAgent(self, clustername)
	if not rc:
		luci_log.debug_verbose('validateVM4: no ricci for %s' % clustername)
		return (False, {'errors': ['Unable to contact a ricci agent for this cluster.']})

	batch_number, result = setClusterConf(rc, stringbuf)
	if batch_number is None or result is None:
		luci_log.debug_verbose('validateVM5: missing batch and/or result')
		return (False, {'errors': [ 'Error creating virtual machine %s.' % vm_name ]})

	try:
		if delete_vm is True:
			set_node_flag(self, clustername, rc.hostname(), str(batch_number), VM_CONFIG, "Deleting virtual machine service \'%s\'" % vm_name)
		elif isNew is True:
			set_node_flag(self, clustername, rc.hostname(), str(batch_number), VM_ADD, "Creating virtual machine service \'%s\'" % vm_name)
		else:
			set_node_flag(self, clustername, rc.hostname(), str(batch_number), VM_CONFIG, "Configuring virtual machine service \'%s\'" % vm_name)
	except Exception, e:
		luci_log.debug_verbose('validateVM6: failed to set flags: %s' % str(e))

	response = request.RESPONSE
	response.redirect(request['URL'] + "?pagetype=" + SERVICES + "&clustername=" + clustername + '&busyfirst=true')

formValidators = {
	6: validateCreateCluster,
	7: validateConfigCluster,
	15: validateAddClusterNode,
	18: validateVM,
	19: validateVM,
	21: validateServiceAdd,
	24: validateServiceAdd,
	31: validateResourceAdd,
	33: validateResourceAdd,
	41: validateFdom,
	44: validateFdom,
	51: validateFenceAdd,
	54: validateFenceEdit,
	55: validateDaemonProperties,
	57: deleteFenceDevice,
	58: validateNodeFenceConfig
}

def validatePost(self, request):
	try:
		pagetype = int(request.form['pagetype'])
	except Exception, e:
		luci_log.debug_verbose('VP0: error: %s' % str(e))
		return None

	if not pagetype in formValidators:
		luci_log.debug_verbose('VP1: no handler for page type %d' % pagetype)
		return None
	else:
		return formValidators[pagetype](self, request)


def createCluChooser(self, request, systems):
  dummynode = {}

  if request.REQUEST_METHOD == 'POST':
    ret = validatePost(self, request)
    try:
      request.SESSION.set('checkRet', ret[1])
    except:
      request.SESSION.set('checkRet', {})
  else:
    try:
      request.SESSION.set('checkRet', {})
    except:
      pass

  #First, see if a cluster is chosen, then
  #check that the current user can access that system
  cname = None
  try:
    cname = request[CLUNAME]
  except:
    cname = ""

  try:
    url = request['URL']
  except:
    url = "/luci/cluster/index_html"

  try:
    pagetype = request[PAGETYPE]
  except:
    pagetype = '3'


  cldata = {}
  cldata['Title'] = "Cluster List"
  cldata['cfg_type'] = "clusters"
  cldata['absolute_url'] = url + "?pagetype=" + CLUSTERLIST
  cldata['Description'] = "Clusters available for configuration"
  if pagetype == CLUSTERLIST:
    cldata['currentItem'] = True
  else:
    cldata['currentItem'] = False

  UserHasPerms = havePermCreateCluster(self)
  if UserHasPerms:
    cladd = {}
    cladd['Title'] = "Create a New Cluster"
    cladd['cfg_type'] = "clusteradd"
    cladd['absolute_url'] = url + "?pagetype=" + CLUSTER_ADD
    cladd['Description'] = "Create a Cluster"
    if pagetype == CLUSTER_ADD:
      cladd['currentItem'] = True
    else:
      cladd['currentItem'] = False

  clcfg = {}
  clcfg['Title'] = "Configure"
  clcfg['cfg_type'] = "clustercfg"
  clcfg['absolute_url'] = url + "?pagetype=" + CLUSTERS
  clcfg['Description'] = "Configure a cluster"
  if pagetype == CLUSTERS:
    clcfg['currentItem'] = True
  else:
    clcfg['currentItem'] = False

  #test...
  #clcfg['show_children'] = True
  #Add all cluster type pages here:
  if pagetype == CLUSTER or pagetype == CLUSTER_CONFIG:
    clcfg['show_children'] = True
  else:
    clcfg['show_children'] = False

  #loop through all clusters
  syslist = list()
  for system in systems:
    clsys = {}
    clsys['Title'] = system[0]
    clsys['cfg_type'] = "cluster"
    clsys['absolute_url'] = url + "?pagetype=" + CLUSTER + "&clustername=" + system[0]
    clsys['Description'] = "Configure this cluster"

    if pagetype == CLUSTER or pagetype == CLUSTER_CONFIG:
      if cname == system[0]:
        clsys['currentItem'] = True
      else:
        clsys['currentItem'] = False
    else:
      clsys['currentItem'] = False
    syslist.append(clsys)

  clcfg['children'] = syslist

  mylist = list()
  mylist.append(cldata)
  if UserHasPerms:
    mylist.append(cladd)
  mylist.append(clcfg)
  dummynode['children'] = mylist

  return dummynode

def getnodes(self, model):
	try:
		return map(lambda x: str(x.getName()), model.getNodes())
	except Exception, e:
		luci_log.debug_verbose('getnodes0: %s' % str(e))
	return []

def createCluConfigTree(self, request, model):
  dummynode = {}

  if not model:
    return {}

  #There should be a positive page type
  try:
    pagetype = request[PAGETYPE]
  except:
    pagetype = '3'

  try:
    url = request['URL']
  except:
    url = "/luci/cluster/index_html"

  #The only way this method can run is if there exists
  #a clustername query var
  cluname = request['clustername']

  nd = {}
  nd['Title'] = "Nodes"
  nd['cfg_type'] = "nodes"
  nd['absolute_url'] = url + "?pagetype=" + NODES + "&clustername=" + cluname
  nd['Description'] = "Node configuration for this cluster"
  if pagetype == NODES or pagetype == NODE_GRID or pagetype == NODE_LIST or pagetype == NODE_CONFIG or pagetype == NODE_ADD or pagetype == NODE:
    nd['show_children'] = True
  else:
    nd['show_children'] = False
  if pagetype == "0":
    nd['show_children'] = False

  if pagetype == NODES:
    nd['currentItem'] = True
  else:
    nd['currentItem'] = False


  ndadd = {}
  ndadd['Title'] = "Add a Node"
  ndadd['cfg_type'] = "nodeadd"
  ndadd['absolute_url'] = url + "?pagetype=" + NODE_ADD + "&clustername=" + cluname
  ndadd['Description'] = "Add a node to this cluster"
  if pagetype == NODE_ADD:
    ndadd['currentItem'] = True
  else:
    ndadd['currentItem'] = False

  ndcfg = {}
  ndcfg['Title'] = "Configure"
  ndcfg['cfg_type'] = "nodecfg"
  ndcfg['absolute_url'] = url + "?pagetype=" + NODE_CONFIG + "&clustername=" + cluname
  ndcfg['Description'] = "Configure cluster nodes"
  if pagetype == NODE_CONFIG or pagetype == NODE or pagetype == NODES or pagetype == NODE_LIST or pagetype == NODE_GRID or pagetype == NODE_ADD:
    ndcfg['show_children'] = True
  else:
    ndcfg['show_children'] = False
  if pagetype == NODE_CONFIG:
    ndcfg['currentItem'] = True
  else:
    ndcfg['currentItem'] = False

  nodes = model.getNodes()
  nodenames = list()
  for node in nodes:
    nodenames.append(node.getName())

  cfgablenodes = list()
  for nodename in nodenames:
    cfg = {}
    cfg['Title'] = nodename
    cfg['cfg_type'] = "node"
    cfg['absolute_url'] = url + "?pagetype=" + NODE + "&nodename=" + nodename + "&clustername=" + cluname
    cfg['Description'] = "Configure this cluster node"
    if pagetype == NODE:
      try:
        nname = request['nodename']
      except KeyError, e:
        nname = ""
      if nodename == nname:
        cfg['currentItem'] = True
      else:
        cfg['currentItem'] = False
    else:
      cfg['currentItem'] = False

    cfgablenodes.append(cfg)

  #Now add nodename structs as children of the config element
  ndcfg['children'] = cfgablenodes

  ndkids = list()
  ndkids.append(ndadd)
  ndkids.append(ndcfg)

  nd['children'] = ndkids

  ##################################################################
  sv = {}
  sv['Title'] = "Services"
  sv['cfg_type'] = "services"
  sv['absolute_url'] = url + "?pagetype=" + SERVICES + "&clustername=" + cluname
  sv['Description'] = "Service configuration for this cluster"
  if pagetype == SERVICES or pagetype == SERVICE_CONFIG or pagetype == SERVICE_ADD or pagetype == SERVICE or pagetype == SERVICE_LIST or pagetype == VM_ADD or pagetype == VM_CONFIG:
    sv['show_children'] = True
  else:
    sv['show_children'] = False
  if pagetype == SERVICES or pagetype == SERVICE_LIST:
    sv['currentItem'] = True
  else:
    sv['currentItem'] = False

  svadd = {}
  svadd['Title'] = "Add a Service"
  svadd['cfg_type'] = "serviceadd"
  svadd['absolute_url'] = url + "?pagetype=" + SERVICE_ADD + "&clustername=" + cluname
  svadd['Description'] = "Add a Service to this cluster"
  if pagetype == SERVICE_ADD:
    svadd['currentItem'] = True
  else:
    svadd['currentItem'] = False

  if model.getIsVirtualized() == True:
    vmadd = {}
    vmadd['Title'] = "Add a Virtual Service"
    vmadd['cfg_type'] = "vmadd"
    vmadd['absolute_url'] = url + "?pagetype=" + VM_ADD + "&clustername=" + cluname
    vmadd['Description'] = "Add a Virtual Service to this cluster"
    if pagetype == VM_ADD:
      vmadd['currentItem'] = True
    else:
      vmadd['currentItem'] = False

  svcfg = {}
  svcfg['Title'] = "Configure a Service"
  svcfg['cfg_type'] = "servicecfg"
  svcfg['absolute_url'] = url + "?pagetype=" + SERVICE_CONFIG + "&clustername=" + cluname
  svcfg['Description'] = "Configure a Service for this cluster"
  if pagetype == SERVICE_CONFIG or pagetype == SERVICE or pagetype == VM_CONFIG:
    svcfg['show_children'] = True
  else:
    svcfg['show_children'] = False
  if pagetype == SERVICE_CONFIG or pagetype == VM_CONFIG:
    svcfg['currentItem'] = True
  else:
    svcfg['currentItem'] = False

  services = model.getServices()
  serviceable = list()
  for service in services:
    servicename = service.getName()
    svc = {}
    svc['Title'] = servicename
    svc['cfg_type'] = "service"
    svc['absolute_url'] = url + "?pagetype=" + SERVICE + "&servicename=" + servicename + "&clustername=" + cluname
    svc['Description'] = "Configure this service"
    if pagetype == SERVICE:
      try:
        sname = request['servicename']
      except KeyError, e:
        sname = ""
      if servicename == sname:
        svc['currentItem'] = True
      else:
        svc['currentItem'] = False
    else:
      svc['currentItem'] = False

    serviceable.append(svc)

  vms = model.getVMs()
  for vm in vms:
    name = vm.getName()
    svc = {}
    svc['Title'] = name
    svc['cfg_type'] = "vm"
    svc['absolute_url'] = url + "?pagetype=" + VM_CONFIG + "&servicename=" + name + "&clustername=" + cluname
    svc['Description'] = "Configure this Virtual Service"
    if pagetype == VM_CONFIG:
      try:
        xname = request['servicename']
      except KeyError, e:
        xname = ""
      if name == xname:
        svc['currentItem'] = True
      else:
        svc['currentItem'] = False
    else:
      svc['currentItem'] = False

    serviceable.append(svc)

  svcfg['children'] = serviceable



  kids = list()
  kids.append(svadd)
  if model.getIsVirtualized() == True:
    kids.append(vmadd)
  kids.append(svcfg)
  sv['children'] = kids
#############################################################
  rv = {}
  rv['Title'] = "Resources"
  rv['cfg_type'] = "resources"
  rv['absolute_url'] = url + "?pagetype=" + RESOURCES + "&clustername=" + cluname
  rv['Description'] = "Resource configuration for this cluster"
  if pagetype == RESOURCES or pagetype == RESOURCE_CONFIG or pagetype == RESOURCE_ADD or pagetype == RESOURCE:
    rv['show_children'] = True
  else:
    rv['show_children'] = False
  if pagetype == RESOURCES:
    rv['currentItem'] = True
  else:
    rv['currentItem'] = False

  rvadd = {}
  rvadd['Title'] = "Add a Resource"
  rvadd['cfg_type'] = "resourceadd"
  rvadd['absolute_url'] = url + "?pagetype=" + RESOURCE_ADD + "&clustername=" + cluname
  rvadd['Description'] = "Add a Resource to this cluster"
  if pagetype == RESOURCE_ADD:
    rvadd['currentItem'] = True
  else:
    rvadd['currentItem'] = False

  rvcfg = {}
  rvcfg['Title'] = "Configure a Resource"
  rvcfg['cfg_type'] = "resourcecfg"
  rvcfg['absolute_url'] = url + "?pagetype=" + RESOURCE_CONFIG + "&clustername=" + cluname
  rvcfg['Description'] = "Configure a Resource for this cluster"
  if pagetype == RESOURCE_CONFIG or pagetype == RESOURCE:
    rvcfg['show_children'] = True
  else:
    rvcfg['show_children'] = False
  if pagetype == RESOURCE_CONFIG:
    rvcfg['currentItem'] = True
  else:
    rvcfg['currentItem'] = False

  resources = model.getResources()
  resourceable = list()
  for resource in resources:
    resourcename = resource.getName()
    rvc = {}
    rvc['Title'] = resourcename
    rvc['cfg_type'] = "resource"
    rvc['absolute_url'] = url + "?pagetype=" + RESOURCE + "&resourcename=" + resourcename + "&clustername=" + cluname
    rvc['Description'] = "Configure this resource"
    if pagetype == RESOURCE:
      try:
        rname = request['resourcename']
      except KeyError, e:
        rname = ""
      if resourcename == rname:
        rvc['currentItem'] = True
      else:
        rvc['currentItem'] = False
    else:
      rvc['currentItem'] = False

    resourceable.append(rvc)
  rvcfg['children'] = resourceable



  kids = list()
  kids.append(rvadd)
  kids.append(rvcfg)
  rv['children'] = kids
 ################################################################
  fd = {}
  fd['Title'] = "Failover Domains"
  fd['cfg_type'] = "failoverdomains"
  fd['absolute_url'] = url + "?pagetype=" + FDOMS + "&clustername=" + cluname
  fd['Description'] = "Failover domain configuration for this cluster"
  if pagetype == FDOMS or pagetype == FDOM_CONFIG or pagetype == FDOM_ADD or pagetype == FDOM:
    fd['show_children'] = True
  else:
    fd['show_children'] = False
  if pagetype == FDOMS:
    fd['currentItem'] = True
  else:
    fd['currentItem'] = False

  fdadd = {}
  fdadd['Title'] = "Add a Failover Domain"
  fdadd['cfg_type'] = "failoverdomainadd"
  fdadd['absolute_url'] = url + "?pagetype=" + FDOM_ADD + "&clustername=" + cluname
  fdadd['Description'] = "Add a Failover Domain to this cluster"
  if pagetype == FDOM_ADD:
    fdadd['currentItem'] = True
  else:
    fdadd['currentItem'] = False

  fdcfg = {}
  fdcfg['Title'] = "Configure a Failover Domain"
  fdcfg['cfg_type'] = "failoverdomaincfg"
  fdcfg['absolute_url'] = url + "?pagetype=" + FDOM_CONFIG + "&clustername=" + cluname
  fdcfg['Description'] = "Configure a Failover Domain for this cluster"
  if pagetype == FDOM_CONFIG or pagetype == FDOM:
    fdcfg['show_children'] = True
  else:
    fdcfg['show_children'] = False
  if pagetype == FDOM_CONFIG:
    fdcfg['currentItem'] = True
  else:
    fdcfg['currentItem'] = False

  fdoms = model.getFailoverDomains()
  fdomable = list()
  for fdom in fdoms:
    fdomname = fdom.getName()
    fdc = {}
    fdc['Title'] = fdomname
    fdc['cfg_type'] = "fdom"
    fdc['absolute_url'] = url + "?pagetype=" + FDOM + "&fdomname=" + fdomname + "&clustername=" + cluname
    fdc['Description'] = "Configure this Failover Domain"
    if pagetype == FDOM:
      try:
        fname = request['fdomname']
      except KeyError, e:
        fname = ""
      if fdomname == fname:
        fdc['currentItem'] = True
      else:
        fdc['currentItem'] = False
    else:
      fdc['currentItem'] = False

    fdomable.append(fdc)
  fdcfg['children'] = fdomable



  kids = list()
  kids.append(fdadd)
  kids.append(fdcfg)
  fd['children'] = kids
#############################################################
  fen = {}
  fen['Title'] = "Shared Fence Devices"
  fen['cfg_type'] = "fencedevicess"
  fen['absolute_url'] = url + "?pagetype=" + FENCEDEVS + "&clustername=" + cluname
  fen['Description'] = "Fence Device configuration for this cluster"
  if pagetype == FENCEDEVS or pagetype == FENCEDEV_CONFIG or pagetype == FENCEDEV_ADD or pagetype == FENCEDEV:
    fen['show_children'] = True
  else:
    fen['show_children'] = False
  if pagetype == FENCEDEVS:
    fen['currentItem'] = True
  else:
    fen['currentItem'] = False

  fenadd = {}
  fenadd['Title'] = "Add a Fence Device"
  fenadd['cfg_type'] = "fencedeviceadd"
  fenadd['absolute_url'] = url + "?pagetype=" + FENCEDEV_ADD + "&clustername=" + cluname
  fenadd['Description'] = "Add a Fence Device to this cluster"
  if pagetype == FENCEDEV_ADD:
    fenadd['currentItem'] = True
  else:
    fenadd['currentItem'] = False

  fencfg = {}
  fencfg['Title'] = "Configure a Fence Device"
  fencfg['cfg_type'] = "fencedevicecfg"
  fencfg['absolute_url'] = url + "?pagetype=" + FENCEDEV_CONFIG + "&clustername=" + cluname
  fencfg['Description'] = "Configure a Fence Device for this cluster"
  if pagetype == FENCEDEV_CONFIG or pagetype == FENCEDEV:
    fencfg['show_children'] = True
  else:
    fencfg['show_children'] = False
  if pagetype == FENCEDEV_CONFIG:
    fencfg['currentItem'] = True
  else:
    fencfg['currentItem'] = False

  fences = model.getFenceDevices()
  fenceable = list()
  for fence in fences:
    fencename = fence.getName()
    fenc = {}
    fenc['Title'] = fencename
    fenc['cfg_type'] = "fencedevice"
    fenc['absolute_url'] = url + "?pagetype=" + FENCEDEV + "&fencename=" + fencename + "&clustername=" + cluname
    fenc['Description'] = "Configure this Fence Device"
    if pagetype == FENCEDEV:
      try:
        fenname = request['fencename']
      except KeyError, e:
        fenname = ""
      if fencename == fenname:
        fenc['currentItem'] = True
      else:
        fenc['currentItem'] = False
    else:
      fenc['currentItem'] = False

    fenceable.append(fenc)
  fencfg['children'] = fenceable



  kids = list()
  kids.append(fenadd)
  kids.append(fencfg)
  fen['children'] = kids
#############################################################

  mylist = list()
  mylist.append(nd)
  mylist.append(sv)
  mylist.append(rv)
  mylist.append(fd)
  mylist.append(fen)

  dummynode['children'] = mylist

  return dummynode


def getClusterName(self, model):
  return model.getClusterName()

def getClusterAlias(self, model):
  if not model:
    return ''
  alias = model.getClusterAlias()
  if alias is None:
    return model.getClusterName()
  else:
    return alias

def getClusterURL(self, request, model):
	try:
		clustername = request.clustername
		if not clustername:
			raise Exception, 'cluster name from request is blank'
	except:
		try:
			clustername = model.getClusterName()
			if not clustername:
				raise Exception, 'cluster name from model is blank'
		except:
			luci_log.debug_verbose('GCURL0: unable to get cluster name')
			return '.'

	return '/luci/cluster/index_html?pagetype=7&clustername=%s' % clustername

def getTabs(self, req):
  ###XXX Make this method return only tabs current user can use
  portaltabs = list()
  if not userAuthenticated(self):
    return portaltabs

  selectedtab = "homebase"
  try:
    baseurl = req['URL']
    if baseurl.find("cluster") > (-1):
      selectedtab = "cluster"
    elif baseurl.find("storage") > (-1):
      selectedtab = "storage"
    else:
      selectedtab = "homebase"
  except KeyError, e:
    selectedtab = None

  htab = { 'Title':"homebase",
           'Description':"Home base for this luci server",
           'Taburl':"/luci/homebase"}
  if selectedtab == "homebase":
    htab['isSelected'] = True
  else:
    htab['isSelected'] = False


  ctab = { 'Title':"cluster",
           'Description':"Cluster configuration page",
           'Taburl':"/luci/cluster/index_html?pagetype=3"}
  if selectedtab == "cluster":
    ctab['isSelected'] = True
  else:
    ctab['isSelected'] = False

  stab = { 'Title':"storage",
           'Description':"Storage configuration page",
           'Taburl':"/luci/storage"}
  if selectedtab == "storage":
    stab['isSelected'] = True
  else:
    stab['isSelected'] = False

  portaltabs.append(htab)
  portaltabs.append(ctab)
  portaltabs.append(stab)

  return portaltabs



def check_clusters(self, clusters):
  clist = list()
  for cluster in clusters:
    if cluster_permission_check(cluster[1]):
      clist.append(cluster)

  return clist

def cluster_permission_check(cluster):
	#Does this take too long?
	try:
		sm = AccessControl.getSecurityManager()
		user = sm.getUser()
		if user.has_permission('View', cluster):
			return True
	except:
		pass
	return False

def getRicciAgent(self, clustername):
	#Check cluster permission here! return none if false
	path = str(CLUSTER_FOLDER_PATH + clustername)

	try:
		clusterfolder = self.restrictedTraverse(path)
		if not clusterfolder:
			luci_log.debug('GRA0: cluster folder %s for %s is missing.' \
				% (path, clustername))
			raise Exception, 'no cluster folder at %s' % path
		nodes = clusterfolder.objectItems('Folder')
		if len(nodes) < 1:
			luci_log.debug('GRA1: no cluster nodes for %s found.' % clustername)
			raise Exception, 'no cluster nodes were found at %s' % path
	except Exception, e:
		try:
			luci_log.debug('GRA2: cluster folder %s for %s is missing: %s.' \
				% (path, clustername, str(e)))

			if len(clusterfolder.objectItems()) < 1:
				clusters = self.restrictedTraverse(str(CLUSTER_FOLDER_PATH))
				clusters.manage_delObjects([clustername])
		except Exception, e:
			luci_log.debug_verbose('GRA3: %s' % str(e))

		return None

	cluname = clustername.lower()

	for node in nodes:
		try:
			hostname = node[1].getId()
		except:
			try:
				hostname = node[0]
			except Exception, e:
				luci_log.debug_verbose('GRA2a: %s' % str(e))
				continue

		try:
			rc = RicciCommunicator(hostname)
			if not rc:
				raise Exception, 'rc is None'
			ricci_hostname = rc.hostname()
			if not ricci_hostname:
				raise Exception, 'ricci_hostname is blank'
		except Exception, e:
			luci_log.debug('GRA3: ricci error: %s' % str(e))
			continue

		try:
			clu_info = rc.cluster_info()
		except Exception, e:
			luci_log.debug('GRA4: cluster_info error for %s: %s' \
				% (ricci_hostname, str(e)))
			continue

		try:
			cur_name = str(clu_info[0]).strip().lower()
			if not cur_name:
				raise Exception, 'cluster name is none for %s' % ricci_hostname
		except Exception, e:
			luci_log.debug_verbose('GRA4a: %s' % str(e))
			cur_name = None

		try:
			cur_alias = str(clu_info[1]).strip().lower()
			if not cur_alias:
				raise Exception, 'cluster alias is none'
		except Exception, e:
			luci_log.debug_verbose('GRA4b: %s' % str(e))
			cur_alias = None

		if (cur_name is not None and cluname != cur_name) and (cur_alias is not None and cluname != cur_alias):
			try:
				luci_log.debug('GRA5: %s reports it\'s in cluster %s:%s; we expect %s' \
					 % (hostname, clu_info[0], clu_info[1], cluname))
				setNodeFlag(self, node, CLUSTER_NODE_NOT_MEMBER)
			except:
				pass
			continue

		try:
			if rc.authed():
				return rc

			try:
				setNodeFlag(node[1], CLUSTER_NODE_NEED_AUTH)
			except:
				pass
			raise Exception, '%s not authed' % rc.hostname()
		except Exception, e:
			luci_log.debug_verbose('GRA6: %s' % str(e))
			continue

	luci_log.debug('GRA7: no ricci agent could be found for cluster %s' \
		% cluname)
	return None

def getRicciAgentForCluster(self, req):
	clustername = None
	try:
		clustername = req['clustername']
		if not clustername:
			clustername = None
			raise
	except:
		try:
			clustername = req.form['clusterName']
			if not clustername:
				clustername = None
		except:
			pass

	if clustername is None:
		luci_log.debug('GRAFC0: no cluster name was found')
		return None
	return getRicciAgent(self, clustername)

def getClusterStatusModel(model):
	results = list()
	vals = {}

	try:
		clustername = model.getClusterName()
		clusteralias = model.getClusterAlias()
		vals['type'] = 'cluster'
		vals['alias'] = clusteralias
		vals['name'] = clustername
		vals['error'] = True
		vals['votes'] = '[unknown]'
		vals['quorate'] = '[unknown]'
		vals['minQuorum'] = '[unknown]'
		results.append(vals)
	except Exception, e:
		luci_log.debug_verbose('GCSM0: %s' % str(e))
		return None

	try:
		nodelist = model.getNodes()
	except Exception, e:
		luci_log.debug_verbose('GCSM1: %s' % str(e))
		return None

	for node in nodelist:
		node_val = {}
		node_val['type'] = 'node'
		try:
			node_name = node.getName()
			if not node_name:
				raise Exception, 'cluster node name is unknown'
		except:
			node_name = '[unknown]'

		node_val['name'] = node_name
		node_val['clustered'] = '[unknown]'
		node_val['online'] = '[unknown]'
		node_val['error'] = True

		try:
			votes = node.getVotes()
			if not votes:
				raise Exception, 'unknown unmber of votes'
		except:
			votes = '[unknown]'

		node_val['votes'] = votes
		results.append(node_val)
	return results

def getClusterStatusDB(self, clustername):
	results = list()
	vals = {}

	vals['type'] = 'cluster'
	vals['alias'] = clustername
	vals['name'] = clustername
	vals['error'] = True
	vals['quorate'] = '[unknown]'
	vals['votes'] = '[unknown]'
	vals['minQuorum'] = '[unknown]'
	results.append(vals)

	try:
		cluster_path = CLUSTER_FOLDER_PATH + clustername
		nodelist = self.restrictedTraverse(cluster_path).objectItems('Folder')
	except Exception, e:
		luci_log.debug_verbose('GCSDB0: %s -> %s: %s' \
			% (clustername, cluster_path, str(e)))
		return results

	if len(nodelist) < 1:
		luci_log.debug_verbose('GCSDB0a: removing cluster %s because it has no nodes' % clustername)
		try:
			clusters_dir = self.restrictedTraverse(CLUSTER_FOLDER_PATH)
			clusters_dir.manage_delObjects([clustername])
		except Exception, e:
			luci_log.debug_verbose('GCSDB0b: %s: %s' % (clustername, str(e)))
	else:
		for node in nodelist:
			try:
				node_val = {}
				node_val['type'] = 'node'
				node_val['name'] = node[0]
				node_val['clustered'] = '[unknown]'
				node_val['online'] = '[unknown]'
				node_val['error'] = True
				results.append(node_val)
			except Exception, e:
				luci_log.debug_verbose('GCSDB1: %s' % str(e))

	return results

def getClusterStatus(self, request, rc, cluname=None):
	try:
		doc = getClusterStatusBatch(rc)
		if not doc:
			raise Exception, 'doc is None'
	except Exception, e:
		luci_log.debug_verbose('GCS0: error: %s' % str(e))
		doc = None

	if doc is None and not cluname:
		try:
			model = request.SESSION.get('model')
			cinfo = getClusterStatusModel(model)
			if not cinfo or len(cinfo) < 1:
				raise Exception, 'cinfo is None'
			return cinfo
		except Exception, e:
			luci_log.debug_verbose('GCS1: %s' % str(e))
			doc = None

	if not doc:
		try:
			clustername = cluname
			if clustername is None:
				try:
					clustername = request['clustername']
				except:
					try:
						clustername = request.form['clustername']
					except:
						pass

			if not clustername:
				raise Exception, 'unable to determine cluster name'

			cinfo = getClusterStatusDB(self, clustername)
			if not cinfo or len(cinfo) < 1:
				raise Exception, 'cinfo is None'
			return cinfo
		except Exception, e:
			luci_log.debug_verbose('GCS1a: unable to get cluster info from DB: %s' % str(e))
		return []

	results = list()
	vals = {}
	vals['type'] = "cluster"

	try:
		vals['alias'] = doc.firstChild.getAttribute('alias')
	except AttributeError, e:
		vals['alias'] = doc.firstChild.getAttribute('name')

	vals['votes'] = doc.firstChild.getAttribute('votes')
	vals['name'] = doc.firstChild.getAttribute('name')
	vals['minQuorum'] = doc.firstChild.getAttribute('minQuorum')
	vals['quorate'] = doc.firstChild.getAttribute('quorate')
	results.append(vals)

	for node in doc.firstChild.childNodes:
		if node.nodeName == 'node':
			vals = {}
			vals['type'] = "node"
			vals['clustered'] = node.getAttribute('clustered')
			vals['name'] = node.getAttribute('name')
			vals['online'] = node.getAttribute('online')
			vals['uptime'] = node.getAttribute('uptime')
			vals['votes'] = node.getAttribute('votes')
			results.append(vals)
		elif node.nodeName == 'service':
			vals = {}
			vals['type'] = 'service'
			vals['name'] = node.getAttribute('name')
			vals['nodename'] = node.getAttribute('nodename')
			vals['running'] = node.getAttribute('running')
			try:
				vals['is_vm'] = node.getAttribute('vm').lower() == 'true'
			except:
				vals['is_vm'] = False
			vals['failed'] = node.getAttribute('failed')
			vals['autostart'] = node.getAttribute('autostart')
			results.append(vals)
	return results

def getServicesInfo(self, status, model, req):
	map = {}
	maplist = list()

	try:
		baseurl = req['URL']
		if not baseurl:
			raise KeyError, 'is blank'
	except:
		baseurl = '/luci/cluster/index_html'

	try:
		nodes = model.getNodes()
		cluname = req['clustername']
		if not cluname:
			raise KeyError, 'is blank'
	except:
		try:
			cluname = req.form['clusterName']
			if not cluname:
				raise KeyError, 'is blank'
		except:
			cluname = '[error retrieving cluster name]'

	for item in status:
		if item['type'] == "service":
			itemmap = {}
			itemmap['name'] = item['name']

			cur_node = None
			if item['running'] == "true":
				cur_node = item['nodename']
				itemmap['running'] = "true"
				itemmap['nodename'] = cur_node
				itemmap['disableurl'] = baseurl + "?" + "clustername=" + cluname +"&servicename=" + item['name'] + "&pagetype=" + SERVICE_STOP
				itemmap['restarturl'] = baseurl + "?" + "clustername=" + cluname +"&servicename=" + item['name'] + "&pagetype=" + SERVICE_RESTART
			else:
				itemmap['enableurl'] = baseurl + "?" + "clustername=" + cluname +"&servicename=" + item['name'] + "&pagetype=" + SERVICE_START

			itemmap['autostart'] = item['autostart']

			try:
				svc = model.retrieveServiceByName(item['name'])
				itemmap['cfgurl'] = baseurl + "?" + "clustername=" + cluname + "&servicename=" + item['name'] + "&pagetype=" + SERVICE
				itemmap['delurl'] = baseurl + "?" + "clustername=" + cluname + "&servicename=" + item['name'] + "&pagetype=" + SERVICE_DELETE
			except:
				try:
					svc = model.retrieveVMsByName(item['name'])
					itemmap['is_vm'] = True
					itemmap['cfgurl'] = baseurl + "?" + "clustername=" + cluname + "&servicename=" + item['name'] + "&pagetype=" + VM_CONFIG 
					itemmap['delurl'] = baseurl + "?" + "clustername=" + cluname + "&servicename=" + item['name'] + "&pagetype=" + VM_CONFIG
				except:
					continue

			starturls = list()
			for node in nodes:
				if node.getName() != cur_node:
					starturl = {}
					cur_nodename = node.getName()
					starturl['nodename'] = cur_nodename
					starturl['url'] = baseurl + '?' + 'clustername=' + cluname +'&servicename=' + item['name'] + '&pagetype=' + SERVICE_START + '&nodename=' + node.getName()
					starturls.append(starturl)

					if itemmap.has_key('is_vm') and itemmap['is_vm'] is True:
						migrate_url = { 'nodename': cur_nodename }
						migrate_url['migrate'] = True
						migrate_url['url'] = baseurl + '?' + 'clustername=' + cluname +'&servicename=' + item['name'] + '&pagetype=' + SERVICE_MIGRATE + '&nodename=' + node.getName()
						starturls.append(migrate_url)

			itemmap['links'] = starturls

			dom = svc.getAttribute("domain")
			if dom is not None:
				itemmap['faildom'] = dom
			else:
				itemmap['faildom'] = "No Failover Domain"
			maplist.append(itemmap)

	map['services'] = maplist
	return map

def get_fdom_names(model):
	return map(lambda x: x.getName(), model.getFailoverDomains())

def getServiceInfo(self, status, model, req):
	#set up struct for service config page
	hmap = {}
	root_uuid = 'toplevel'

	try:
		baseurl = req['URL']
		if not baseurl:
			raise KeyError, 'is blank'
	except:
		baseurl = '/luci/cluster/index_html'

	try:
		hmap['fdoms'] = get_fdom_names(model)
	except:
		hmap['fdoms'] = list()

	try:
		cluname = req['clustername']
		if not cluname:
			raise KeyError, 'is blank'
	except KeyError, e:
		try:
			cluname = req.form['clusterName']
			if not cluname:
				raise
		except:
			cluname = '[error retrieving cluster name]'

	hmap['root_uuid'] = root_uuid
	# uuids for the service page needed when new resources are created
	hmap['uuid_list'] = map(lambda x: make_uuid('resource'), xrange(30))

	try:
		servicename = req['servicename']
	except KeyError, e:
		hmap['resource_list'] = {}
		return hmap

	for item in status:
		innermap = {}
		if item['type'] == 'service':
			if item['name'] == servicename:
				hmap['name'] = servicename
				hmap['autostart'] = item['autostart']

				starturls = list()
				if item['running'] == 'true':
					hmap['running'] = 'true'
					nodename = item['nodename']
					innermap['current'] = 'This service is currently running on %s' % nodename

					innermap['disableurl'] = baseurl + "?" + "clustername=" + cluname +"&servicename=" + servicename + "&pagetype=" + SERVICE_STOP
					innermap['restarturl'] = baseurl + "?" + "clustername=" + cluname +"&servicename=" + servicename + "&pagetype=" + SERVICE_RESTART
					innermap['delurl'] = baseurl + "?" + "clustername=" + cluname +"&servicename=" + servicename + "&pagetype=" + SERVICE_DELETE

					#In this case, determine where it can run...
					nodes = model.getNodes()
					for node in nodes:
						if node.getName() != nodename:
							starturl = {}
							cur_nodename = node.getName()
							starturl['nodename'] = cur_nodename
							starturl['url'] = baseurl + "?" + "clustername=" + cluname +"&servicename=" + servicename + "&pagetype=" + SERVICE_START + "&nodename=" + node.getName()
							starturls.append(starturl)

							if item.has_key('is_vm') and item['is_vm'] is True:
								migrate_url = { 'nodename': cur_nodename }
								migrate_url['url'] = baseurl + "?" + "clustername=" + cluname +"&servicename=" + servicename + "&pagetype=" + SERVICE_MIGRATE + "&nodename=" + node.getName()
								migrate_url['migrate'] = True
								starturls.append(migrate_url)
					innermap['links'] = starturls
				else:
					#Do not set ['running'] in this case...ZPT will detect it is missing
					innermap['current'] = "This service is currently stopped"
					innermap['enableurl'] = baseurl + "?" + "clustername=" + cluname +"&servicename=" + servicename + "&pagetype=" + SERVICE_START
					innermap['delurl'] = baseurl + "?" + "clustername=" + cluname +"&servicename=" + servicename + "&pagetype=" + SERVICE_DELETE

					nodes = model.getNodes()
					starturls = list()
					for node in nodes:
						starturl = {}
						cur_nodename = node.getName()

						starturl['nodename'] = cur_nodename
						starturl['url'] = baseurl + "?" + "clustername=" + cluname +"&servicename=" + servicename + "&pagetype=" + SERVICE_START + "&nodename=" + node.getName()
						starturls.append(starturl)

						if item.has_key('is_vm') and item['is_vm'] is True:
							migrate_url = { 'nodename': cur_nodename }
							migrate_url['url'] = baseurl + "?" + "clustername=" + cluname +"&servicename=" + servicename + "&pagetype=" + SERVICE_MIGRATE + "&nodename=" + node.getName()
							migrate_url['migrate'] = True
							starturls.append(migrate_url)
					innermap['links'] = starturls
				hmap['innermap'] = innermap

	#Now build hashes for resources under service.
	#first get service by name from model
	svc = model.getService(servicename)
	try:
		hmap['domain'] = svc.getAttribute('domain')
	except:
		hmap['domain'] = None

	try:
		hmap['recovery'] = svc.getAttribute('recovery')
	except:
		hmap['recovery'] = None

	try:
		if int(svc.getAttribute('exclusive')):
			hmap['exclusive'] = 'true' 
		else:
			hmap['exclusive'] = 'false'
	except:
		hmap['exclusive'] = 'false'

	resource_list = list()
	if svc is not None:
		indent_ctr = 0
		children = svc.getChildren()
		for child in children:
			recurse_resources(root_uuid, child, resource_list, indent_ctr)

	hmap['resource_list'] = resource_list
	return hmap

def recurse_resources(parent_uuid, child, resource_list, indent_ctr, parent=None):
	#First, add the incoming child as a resource
	#Next, check for children of it
	#Call yourself on every children
	#then return
	rc_map = {}
	if parent is not None:
		rc_map['parent'] = parent
	rc_map['name'] = child.getName()

	#Note: Final version needs all resource attrs
	if child.isRefObject() == True:
		rc_map['ref_object'] = True
		rc_map['tag_name'] = child.getObj().TAG_NAME
		rc_map['type'] = child.getObj().getResourceType()
		rc_map['attrs'] = child.getObj().getAttributes()
	else:
		rc_map['tag_name'] = child.TAG_NAME
		rc_map['type'] = child.getResourceType()
		rc_map['attrs'] = child.getAttributes()

	rc_map['indent_ctr'] = indent_ctr

	rc_map['uuid'] = make_uuid('resource')
	rc_map['parent_uuid'] = parent_uuid

	resource_list.append(rc_map)
	kids = child.getChildren()
	child_depth = 0
	new_indent_ctr = indent_ctr + 1
	for kid in kids:
		cdepth = recurse_resources(rc_map['uuid'], kid, resource_list, new_indent_ctr, child)
		child_depth = max(cdepth, child_depth)

	rc_map['max_depth'] = child_depth
	return child_depth + 1

def serviceStart(self, rc, req):
	svcname = None
	try:
		svcname = req['servicename']
	except:
		try:
			svcname = req.form['servicename']
		except:
			pass

	if svcname is None:
		luci_log.debug_verbose('serviceStart0: no service name')
		return None

	nodename = None
	try:
		nodename = req['nodename']
	except:
		try:
			nodename = req.form['nodename']
		except:
			pass

	cluname = None
	try:
		cluname = req['clustername']
	except KeyError, e:
		try:
			cluname = req.form['clustername']
		except:
			pass

	if cluname is None:
		luci_log.debug_verbose('serviceStart2: no cluster name for svc %s' \
			% svcname)
		return None

	batch_number, result = startService(rc, svcname, nodename)
	if batch_number is None or result is None:
		luci_log.debug_verbose('serviceStart3: SS(%s,%s,%s) call failed' \
			% (svcname, cluname, nodename))
		return None

	try:
		status_msg = "Starting service \'%s\'" % svcname
		if nodename:
			status_msg += " on node \'%s\'" % nodename
		set_node_flag(self, cluname, rc.hostname(), str(batch_number), SERVICE_START, status_msg)
	except Exception, e:
		luci_log.debug_verbose('serviceStart4: error setting flags for service %s at node %s for cluster %s' % (svcname, nodename, cluname))

	response = req.RESPONSE
	response.redirect(req['URL'] + "?pagetype=" + SERVICE_LIST + "&clustername=" + cluname + '&busyfirst=true')

def serviceMigrate(self, rc, req):
	svcname = None
	try:
		svcname = req['servicename']
	except:
		try:
			svcname = req.form['servicename']
		except:
			pass

	if svcname is None:
		luci_log.debug_verbose('serviceMigrate0: no service name')
		return None

	nodename = None
	try:
		nodename = req['nodename']
	except:
		try:
			nodename = req.form['nodename']
		except:
			pass

	if nodename is None:
		luci_log.debug_verbose('serviceMigrate1: no target node name')
		return None
		
	cluname = None
	try:
		cluname = req['clustername']
	except KeyError, e:
		try:
			cluname = req.form['clustername']
		except:
			pass

	if cluname is None:
		luci_log.debug_verbose('serviceMigrate2: no cluster name for svc %s' \
			% svcname)
		return None

	batch_number, result = migrateService(rc, svcname, nodename)
	if batch_number is None or result is None:
		luci_log.debug_verbose('serviceMigrate3: SS(%s,%s,%s) call failed' \
			% (svcname, cluname, nodename))
		return None

	try:
		set_node_flag(self, cluname, rc.hostname(), str(batch_number), SERVICE_START, "Migrating service \'%s\' to node \'%s\'" % (svcname, nodename))
	except Exception, e:
		luci_log.debug_verbose('serviceMigrate4: error setting flags for service %s at node %s for cluster %s' % (svcname, nodename, cluname))

	response = req.RESPONSE
	response.redirect(req['URL'] + "?pagetype=" + SERVICE_LIST + "&clustername=" + cluname + '&busyfirst=true')

def serviceRestart(self, rc, req):
	svcname = None
	try:
		svcname = req['servicename']
	except:
		try:
			svcname = req.form['servicename']
		except:
			pass

	if svcname is None:
		luci_log.debug_verbose('serviceRestart0: no service name')
		return None

	cluname = None
	try:
		cluname = req['clustername']
	except:
		try:
			cluname = req.form['clustername']
		except:
			pass

	if cluname is None:
		luci_log.debug_verbose('serviceRestart1: no cluster for %s' % svcname)
		return None

	batch_number, result = restartService(rc, svcname)
	if batch_number is None or result is None:
		luci_log.debug_verbose('serviceRestart2: %s failed' % svcname)
		return None

	try:
		set_node_flag(self, cluname, rc.hostname(), str(batch_number), SERVICE_RESTART, "Restarting service \'%s\'" % svcname)
	except Exception, e:
		luci_log.debug_verbose('serviceRestart3: error setting flags for service %s for cluster %s' % (svcname, cluname))

	response = req.RESPONSE
	response.redirect(req['URL'] + "?pagetype=" + SERVICE_LIST + "&clustername=" + cluname + '&busyfirst=true')

def serviceStop(self, rc, req):
	svcname = None
	try:
		svcname = req['servicename']
	except:
		try:
			svcname = req.form['servicename']
		except:
			pass

	if svcname is None:
		luci_log.debug_verbose('serviceStop0: no service name')
		return None

	cluname = None
	try:
		cluname = req['clustername']
	except:
		try:
			cluname = req.form['clustername']
		except:
			pass

	if cluname is None:
		luci_log.debug_verbose('serviceStop1: no cluster name for %s' % svcname)
		return None

	batch_number, result = stopService(rc, svcname)
	if batch_number is None or result is None:
		luci_log.debug_verbose('serviceStop2: stop %s failed' % svcname)
		return None

	try:
		set_node_flag(self, cluname, rc.hostname(), str(batch_number), SERVICE_STOP, "Stopping service \'%s\'" % svcname)
	except Exception, e:
		luci_log.debug_verbose('serviceStop3: error setting flags for service %s for cluster %s' % (svcname, cluname))

	response = req.RESPONSE
	response.redirect(req['URL'] + "?pagetype=" + SERVICE_LIST + "&clustername=" + cluname + '&busyfirst=true')

def getFdomInfo(self, model, request):
	fhash = {}
	fhash['members'] = {}

	try:
		fdom = model.getFailoverDomainByName(request['fdomname'])
	except Exception, e:
		luci_log.debug_verbose('getFdomInfo0: %s' % str(e))
		return fhash

	fhash['name'] = fdom.getName()

	ordered_attr = fdom.getAttribute('ordered')
	if ordered_attr is not None and (ordered_attr == "true" or ordered_attr == "1"):
		fhash['prioritized'] = '1'
	else:
		fhash['prioritized'] = '0'

	restricted_attr = fdom.getAttribute('restricted')
	if restricted_attr is not None and (restricted_attr == "true" or restricted_attr == "1"):
		fhash['restricted'] = '1'
	else:
		fhash['restricted'] = '0'

	nodes = fdom.getChildren()
	for node in nodes:
		try:
			priority = node.getAttribute('priority')
		except:
			priority = '1'
		fhash['members'][node.getName()] = { 'priority': priority }
	return fhash

def getFdomsInfo(self, model, request, clustatus):
  slist = list()
  nlist = list()
  for item in clustatus:
    if item['type'] == "node":
      nlist.append(item)
    elif item['type'] == "service":
      slist.append(item)
  fdomlist = list()
  clustername = request['clustername']
  baseurl = request['URL']
  fdoms = model.getFailoverDomains()
  svcs = model.getServices()
  for fdom in fdoms:
    fdom_map = {}
    fdom_map['name'] = fdom.getName()
    fdom_map['cfgurl'] = baseurl + "?pagetype=" + FDOM + "&clustername=" + clustername + '&fdomname=' + fdom.getName()
    ordered_attr = fdom.getAttribute('ordered')
    restricted_attr = fdom.getAttribute('restricted')
    if ordered_attr is not None and (ordered_attr == "true" or ordered_attr == "1"):
      fdom_map['ordered'] = True
    else:
      fdom_map['ordered'] = False
    if restricted_attr is not None and (restricted_attr == "true" or restricted_attr == "1"):
      fdom_map['restricted'] = True
    else:
      fdom_map['restricted'] = False
    nodes = fdom.getChildren()
    nodelist = list()
    for node in nodes:
      nodesmap = {}
      ndname = node.getName()
      for nitem in nlist:
        if nitem['name'] == ndname:
          break
      nodesmap['nodename'] = ndname
      nodesmap['nodecfgurl'] = baseurl + "?clustername=" + clustername + "&nodename=" + ndname + "&pagetype=" + NODE
      if nitem['clustered'] == "true":
        nodesmap['status'] = NODE_ACTIVE
      elif nitem['online'] == "false":
        nodesmap['status'] = NODE_UNKNOWN
      else:
        nodesmap['status'] = NODE_INACTIVE
      priority_attr =  node.getAttribute('priority')
      if priority_attr is not None:
        nodesmap['priority'] = "0"
      nodelist.append(nodesmap)
    fdom_map['nodeslist'] = nodelist

    svclist = list()
    for svc in svcs:
      svcname = svc.getName()
      for sitem in slist:
        if sitem['name'] == svcname:
          break  #found more info about service...

      domain = svc.getAttribute("domain")
      if domain is not None:
        if domain == fdom.getName():
          svcmap = {}
          svcmap['name'] = svcname
          svcmap['status'] = sitem['running']
          svcmap['svcurl'] = baseurl + "?pagetype=" + SERVICE + "&clustername=" + clustername + "&servicename=" + svcname
          svcmap['location'] = sitem['nodename']
          svclist.append(svcmap)
    fdom_map['svclist'] = svclist
    fdomlist.append(fdom_map)
  return fdomlist

def clusterTaskProcess(self, model, request):
	try:
		task = request['task']
	except:
		try:
			task = request.form['task']
		except:
			luci_log.debug_verbose('CTP1: no task specified')
			task = None

	if not model:
		try:
			cluname = request['clustername']
			if not cluname:
				raise Exception, 'cluname is blank'
		except:
			try:
				cluname = request.form['clustername']
				if not cluname:
					raise Exception, 'cluname is blank'
			except:
				luci_log.debug_verbose('CTP0: no model/no cluster name')
				return 'Unable to determine the cluster name.'
		try:
			model = getModelForCluster(self, cluname)
		except Exception, e:
			luci_log.debug_verbose('CPT1: GMFC failed for %s' % cluname)
			model = None

	if not model:
		return 'Unable to get the model object for %s' % cluname

	redirect_page = NODES
	if task == CLUSTER_STOP:
		clusterStop(self, model)
	elif task == CLUSTER_START:
		clusterStart(self, model)
	elif task == CLUSTER_RESTART:
		clusterRestart(self, model)
	elif task == CLUSTER_DELETE:
		ret = clusterDelete(self, model)
		if ret is not None:
			redirect_page = ret
	else:
		return 'An unknown cluster task was requested.'

	response = request.RESPONSE
	response.redirect('%s?pagetype=%s&clustername=%s&busyfirst=true' \
		% (request['URL'], redirect_page, model.getClusterName()))

def getClusterInfo(self, model, req):
  try:
    cluname = req[CLUNAME]
  except:
    try:
      cluname = req.form['clustername']
    except:
      try:
        cluname = req.form['clusterName']
      except:
        luci_log.debug_verbose('GCI0: unable to determine cluster name')
        return {}

  clumap = {}
  if model is None:
    try:
      model = getModelForCluster(self, cluname)
      if not model:
        raise Exception, 'model is none'
      req.SESSION.set('model', model)
    except Exception, e:
      luci_log.debug_verbose('GCI1: unable to get model for cluster %s: %s' % (cluname, str(e)))
      return {}
  else:
    totem = model.getTotemPtr()
    if totem:
      clumap['totem'] = totem.getAttributes()

  prop_baseurl = req['URL'] + '?' + PAGETYPE + '=' + CLUSTER_CONFIG + '&' + CLUNAME + '=' + cluname + '&'
  basecluster_url = prop_baseurl + PROPERTIES_TAB + "=" + PROP_GENERAL_TAB
  #needed:
  clumap['basecluster_url'] = basecluster_url
  #name field
  clumap['clustername'] = model.getClusterAlias()
  #config version
  cp = model.getClusterPtr()
  clumap['config_version'] = cp.getConfigVersion()
  #-------------
  #new cluster params - if rhel5
  #-------------

  clumap['fence_xvmd'] = model.hasFenceXVM()
  gulm_ptr = model.getGULMPtr()
  if not gulm_ptr:
    #Fence Daemon Props
    fencedaemon_url = prop_baseurl + PROPERTIES_TAB + "=" + PROP_FENCE_TAB
    clumap['fencedaemon_url'] = fencedaemon_url
    fdp = model.getFenceDaemonPtr()
    pjd = fdp.getAttribute('post_join_delay')
    if pjd is None:
      pjd = "6"
    pfd = fdp.getAttribute('post_fail_delay')
    if pfd is None:
      pfd = "0"
    #post join delay
    clumap['pjd'] = pjd
    #post fail delay
    clumap['pfd'] = pfd

    #-------------
    #if multicast
    multicast_url = prop_baseurl + PROPERTIES_TAB + "=" + PROP_MCAST_TAB
    clumap['multicast_url'] = multicast_url
    #mcast addr
    is_mcast = model.isMulticast()
    if is_mcast:
      clumap['mcast_addr'] = model.getMcastAddr()
      clumap['is_mcast'] = "True"
    else:
      clumap['is_mcast'] = "False"
      clumap['mcast_addr'] = "1.2.3.4"
    clumap['gulm'] = False
  else:
    #-------------
    #GULM params (rhel4 only)
    lockserv_list = list()
    clunodes = model.getNodes()
    gulm_lockservs = map(lambda x: x.getName(), gulm_ptr.getChildren())
    lockserv_list = map(lambda x: (x, True), gulm_lockservs)
    for node in clunodes:
      n = node.getName()
      if not n in gulm_lockservs:
        lockserv_list.append((n, False))
    clumap['gulm'] = True
    clumap['gulm_url'] = prop_baseurl + PROPERTIES_TAB + '=' + PROP_GULM_TAB
    clumap['gulm_lockservers'] = lockserv_list

  #-------------
  #quorum disk params
  quorumd_url = prop_baseurl + PROPERTIES_TAB + "=" + PROP_QDISK_TAB
  clumap['quorumd_url'] = quorumd_url
  is_quorumd = model.isQuorumd()
  clumap['is_quorumd'] = is_quorumd
  clumap['interval'] = ""
  clumap['tko'] = ""
  clumap['votes'] = ""
  clumap['min_score'] = ""
  clumap['device'] = ""
  clumap['label'] = ""

  #list struct for heuristics...
  hlist = list()

  if is_quorumd:
    qdp = model.getQuorumdPtr()
    interval = qdp.getAttribute('interval')
    if interval is not None:
      clumap['interval'] = interval

    tko = qdp.getAttribute('tko')
    if tko is not None:
      clumap['tko'] = tko

    votes = qdp.getAttribute('votes')
    if votes is not None:
      clumap['votes'] = votes

    min_score = qdp.getAttribute('min_score')
    if min_score is not None:
      clumap['min_score'] = min_score

    device = qdp.getAttribute('device')
    if device is not None:
      clumap['device'] = device

    label = qdp.getAttribute('label')
    if label is not None:
      clumap['label'] = label

    heuristic_kids = qdp.getChildren()

    for kid in heuristic_kids:
      hmap = {}
      hprog = kid.getAttribute('program')
      if hprog is None:
        continue

      hscore = kid.getAttribute('score')
      hmap['hprog'] = hprog
      if hscore is not None:
        hmap['hscore'] = hscore
      else:
        hmap['hscore'] = ""

      hinterval = kid.getAttribute('interval')
      if hinterval is not None:
        hmap['hinterval'] = hinterval
      else:
        hmap['hinterval'] = ""
      hlist.append(hmap)
  clumap['hlist'] = hlist

  return clumap

def getClustersInfo(self, status, req):
  map = {}
  nodelist = list()
  svclist = list()
  clulist = list()
  baseurl = req['URL']

  for item in status:
    if item['type'] == "node":
      nodelist.append(item)
    elif item['type'] == "service":
      svclist.append(item)
    elif item['type'] == "cluster":
      clulist.append(item)
    else:
      continue
  if len(clulist) < 1:
    return {}
  clu = clulist[0]
  if 'error' in clu:
    map['error'] = True
  clustername = clu['name']
  if clu['alias'] != "":
    map['clusteralias'] = clu['alias']
  else:
    map['clusteralias'] = clustername
  map['clustername'] = clustername
  if clu['quorate'] == "true":
    map['status'] = "Quorate"
    map['running'] = "true"
  else:
    map['status'] = "Not Quorate"
    map['running'] = "false"
  map['votes'] = clu['votes']
  map['minquorum'] = clu['minQuorum']

  map['clucfg'] = baseurl + "?" + PAGETYPE + "=" + CLUSTER_CONFIG + "&" + CLUNAME + "=" + clustername

  map['restart_url'] = baseurl + "?" + PAGETYPE + "=" + CLUSTER_PROCESS + "&" + CLUNAME + "=" + clustername + '&task=' + CLUSTER_RESTART
  map['stop_url'] = baseurl + "?" + PAGETYPE + "=" + CLUSTER_PROCESS + "&" + CLUNAME + "=" + clustername + '&task=' + CLUSTER_STOP
  map['start_url'] = baseurl + "?" + PAGETYPE + "=" + CLUSTER_PROCESS + "&" + CLUNAME + "=" + clustername + '&task=' + CLUSTER_START
  map['delete_url'] = baseurl + "?" + PAGETYPE + "=" + CLUSTER_PROCESS + "&" + CLUNAME + "=" + clustername + '&task=' + CLUSTER_DELETE

  svc_dict_list = list()
  for svc in svclist:
      svc_dict = {}
      svc_dict['nodename'] = svc['nodename']
      svcname = svc['name']
      svc_dict['name'] = svcname
      svc_dict['srunning'] = svc['running']

      if svc.has_key('is_vm') and svc['is_vm'] is True:
        target_page = VM_CONFIG
      else:
        target_page = SERVICE
      svcurl = baseurl + "?" + PAGETYPE + "=" + target_page + "&" + CLUNAME + "=" + clustername + "&servicename=" + svcname
      svc_dict['servicename'] = svcname
      svc_dict['svcurl'] = svcurl
      svc_dict_list.append(svc_dict)
  map['currentservices'] = svc_dict_list
  node_dict_list = list()

  for item in nodelist:
    nmap = {}
    name = item['name']
    nmap['nodename'] = name
    cfgurl = baseurl + "?" + PAGETYPE + "=" + NODE + "&" + CLUNAME + "=" + clustername + "&nodename=" + name
    nmap['configurl'] = cfgurl
    if item['clustered'] == "true":
      nmap['status'] = NODE_ACTIVE
    elif item['online'] == "false":
      nmap['status'] = NODE_UNKNOWN
    else:
      nmap['status'] = NODE_INACTIVE
    node_dict_list.append(nmap)

  map['currentnodes'] = node_dict_list
  return map

def nodeLeave(self, rc, clustername, nodename_resolved):
	path = str(CLUSTER_FOLDER_PATH + clustername + '/' + nodename_resolved)

	try:
		nodefolder = self.restrictedTraverse(path)
		if not nodefolder:
			raise Exception, 'cannot find database object at %s' % path
	except Exception, e:
		luci_log.debug('NLO: node_leave_cluster err: %s' % str(e))
		return None

	objname = str(nodename_resolved + "____flag")
	fnpresent = noNodeFlagsPresent(self, nodefolder, objname, nodename_resolved)

	if fnpresent is None:
		luci_log.debug('NL1: An error occurred while checking flags for %s' \
			% nodename_resolved)
		return None

	if fnpresent == False:
		luci_log.debug('NL2: flags are still present for %s -- bailing out' \
			% nodename_resolved)
		return None

	batch_number, result = nodeLeaveCluster(rc)
	if batch_number is None or result is None:
		luci_log.debug_verbose('NL3: nodeLeaveCluster error: batch_number and/or result is None')
		return None

	try:
		set_node_flag(self, clustername, rc.hostname(), str(batch_number), NODE_LEAVE_CLUSTER, "Node \'%s\' leaving cluster" % nodename_resolved)
	except Exception, e:
		luci_log.debug_verbose('NL4: failed to set flags: %s' % str(e))
	return True

def nodeJoin(self, rc, clustername, nodename_resolved):
	batch_number, result = nodeJoinCluster(rc)
	if batch_number is None or result is None:
		luci_log.debug_verbose('NJ0: batch_number and/or result is None')
		return None

	try:
		set_node_flag(self, clustername, rc.hostname(), str(batch_number), NODE_JOIN_CLUSTER, "Node \'%s\' joining cluster" % nodename_resolved)
	except Exception, e:
		luci_log.debug_verbose('NJ1: failed to set flags: %s' % str(e))
	return True

def clusterStart(self, model):
	if model is None:
		return None

	clustername = model.getClusterName()
	nodes = model.getNodes()
	if not nodes or len(nodes) < 1:
		return None

	errors = 0
	for node in nodes:
		nodename = node.getName().strip()
		nodename_resolved = resolve_nodename(self, clustername, nodename)

		try:
			rc = RicciCommunicator(nodename_resolved)
		except Exception, e:
			luci_log.debug_verbose('CStart: RC %s: %s' \
				% (nodename_resolved, str(e)))
			errors += 1
			continue
		if nodeJoin(self, rc, clustername, nodename_resolved) is None:
			luci_log.debug_verbose('CStart1: nodeJoin %s' % nodename_resolved)
			errors += 1

	return errors

def clusterStop(self, model, delete=False):
	if model is None:
		return None

	clustername = model.getClusterName()
	nodes = model.getNodes()
	if not nodes or len(nodes) < 1:
		return None

	errors = 0
	for node in nodes:
		nodename = node.getName().strip()
		nodename_resolved = resolve_nodename(self, clustername, nodename)

		try:
			rc = RicciCommunicator(nodename_resolved)
		except Exception, e:
			luci_log.debug_verbose('CStop0: [%d] RC %s: %s' \
				% (delete is True, str(nodename_resolved), str(e)))
			errors += 1
			continue

		if delete is True:
			if nodeDelete(self, rc, model, clustername, nodename, nodename_resolved, delete_cluster=True) is None:
				luci_log.debug_verbose('CStop1: [1] nodeDelete failed')
				errors += 1
		else:
			if nodeLeave(self, rc, clustername, nodename_resolved) is None:
				luci_log.debug_verbose('CStop2: [0] nodeLeave %s' \
					% (nodename_resolved))
				errors += 1
	return errors

def clusterRestart(self, model):
	snum_err = clusterStop(self, model)
	if snum_err:
		luci_log.debug_verbose('cluRestart0: clusterStop: %d errs' % snum_err)
	jnum_err = clusterStart(self, model)
	if jnum_err:
		luci_log.debug_verbose('cluRestart0: clusterStart: %d errs' % jnum_err)
	return snum_err + jnum_err

def clusterDelete(self, model):
	num_errors = clusterStop(self, model, delete=True)
	try:
		clustername = model.getClusterName()
	except Exception, e:
		luci_log.debug_verbose('clusterDelete0: unable to get cluster name')
		return None

	if num_errors < 1:
		try:
			delCluster(self, clustername)
		except Exception, e:
			luci_log.debug_verbose('clusterDelete1: %s: %s' \
				% (clustername, str(e)))

		try:
			clusterfolder = self.restrictedTraverse(str(CLUSTER_FOLDER_PATH + clustername))
			if len(clusterfolder.objectItems()) < 1:
				clusters = self.restrictedTraverse(str(CLUSTER_FOLDER_PATH))
				clusters.manage_delObjects([clustername])
		except Exception, e:
			luci_log.debug_verbose('clusterDelete2: %s %s' \
				% (clustername, str(e)))
		return CLUSTERLIST
	else:
		luci_log.debug_verbose('clusterDelete2: %s: %d errors' \
			% (clustername, num_errors))

def forceNodeReboot(self, rc, clustername, nodename_resolved):
	batch_number, result = nodeReboot(rc)
	if batch_number is None or result is None:
		luci_log.debug_verbose('FNR0: batch_number and/or result is None')
		return None

	try:
		set_node_flag(self, clustername, rc.hostname(), str(batch_number), NODE_REBOOT, "Node \'%s\' is being rebooted" % nodename_resolved)
	except Exception, e:
		luci_log.debug_verbose('FNR1: failed to set flags: %s' % str(e))
	return True

def forceNodeFence(self, clustername, nodename, nodename_resolved):
	path = str(CLUSTER_FOLDER_PATH + clustername)

	try:
		clusterfolder = self.restrictedTraverse(path)
		if not clusterfolder:
			raise Exception, 'no cluster folder at %s' % path
	except Exception, e:
		luci_log.debug('FNF0: The cluster folder %s could not be found: %s' \
			 % (clustername, str(e)))
		return None

	try:
		nodes = clusterfolder.objectItems('Folder')
		if not nodes or len(nodes) < 1:
			raise Exception, 'no cluster nodes'
	except Exception, e:
		luci_log.debug('FNF1: No cluster nodes for %s were found: %s' \
			% (clustername, str(e)))
		return None

	found_one = False
	for node in nodes:
		if node[1].getId().find(nodename) != (-1):
			continue

		try:
			rc = RicciCommunicator(node[1].getId())
			if not rc:
				raise Exception, 'rc is None'
		except Exception, e:
			luci_log.debug('FNF2: ricci error for host %s: %s' \
				% (node[0], str(e)))
			continue

		if not rc.authed():
			rc = None
			try:
				snode = getStorageNode(self, node[1].getId())
				setNodeFlag(snode, CLUSTER_NODE_NEED_AUTH)
			except:
				pass

			try:
				setNodeFlag(node[1], CLUSTER_NODE_NEED_AUTH)
			except:
				pass

			continue
		found_one = True
		break

	if not found_one:
		return None

	batch_number, result = nodeFence(rc, nodename)
	if batch_number is None or result is None:
		luci_log.debug_verbose('FNF3: batch_number and/or result is None')
		return None

	try:
		set_node_flag(self, clustername, rc.hostname(), str(batch_number), NODE_FENCE, "Node \'%s\' is being fenced" % nodename_resolved)
	except Exception, e:
		luci_log.debug_verbose('FNF4: failed to set flags: %s' % str(e))
	return True

def nodeDelete(self, rc, model, clustername, nodename, nodename_resolved, delete_cluster=False):
	# We need to get a node name other than the node
	# to be deleted, then delete the node from the cluster.conf
	# and propogate it. We will need two ricci agents for this task,
	# unless we are deleting the cluster itself.

	if not delete_cluster:
		# Make sure we can find a second node before we hose anything.
		found_one = False

		path = str(CLUSTER_FOLDER_PATH + clustername)

		try:
			clusterfolder = self.restrictedTraverse(path)
			if not clusterfolder:
				raise Exception, 'no cluster folder at %s' % path
		except Exception, e:
			luci_log.debug_verbose('ND0: node delete error for cluster %s: %s' \
				% (clustername, str(e)))
			return None

		try:
			nodes = clusterfolder.objectItems('Folder')
			if not nodes or len(nodes) < 1:
				raise Exception, 'no cluster nodes in DB'
		except Exception, e:
			luci_log.debug_verbose('ND1: node delete error for cluster %s: %s' \
				% (clustername, str(e)))

		for node in nodes:
			if node[1].getId().find(nodename) != (-1):
				continue
			# here we make certain the node is up...
			# XXX- we should also make certain this host is still
			# in the cluster we believe it is.

			try:
				rc2 = RicciCommunicator(node[1].getId())
				if not rc2:
					raise Exception, 'ND1a: rc2 is None'
			except Exception, e:
				luci_log.info('ND2: ricci %s error: %s' % (node[0], str(e)))
				continue

			if not rc2.authed():
				try:
					setNodeFlag(node[1], CLUSTER_NODE_NEED_AUTH)
				except:
					pass

				try:
					snode = getStorageNode(self, node[0])
					setNodeFlag(snode, CLUSTER_NODE_NEED_AUTH)
				except:
					pass

				luci_log.debug_verbose('ND3: %s is not authed' % node[0])
				rc2 = None
				continue
			else:
				found_one = True
				break

		if not found_one:
			luci_log.debug_verbose('ND4: unable to find ricci agent to delete %s from %s' % (nodename, clustername))
			return None

	# First, delete cluster.conf from node to be deleted.
	# next, have node leave cluster.
	batch_number, result = nodeLeaveCluster(rc, purge=True)
	if batch_number is None or result is None:
		luci_log.debug_verbose('ND5: batch_number and/or result is None')
		return None

	# Unless we're deleting the whole cluster, it is not worth
	# flagging this node in DB, as we are going to delete it
	# anyway. Now, we need to delete node from model and send out
	# new cluster.conf

	if delete_cluster:
		try:
			set_node_flag(self, clustername, rc.hostname(), str(batch_number), CLUSTER_DELETE, "Deleting cluster \"%s\": Deleting node \'%s\'" \
				% (clustername, nodename_resolved))
		except Exception, e:
			luci_log.debug_verbose('ND5a: failed to set flags: %s' % str(e))
	else:
		delete_target = None
		nodelist = model.getNodes()
		find_node = nodename.lower()
		for n in nodelist:
			try:
				if n.getName().lower() == find_node:
					delete_target = n
					break
			except:
				continue

		if delete_target is None:
			luci_log.debug_verbose('ND6: no delete target for %s in cluster %s' \
				% (nodename, clustername))
			return None

		try:
			model.deleteNode(delete_target)
		except Exception, e:
			luci_log.debug_verbose('ND6a: deleteNode %s failed: %s' \
				% (delete_target.getName(), str(e)))

		try:
			model.setModified(True)
			str_buf = model.exportModelAsString()
			if not str_buf:
				raise Exception, 'model string is blank'
		except Exception, e:
			luci_log.debug_verbose('ND7: exportModelAsString: %s' % str(e))
			return None

		# propagate the new cluster.conf via the second node
		batch_number, result = setClusterConf(rc2, str(str_buf))
		if batch_number is None:
			luci_log.debug_verbose('ND8: batch number is None after del node in NTP')
			return None

	# Now we need to delete the node from the DB
	path = str(CLUSTER_FOLDER_PATH + clustername)
	try:
		clusterfolder = self.restrictedTraverse(path)
		clusterfolder.manage_delObjects([nodename_resolved])
	except Exception, e:
		luci_log.debug_verbose('ND9: error deleting %s at %s: %s' \
			% (nodename_resolved, path, str(e)))

	if delete_cluster:
		return True

	try:
		set_node_flag(self, clustername, rc2.hostname(), str(batch_number), NODE_DELETE, "Deleting node \'%s\'" % nodename_resolved)
	except Exception, e:
		luci_log.debug_verbose('ND10: failed to set flags: %s' % str(e))
	return True

def nodeTaskProcess(self, model, request):
	try:
		clustername = request['clustername']
	except:
		try:
			clustername = request.form['clustername']
		except:
			luci_log.debug('NTP0: missing cluster name')
			return (False, {'errors': [ 'No cluster name was given.' ]})

	try:
		nodename = request['nodename']
	except:
		try:
			nodename = request.form['nodename']
		except:
			luci_log.debug('NTP1: missing node name')
			return (False, {'errors': [ 'No node name was given.' ]})

	try:
		task = request['task']
	except:
		try:
			task = request.form['task']
		except:
			luci_log.debug('NTP2: missing task')
			return (False, {'errors': [ 'No node task was given.' ]})

	nodename_resolved = resolve_nodename(self, clustername, nodename)

	if task != NODE_FENCE:
		# Fencing is the only task for which we don't
		# want to talk to the node on which the action is
		# to be performed.
		try:
			rc = RicciCommunicator(nodename_resolved)
			if not rc:
				raise Exception, 'rc is None'
		except RicciError, e:
			luci_log.debug('NTP3: ricci error from %s: %s' \
				% (nodename_resolved, str(e)))
			return (False, {'errors': [ 'Unable to connect to the ricci agent on %s.' % nodename_resolved ]})
		except:
			luci_log.debug('NTP4: ricci error from %s: %s' \
				% (nodename_resolved, str(e)))
			return (False, {'errors': [ 'Unable to connect to the ricci agent on %s.' % nodename_resolved ]})

		cluinfo = rc.cluster_info()
		if not cluinfo[0] and not cluinfo[1]:
			luci_log.debug('NTP5: node %s not in a cluster (expected %s)' \
				% (nodename_resolved, clustername))
			return (False, {'errors': [ 'Node %s reports it is not in a cluster.' % nodename_resolved ]})

		cname = clustername.lower()
		if cname != cluinfo[0].lower() and cname != cluinfo[1].lower():
			luci_log.debug('NTP6: node %s in unknown cluster %s:%s (expected %s)' % (nodename_resolved, cluinfo[0], cluinfo[1], clustername))
			return (False, {'errors': [ 'Node %s reports it in cluster \"%s\". We expect it to be a member of cluster \"%s\"' % (nodename_resolved, cluinfo[0], clustername) ]})

		if not rc.authed():
			rc = None
			try:
				snode = getStorageNode(self, nodename)
				setNodeFlag(snode, CLUSTER_NODE_NEED_AUTH)
			except:
				# we'll hit it again, and try again then
				pass

			try:
				cnode = getClusterNode(self, nodename, clustername)
				setNodeFlag(cnode, CLUSTER_NODE_NEED_AUTH)
			except:
				# we'll hit it again, and try again then
				pass

		if rc is None:
			luci_log.debug('NTP7: node %s is not authenticated' \
				% nodename_resolved)
			return (False, {'errors': [ 'Node %s is not authenticated' % nodename_resolved ]})

	if task == NODE_LEAVE_CLUSTER:
		if nodeLeave(self, rc, clustername, nodename_resolved) is None:
			luci_log.debug_verbose('NTP8: nodeLeave failed')
			return (False, {'errors': [ 'Node %s failed to leave cluster %s' % (nodename_resolved, clustername) ]})

		response = request.RESPONSE
		response.redirect(request['URL'] + "?pagetype=" + NODES + "&clustername=" + clustername + '&busyfirst=true')
	elif task == NODE_JOIN_CLUSTER:
		if nodeJoin(self, rc, clustername, nodename_resolved) is None:
			luci_log.debug_verbose('NTP9: nodeJoin failed')
			return (False, {'errors': [ 'Node %s failed to join cluster %s' % (nodename_resolved, clustername) ]})

		response = request.RESPONSE
		response.redirect(request['URL'] + "?pagetype=" + NODES + "&clustername=" + clustername + '&busyfirst=true')
	elif task == NODE_REBOOT:
		if forceNodeReboot(self, rc, clustername, nodename_resolved) is None:
			luci_log.debug_verbose('NTP10: nodeReboot failed')
			return (False, {'errors': [ 'Node %s failed to reboot' \
				% nodename_resolved ]})

		response = request.RESPONSE
		response.redirect(request['URL'] + "?pagetype=" + NODES + "&clustername=" + clustername + '&busyfirst=true')
	elif task == NODE_FENCE:
		if forceNodeFence(self, clustername, nodename, nodename_resolved) is None:
			luci_log.debug_verbose('NTP11: nodeFencefailed')
			return (False, {'errors': [ 'Fencing of node %s failed.' \
				% nodename_resolved]})

		response = request.RESPONSE
		response.redirect(request['URL'] + "?pagetype=" + NODES + "&clustername=" + clustername + '&busyfirst=true')
	elif task == NODE_DELETE:
		if nodeDelete(self, rc, model, clustername, nodename, nodename_resolved) is None:
			luci_log.debug_verbose('NTP12: nodeDelete failed')
			return (False, {'errors': [ 'Deletion of node %s from cluster %s failed.' % (nodename_resolved, clustername) ]})

		response = request.RESPONSE
		response.redirect(request['URL'] + "?pagetype=" + NODES + "&clustername=" + clustername + '&busyfirst=true')

def getNodeInfo(self, model, status, request):
  infohash = {}
  item = None
  baseurl = request['URL']
  nodestate = NODE_ACTIVE
  svclist = list()
  for thing in status:
    if thing['type'] == "service":
      svclist.append(thing)

  #Get cluster name and node name from request
  try:
    clustername = request['clustername']
    nodename = request['nodename']
  except Exception, e:
    luci_log.debug_verbose('getNodeInfo0: %s' % str(e))
    return {}

  #extract correct node line from cluster status
  found = False
  for item in status:
    if (item['type'] == "node") and (item['name'] == nodename):
      found = True
      break
  if found == False:
    luci_log.debug_verbose('getNodeInfo1: Unable to resolve node name in cluster status')
    return {}

  #Now determine state of node...
  if item['online'] == "false":
    nodestate = NODE_UNKNOWN
  elif item['clustered'] == "true":
    nodestate = NODE_ACTIVE
  else:
    nodestate = NODE_INACTIVE

  infohash['nodestate'] = nodestate
  infohash['nodename'] = nodename

  #set up drop down links
  if nodestate == NODE_ACTIVE:
    infohash['jl_url'] = baseurl + "?pagetype=" + NODE_PROCESS + "&task=" + NODE_LEAVE_CLUSTER + "&nodename=" + nodename + "&clustername=" + clustername
    infohash['reboot_url'] = baseurl + "?pagetype=" +NODE_PROCESS + "&task=" + NODE_REBOOT + "&nodename=" + nodename + "&clustername=" + clustername
    infohash['fence_url'] = baseurl + "?pagetype=" + NODE_PROCESS + "&task=" + NODE_FENCE + "&nodename=" + nodename + "&clustername=" + clustername
    infohash['delete_url'] = baseurl + "?pagetype=" + NODE_PROCESS + "&task=" + NODE_DELETE + "&nodename=" + nodename + "&clustername=" + clustername
  elif nodestate == NODE_INACTIVE:
    infohash['jl_url'] = baseurl + "?pagetype=" + NODE_PROCESS + "&task=" + NODE_JOIN_CLUSTER + "&nodename=" + nodename + "&clustername=" + clustername
    infohash['reboot_url'] = baseurl + "?pagetype=" + NODE_PROCESS + "&task=" + NODE_REBOOT + "&nodename=" + nodename + "&clustername=" + clustername
    infohash['fence_url'] = baseurl + "?pagetype=" + NODE_PROCESS + "&task=" + NODE_FENCE + "&nodename=" + nodename + "&clustername=" + clustername
    infohash['delete_url'] = baseurl + "?pagetype=" + NODE_PROCESS + "&task=" + NODE_DELETE + "&nodename=" + nodename + "&clustername=" + clustername
  else:
    infohash['fence_url'] = baseurl + "?pagetype=" + NODE_PROCESS + "&task=" + NODE_FENCE + "&nodename=" + nodename + "&clustername=" + clustername

  #figure out current services running on this node
  svc_dict_list = list()
  for svc in svclist:
    if svc['nodename'] == nodename:
      svc_dict = {}
      svcname = svc['name']
      svcurl = baseurl + "?" + PAGETYPE + "=" + SERVICE + "&" + CLUNAME + "=" + clustername + "&servicename=" + svcname
      svc_dict['servicename'] = svcname
      svc_dict['svcurl'] = svcurl
      svc_dict_list.append(svc_dict)

  infohash['currentservices'] = svc_dict_list

  fdom_dict_list = list()
  gulm_cluster = False
  if model:
    gulm_cluster = model.getGULMPtr() is not None
    try:
      infohash['gulm_lockserver'] = model.isNodeLockserver(nodename)
    except:
      infohash['gulm_lockserver'] = False
    #next is faildoms
    fdoms = model.getFailoverDomainsForNode(nodename)
    for fdom in fdoms:
      fdom_dict = {}
      fdom_dict['name'] = fdom.getName()
      fdomurl = baseurl + "?" + PAGETYPE + "=" + FDOM_CONFIG + "&" + CLUNAME + "=" + clustername + "&fdomname=" + fdom.getName()
      fdom_dict['fdomurl'] = fdomurl
      fdom_dict_list.append(fdom_dict)
  else:
    infohash['gulm_lockserver'] = False

  infohash['fdoms'] = fdom_dict_list

  #return infohash
  infohash['d_states'] = None

  nodename_resolved = resolve_nodename(self, clustername, nodename)

  if nodestate == NODE_ACTIVE or nodestate == NODE_INACTIVE:
  #call service module on node and find out which daemons are running
    try:
      rc = RicciCommunicator(nodename_resolved)
      if not rc:
        raise Exception, 'rc is none'
    except Exception, e:
      rc = None
      infohash['ricci_error'] = True
      luci_log.info('Error connecting to %s: %s' \
          % (nodename_resolved, str(e)))

    if rc is not None:
      dlist = list()
      dlist.append("ccsd")
      if not gulm_cluster:
        dlist.append("cman")
        dlist.append("fenced")
      else:
        dlist.append("lock_gulmd")
      dlist.append("rgmanager")
      states = getDaemonStates(rc, dlist)
      infohash['d_states'] = states
  else:
    infohash['ricci_error'] = True

  infohash['logurl'] = '/luci/logs/?nodename=' + nodename_resolved + '&clustername=' + clustername
  return infohash

def getNodesInfo(self, model, status, req):
  resultlist = list()
  nodelist = list()
  svclist = list()

  #Sort into lists...
  for item in status:
    if item['type'] == "node":
      nodelist.append(item)
    elif item['type'] == "service":
      svclist.append(item)
    else:
      continue

  try:
    clustername = req['clustername']
    if not clustername:
      raise KeyError, 'clustername is blank'
  except:
    try:
      clustername = req.form['clustername']
      raise KeyError, 'clustername is blank'
    except:
      try:
        clustername = req.form['clusterName']
      except:
        try:
          clustername = model.getClusterName()
        except:
          luci_log.debug_verbose('GNI0: unable to determine cluster name')
          return {}

  for item in nodelist:
    map = {}
    name = item['name']
    map['nodename'] = name
    try:
      map['gulm_lockserver'] = model.isNodeLockserver(name)
    except:
      map['gulm_lockserver'] = False

    try:
      baseurl = req['URL']
    except:
      baseurl = '/luci/cluster/index_html'

    cfgurl = baseurl + "?" + PAGETYPE + "=" + NODE + "&" + CLUNAME + "=" + clustername + "&nodename=" + name

    map['configurl'] = cfgurl
    map['fenceurl'] = cfgurl + "#fence"
    if item['clustered'] == "true":
      map['status'] = NODE_ACTIVE
      map['status_str'] = NODE_ACTIVE_STR
    elif item['online'] == "false":
      map['status'] = NODE_UNKNOWN
      map['status_str'] = NODE_UNKNOWN_STR
    else:
      map['status'] = NODE_INACTIVE
      map['status_str'] = NODE_INACTIVE_STR

    nodename_resolved = resolve_nodename(self, clustername, name)

    map['logurl'] = '/luci/logs?nodename=' + nodename_resolved + '&clustername=' + clustername

    #set up URLs for dropdown menu...
    if map['status'] == NODE_ACTIVE:
      map['jl_url'] = baseurl + "?pagetype=" + NODE_PROCESS + "&task=" + NODE_LEAVE_CLUSTER + "&nodename=" + name + "&clustername=" + clustername
      map['reboot_url'] = baseurl + "?pagetype=" +NODE_PROCESS + "&task=" + NODE_REBOOT + "&nodename=" + name + "&clustername=" + clustername
      map['fence_it_url'] = baseurl + "?pagetype=" + NODE_PROCESS + "&task=" + NODE_FENCE + "&nodename=" + name + "&clustername=" + clustername
      map['delete_url'] = baseurl + "?pagetype=" + NODE_PROCESS + "&task=" + NODE_DELETE + "&nodename=" + name + "&clustername=" + clustername
    elif map['status'] == NODE_INACTIVE:
      map['jl_url'] = baseurl + "?pagetype=" + NODE_PROCESS + "&task=" + NODE_JOIN_CLUSTER + "&nodename=" + name + "&clustername=" + clustername
      map['reboot_url'] = baseurl + "?pagetype=" + NODE_PROCESS + "&task=" + NODE_REBOOT + "&nodename=" + name + "&clustername=" + clustername
      map['fence_it_url'] = baseurl + "?pagetype=" + NODE_PROCESS + "&task=" + NODE_FENCE + "&nodename=" + name + "&clustername=" + clustername
      map['delete_url'] = baseurl + "?pagetype=" + NODE_PROCESS + "&task=" + NODE_DELETE + "&nodename=" + name + "&clustername=" + clustername
    else:
      map['fence_it_url'] = baseurl + "?pagetype=" + NODE_PROCESS + "&task=" + NODE_FENCE + "&nodename=" + name + "&clustername=" + clustername

    #figure out current services running on this node
    svc_dict_list = list()
    for svc in svclist:
      if svc['nodename'] == name:
        svc_dict = {}
        svcname = svc['name']
        svcurl = baseurl + "?" + PAGETYPE + "=" + SERVICE + "&" + CLUNAME + "=" + clustername + "&servicename=" + svcname
        svc_dict['servicename'] = svcname
        svc_dict['svcurl'] = svcurl
        svc_dict_list.append(svc_dict)

    map['currentservices'] = svc_dict_list
    #next is faildoms

    if model:
      fdoms = model.getFailoverDomainsForNode(name)
    else:
      map['ricci_error'] = True
      fdoms = list()
    fdom_dict_list = list()
    for fdom in fdoms:
      fdom_dict = {}
      fdom_dict['name'] = fdom.getName()
      fdomurl = baseurl + "?" + PAGETYPE + "=" + FDOM_CONFIG + "&" + CLUNAME + "=" + clustername + "&fdomname=" + fdom.getName()
      fdom_dict['fdomurl'] = fdomurl
      fdom_dict_list.append(fdom_dict)

    map['fdoms'] = fdom_dict_list
    resultlist.append(map)

  return resultlist

def getFence(self, model, request):
  if not model:
    luci_log.debug_verbose('getFence0: model is None')
    return {}

  map = {}
  fencename = request['fencename']
  fencedevs = model.getFenceDevices()
  for fencedev in fencedevs:
    if fencedev.getName().strip() == fencename:
      map = fencedev.getAttributes()
      try:
        map['pretty_name'] = FENCE_OPTS[fencedev.getAgentType()]
      except:
        map['unknown'] = True
        map['pretty_name'] = fencedev.getAgentType()

      nodes_used = list()
      nodes = model.getNodes()
      for node in nodes:
        flevels = node.getFenceLevels()
        for flevel in flevels: #These are the method blocks...
          kids = flevel.getChildren()
          for kid in kids: #These are actual devices in each level
            if kid.getName().strip() == fencedev.getName().strip():
              #See if this fd already has an entry for this node
              found_duplicate = False
              for item in nodes_used:
                if item['nodename'] == node.getName().strip():
                  found_duplicate = True
              if found_duplicate == True:
                continue
              baseurl = request['URL']
              clustername = model.getClusterName()
              node_hash = {}
              node_hash['nodename'] = node.getName().strip()
              node_hash['nodeurl'] = baseurl + "?clustername=" + clustername + "&nodename=" + node.getName() + "&pagetype=" + NODE
              nodes_used.append(node_hash)

      map['nodesused'] = nodes_used
      return map

  return map

def getFDForInstance(fds, name):
  for fd in fds:
    if fd.getName().strip() == name:
      return fd

  raise

def getFenceInfo(self, model, request):
  if not model:
    luci_log.debug_verbose('getFenceInfo00: model is None')
    return {}

  try:
    clustername = request['clustername']
  except:
    try:
      clustername = request.form['clustername']
    except:
      luci_log.debug_verbose('getFenceInfo0: unable to determine cluster name')
      return {}

  try:
    baseurl = request['URL']
  except Exception, e:
    luci_log.debug_verbose('getFenceInfo1: no request.URL')
    return {}

  map = {}
  level1 = list() #First level fence devices
  level2 = list() #Second level fence devices
  shared1 = list() #List of available sharable fence devs not used in level1
  shared2 = list() #List of available sharable fence devs not used in level2
  map['level1'] = level1
  map['level2'] = level2
  map['shared1'] = shared1
  map['shared2'] = shared2

  major_num = 1
  minor_num = 100

  try:
    nodename = request['nodename']
  except:
    try:
      nodename = request.form['nodename']
    except:
      luci_log.debug_verbose('getFenceInfo2: unable to extract nodename: %s' \
          % str(e))
      return {}

  #Here we need to get fences for a node - just the first two levels
  #Each level has its own list of fence devs used in that level
  #For each fence dev, a list of instance structs is appended
  #In addition, for each level, a list of available but unused fence devs
  #is returned.
  try:
    node = model.retrieveNodeByName(nodename)
  except GeneralError, e:
    luci_log.debug_verbose('getFenceInfo3: unabel to find node name %s in current node list' % (str(nodename), str(e)))
    return {}

  fds = model.getFenceDevices()

  levels = node.getFenceLevels()
  len_levels = len(levels)

  if len_levels == 0:
    return map

  if len_levels >= 1:
    first_level = levels[0]
    kids = first_level.getChildren()
    last_kid_fd = None  #This is a marker for allowing multi instances
                        #beneath a fencedev
    for kid in kids:
      instance_name = kid.getName().strip()
      try:
        fd = getFDForInstance(fds, instance_name)
      except:
        fd = None #Set to None in case last time thru loop
        continue

      if fd is not None:
        if fd.isShared() == False:  #Not a shared dev...build struct and add
          fencedev = {}
          try:
            fencedev['prettyname'] = FENCE_OPTS[fd.getAgentType()]
          except:
            fencedev['unknown'] = True
            fencedev['prettyname'] = fd.getAgentType()
          fencedev['isShared'] = False
          fencedev['id'] = str(major_num)
          major_num = major_num + 1
          devattrs = fd.getAttributes()
          kees = devattrs.keys()
          for kee in kees:
            fencedev[kee] = devattrs[kee]
          kidattrs = kid.getAttributes()
          kees = kidattrs.keys()
          for kee in kees:
            if kee == "name":
              continue #Don't duplicate name attr
            fencedev[kee] = kidattrs[kee]
          #This fencedev struct is complete, and needs to be placed on the
          #level1 Q. Because it is non-shared, we should set last_kid_fd
          #to none.
          last_kid_fd = None
          level1.append(fencedev)
        else:  #This dev is shared
          if (last_kid_fd is not None) and (fd.getName().strip() == last_kid_fd['name'].strip()):  #just append a new instance struct to last_kid_fd
            instance_struct = {}
            instance_struct['id'] = str(minor_num)
            minor_num = minor_num + 1
            kidattrs = kid.getAttributes()
            kees = kidattrs.keys()
            for kee in kees:
              if kee == "name":
                continue
              instance_struct[kee] = kidattrs[kee]
            #Now just add this struct to last_kid_fd and reset last_kid_fd
            ilist = last_kid_fd['instance_list']
            ilist.append(instance_struct)
            #last_kid_fd = fd
            continue
          else: #Shared, but not used above...so we need a new fencedev struct
            fencedev = {}
            try:
              fencedev['prettyname'] = FENCE_OPTS[fd.getAgentType()]
            except:
              fencedev['unknown'] = True
              fencedev['prettyname'] = fd.getAgentType()
            fencedev['isShared'] = True
            fencedev['cfgurl'] = baseurl + "?clustername=" + clustername + "&fencename=" + fd.getName().strip() + "&pagetype=" + FENCEDEV
            fencedev['id'] = str(major_num)
            major_num = major_num + 1
            inlist = list()
            fencedev['instance_list'] = inlist
            devattrs = fd.getAttributes()
            kees = devattrs.keys()
            for kee in kees:
              fencedev[kee] = devattrs[kee]
            instance_struct = {}
            kidattrs = kid.getAttributes()
            kees = kidattrs.keys()
            for kee in kees:
              if kee == "name":
                continue
              instance_struct[kee] = kidattrs[kee]
            inlist.append(instance_struct)
            level1.append(fencedev)
            last_kid_fd = fencedev
            continue
    map['level1'] = level1

    #level1 list is complete now, but it is still necessary to build shared1
    for fd in fds:
      isUnique = True
      if fd.isShared() == False:
        continue
      for fdev in level1:
        if fd.getName().strip() == fdev['name']:
          isUnique = False
          break
      if isUnique == True:
        shared_struct = {}
        shared_struct['name'] = fd.getName().strip()
        agentname = fd.getAgentType()
        shared_struct['agent'] = agentname
        try:
          shared_struct['prettyname'] = FENCE_OPTS[agentname]
        except:
          shared_struct['unknown'] = True
          shared_struct['prettyname'] = agentname
        shared1.append(shared_struct)
    map['shared1'] = shared1

  #YUK: This next section violates the DRY rule, :-(
  if len_levels >= 2:
    second_level = levels[1]
    kids = second_level.getChildren()
    last_kid_fd = None  #This is a marker for allowing multi instances
                        #beneath a fencedev
    for kid in kids:
      instance_name = kid.getName().strip()
      try:
        fd = getFDForInstance(fds, instance_name)
      except:
        fd = None #Set to None in case last time thru loop
        continue
      if fd is not None:
        if fd.isShared() == False:  #Not a shared dev...build struct and add
          fencedev = {}
          try:
            fencedev['prettyname'] = FENCE_OPTS[fd.getAgentType()]
          except:
            fencedev['unknown'] = True
            fencedev['prettyname'] = fd.getAgentType()
          fencedev['isShared'] = False
          fencedev['id'] = str(major_num)
          major_num = major_num + 1
          devattrs = fd.getAttributes()
          kees = devattrs.keys()
          for kee in kees:
            fencedev[kee] = devattrs[kee]
          kidattrs = kid.getAttributes()
          kees = kidattrs.keys()
          for kee in kees:
            if kee == "name":
              continue #Don't duplicate name attr
            fencedev[kee] = kidattrs[kee]
          #This fencedev struct is complete, and needs to be placed on the
          #level2 Q. Because it is non-shared, we should set last_kid_fd
          #to none.
          last_kid_fd = None
          level2.append(fencedev)
        else:  #This dev is shared
          if (last_kid_fd is not None) and (fd.getName().strip() == last_kid_fd['name'].strip()):  #just append a new instance struct to last_kid_fd
            instance_struct = {}
            instance_struct['id'] = str(minor_num)
            minor_num = minor_num + 1
            kidattrs = kid.getAttributes()
            kees = kidattrs.keys()
            for kee in kees:
              if kee == "name":
                continue
              instance_struct[kee] = kidattrs[kee]
            #Now just add this struct to last_kid_fd and reset last_kid_fd
            ilist = last_kid_fd['instance_list']
            ilist.append(instance_struct)
            #last_kid_fd = fd
            continue
          else: #Shared, but not used above...so we need a new fencedev struct
            fencedev = {}
            try:
              fencedev['prettyname'] = FENCE_OPTS[fd.getAgentType()]
            except:
              fencedev['unknown'] = True
              fencedev['prettyname'] = fd.getAgentType()
            fencedev['isShared'] = True
            fencedev['cfgurl'] = baseurl + "?clustername=" + clustername + "&fencename=" + fd.getName().strip() + "&pagetype=" + FENCEDEV
            fencedev['id'] = str(major_num)
            major_num = major_num + 1
            inlist = list()
            fencedev['instance_list'] = inlist
            devattrs = fd.getAttributes()
            kees = devattrs.keys()
            for kee in kees:
              fencedev[kee] = devattrs[kee]
            instance_struct = {}
            kidattrs = kid.getAttributes()
            kees = kidattrs.keys()
            for kee in kees:
              if kee == "name":
                continue
              instance_struct[kee] = kidattrs[kee]
            inlist.append(instance_struct)
            level2.append(fencedev)
            last_kid_fd = fencedev
            continue
    map['level2'] = level2

    #level2 list is complete but like above, we need to build shared2
    for fd in fds:
      isUnique = True
      if fd.isShared() == False:
        continue
      for fdev in level2:
        if fd.getName().strip() == fdev['name']:
          isUnique = False
          break
      if isUnique == True:
        shared_struct = {}
        shared_struct['name'] = fd.getName().strip()
        agentname = fd.getAgentType()
        shared_struct['agent'] = agentname
        try:
          shared_struct['prettyname'] = FENCE_OPTS[agentname]
        except:
          shared_struct['unknown'] = True
          shared_struct['prettyname'] = agentname
        shared2.append(shared_struct)
    map['shared2'] = shared2

  return map

def getFencesInfo(self, model, request):
  map = {}
  if not model:
    luci_log.debug_verbose('getFencesInfo0: model is None')
    map['fencedevs'] = list()
    return map

  clustername = request['clustername']
  baseurl = request['URL']
  fencedevs = list() #This is for the fencedev list page

  #Get list of fence devices
  fds = model.getFenceDevices()
  for fd in fds:
    #This section determines which nodes use the dev
    #create fencedev hashmap
    nodes_used = list()

    if fd.isShared() == True:
      fencedev = {}
      attr_hash = fd.getAttributes()
      kees = attr_hash.keys()

      for kee in kees:
        fencedev[kee] = attr_hash[kee] #copy attrs over
      try:
        fencedev['pretty_name'] = FENCE_OPTS[fd.getAgentType()]
      except:
        fencedev['unknown'] = True
        fencedev['pretty_name'] = fd.getAgentType()

      fencedev['agent'] = fd.getAgentType()
      #Add config url for this fencedev
      fencedev['cfgurl'] = baseurl + "?clustername=" + clustername + "&fencename=" + fd.getName().strip() + "&pagetype=" + FENCEDEV

      nodes = model.getNodes()
      for node in nodes:
        flevels = node.getFenceLevels()
        for flevel in flevels: #These are the method blocks...
          kids = flevel.getChildren()
          for kid in kids: #These are actual devices in each level
            if kid.getName().strip() == fd.getName().strip():
              #See if this fd already has an entry for this node
              found_duplicate = False
              for item in nodes_used:
                if item['nodename'] == node.getName().strip():
                  found_duplicate = True
              if found_duplicate == True:
                continue
              node_hash = {}
              node_hash['nodename'] = node.getName().strip()
              node_hash['nodeurl'] = baseurl + "?clustername=" + clustername + "&nodename=" + node.getName() + "&pagetype=" + NODE
              nodes_used.append(node_hash)

      fencedev['nodesused'] = nodes_used
      fencedevs.append(fencedev)

  map['fencedevs'] = fencedevs
  return map

def getLogsForNode(self, request):
	try:
		nodename = request['nodename']
	except KeyError, e:
		try:
			nodename = request.form['nodename']
		except:
			luci_log.debug_verbose('Unable to get node name to retrieve logging information')
			return 'Unable to get node name to retrieve logging information'

	clustername = None
	try:
		clustername = request['clustername']
	except KeyError, e:
		try:
			clustername = request.form['clusterName']
			if not clustername:
				raise
		except:
			clustername = None
			luci_log.debug_verbose('Unable to find cluster name while retrieving logging information for %s' % nodename)
	except:
		pass

	if clustername is None:
		nodename_resolved = nodename
	else:
		nodename_resolved = resolve_nodename(self, clustername, nodename)

	try:
		rc = RicciCommunicator(nodename_resolved)
	except RicciError, e:
		luci_log.debug_verbose('Ricci error while getting logs for %s: %s' \
			% (nodename_resolved, str(e)))
		return 'Ricci error while getting logs for %s' % nodename_resolved
	except:
		luci_log.debug_verbose('Unexpected exception while getting logs for %s' \
			% nodename_resolved)
		return 'Ricci error while getting logs for %s' % nodename_resolved

	if not rc.authed():
		try:
			snode = getStorageNode(self, nodename)
			setNodeFlag(snode, CLUSTER_NODE_NEED_AUTH)
		except:
			pass

		if clustername:
			try:
				cnode = getClusterNode(self, nodename, clustername)
				setNodeFlag(cnode, CLUSTER_NODE_NEED_AUTH)
			except:
				pass

		return 'Luci is not authenticated to node %s. Please reauthenticate first.' % nodename

	return getNodeLogs(rc)

def getVMInfo(self, model, request):
  map = {}
  baseurl = request['URL']
  clustername = request['clustername']
  svcname = None

  try:
    svcname = request['servicename']
  except KeyError, e:
    svcname = None
  urlstring = baseurl + "?" + clustername + "&pagetype=29"
  if svcname != None:
    urlstring = urlstring + "&servicename=" + svcname

  map['formurl'] = urlstring

  try:
    vmname = request['servicename']
  except:
    try:
      vmname = request.form['servicename']
    except:
      luci_log.debug_verbose('servicename is missing from request')
      return map

  try:
    vm = model.retrieveVMsByName(vmname)
  except:
    luci_log.debug('An error occurred while attempting to get VM %s' \
      % vmname)
    return map

  attrs = vm.getAttributes()
  keys = attrs.keys()
  for key in keys:
    map[key] = attrs[key]
  return map

def isClusterBusy(self, req):
  items = None
  map = {}
  isBusy = False
  redirect_message = False
  nodereports = list()
  map['nodereports'] = nodereports

  try:
    cluname = req['clustername']
  except KeyError, e:
    try:
      cluname = req.form['clustername']
    except:
      try:
        cluname = req.form['clusterName']
      except:
        luci_log.debug_verbose('ICB0: No cluster name -- returning empty map')
        return map

  path = str(CLUSTER_FOLDER_PATH + cluname)
  try:
    clusterfolder = self.restrictedTraverse(path)
    if not clusterfolder:
      raise Exception, 'clusterfolder is None'
  except Exception, e:
    luci_log.debug_verbose('ICB1: cluster %s [%s] folder missing: %s -- returning empty map' % (cluname, path, str(e)))
    return map
  except:
    luci_log.debug_verbose('ICB2: cluster %s [%s] folder missing: returning empty map' % (cluname, path))

  try:
    items = clusterfolder.objectItems('ManagedSystem')
    if not items or len(items) < 1:
      luci_log.debug_verbose('ICB3: NOT BUSY: no flags at %s for cluster %s' \
          % (cluname, path))
      return map  #This returns an empty map, and should indicate not busy
  except Exception, e:
    luci_log.debug('ICB4: An error occurred while looking for cluster %s flags at path %s: %s' % (cluname, path, str(e)))
    return map
  except:
    luci_log.debug('ICB5: An error occurred while looking for cluster %s flags at path %s' % (cluname, path))
    return map

  luci_log.debug_verbose('ICB6: %s is busy: %d flags' \
      % (cluname, len(items)))
  map['busy'] = "true"
  #Ok, here is what is going on...if there is an item,
  #we need to call the ricci_bridge and get a batch report.
  #This report will tell us one of three things:
  ##1) the batch task is complete...delete ManagedSystem and render
  ##normal page
  ##2) The batch task is NOT done, so meta refresh in 5 secs and try again
  ##3) The ricci agent has no recollection of the task, so handle like 1 above
  ###
  ##Here is what we have to do:
  ##the map should have two lists:
  ##One list of non-cluster create tasks
  ##and one of cluster create task structs
  ##For each item in items, check if this is a cluster create tasktype
  ##If so, call RC, and then call stan's batch report method
  ##check for error...if error, report and then remove flag.
  ##if no error, check if complete. If not complete, report status
  ##If complete, report status and remove flag.

  for item in items:
    tasktype = item[1].getProperty(TASKTYPE)
    if tasktype == CLUSTER_ADD or tasktype == NODE_ADD:
      node_report = {}
      node_report['isnodecreation'] = True
      node_report['iserror'] = False  #Default value
      node_report['desc'] = item[1].getProperty(FLAG_DESC)
      batch_xml = None
      ricci = item[0].split("____") #This removes the 'flag' suffix

      luci_log.debug_verbose('ICB6A: using host %s for rc for item %s' \
          % (ricci[0], item[0]))
      try:
        rc = RicciCommunicator(ricci[0])
        if not rc:
          rc = None
          luci_log.debug_verbose('ICB6b: rc is none')
      except Exception, e:
        rc = None
        luci_log.debug_verbose('ICB7: RC: %s: %s' \
          % (cluname, str(e)))

      batch_id = None
      if rc is not None:
        try:
          batch_id = item[1].getProperty(BATCH_ID)
          luci_log.debug_verbose('ICB8: got batch_id %s from %s' \
              % (batch_id, item[0]))
        except Exception, e:
          try:
            luci_log.debug_verbose('ICB8B: failed to get batch_id from %s: %s' \
                % (item[0], str(e)))
          except:
            luci_log.debug_verbose('ICB8C: failed to get batch_id from %s' \
              % item[0])

        if batch_id is not None:
          try:
            batch_xml = rc.batch_report(batch_id)
            if batch_xml is not None:
              luci_log.debug_verbose('ICB8D: batch_xml for %s from batch_report is not None -- getting batch status' % batch_id)
              (creation_status, total) = batch_status(batch_xml)
              try:
                luci_log.debug_verbose('ICB8E: batch status returned (%d,%d)' \
                    % (creation_status, total))
              except:
                luci_log.debug_verbose('ICB8F: error logging batch status return')
            else:
              luci_log.debug_verbose('ICB9: batch_xml for cluster is None')
          except Exception, e:
            luci_log.debug_verbose('ICB9A: error getting batch_xml from rc.batch_report: %s' % str(e))
            creation_status = RICCI_CONNECT_FAILURE  #No contact with ricci (-1000)
            batch_xml = "bloody_failure" #set to avoid next if statement

      if rc is None or batch_id is None:
          luci_log.debug_verbose('ICB12: unable to connect to a ricci agent for cluster %s to get batch status')
          creation_status = RICCI_CONNECT_FAILURE  #No contact with ricci (-1000)
          batch_xml = "bloody_bloody_failure" #set to avoid next if statement

      if batch_xml is None:  #The job is done and gone from queue
        if redirect_message == False: #We have not displayed this message yet
          node_report['desc'] = REDIRECT_MSG
          node_report['iserror'] = True
          node_report['errormessage'] = ""
          nodereports.append(node_report)
          redirect_message = True

        luci_log.debug_verbose('ICB13: batch job is done -- deleting %s' % item[0])
        clusterfolder.manage_delObjects([item[0]])
        continue

      del_db_obj = False
      if creation_status < 0:  #an error was encountered
        luci_log.debug_verbose('ICB13a: %s: CS %d for %s' % (cluname, creation_status, ricci[0]))
        if creation_status == RICCI_CONNECT_FAILURE:
          laststatus = item[1].getProperty(LAST_STATUS)
          if laststatus == INSTALL_TASK: #This means maybe node is rebooting
            node_report['statusindex'] = INSTALL_TASK
            node_report['statusmessage'] = RICCI_CONNECT_FAILURE_MSG + POSSIBLE_REBOOT_MESSAGE
          elif laststatus == 0:
            node_report['statusindex'] = 0
            node_report['statusmessage'] = RICCI_CONNECT_FAILURE_MSG + PRE_INSTALL
          elif laststatus == DISABLE_SVC_TASK:
            node_report['statusindex'] = DISABLE_SVC_TASK
            node_report['statusmessage'] = RICCI_CONNECT_FAILURE_MSG + PRE_CFG
          elif laststatus == REBOOT_TASK:
            node_report['statusindex'] = REBOOT_TASK
            node_report['statusmessage'] = RICCI_CONNECT_FAILURE_MSG + PRE_CFG
          elif laststatus == SEND_CONF:
            node_report['statusindex'] = SEND_CONF
            node_report['statusmessage'] = RICCI_CONNECT_FAILURE_MSG + PRE_JOIN
          elif laststatus == ENABLE_SVC_TASK:
            node_report['statusindex'] = ENABLE_SVC_TASK
            node_report['statusmessage'] = RICCI_CONNECT_FAILURE_MSG + PRE_JOIN
          else:
            node_report['statusindex'] = 0
            node_report['statusmessage'] = RICCI_CONNECT_FAILURE_MSG + ' Install is in an unknown state.'
          nodereports.append(node_report)
          continue
        elif creation_status == -(INSTALL_TASK):
          node_report['iserror'] = True
          (err_code, err_msg) = extract_module_status(batch_xml, INSTALL_TASK)
          node_report['errormessage'] = CLUNODE_CREATE_ERRORS[INSTALL_TASK] + err_msg
          del_db_obj = True
        elif creation_status == -(DISABLE_SVC_TASK):
          node_report['iserror'] = True
          (err_code, err_msg) = extract_module_status(batch_xml, DISABLE_SVC_TASK)
          node_report['errormessage'] = CLUNODE_CREATE_ERRORS[DISABLE_SVC_TASK] + err_msg
          del_db_obj = True
        elif creation_status == -(REBOOT_TASK):
          node_report['iserror'] = True
          (err_code, err_msg) = extract_module_status(batch_xml, REBOOT_TASK)
          node_report['errormessage'] = CLUNODE_CREATE_ERRORS[REBOOT_TASK] + err_msg
          del_db_obj = True
        elif creation_status == -(SEND_CONF):
          node_report['iserror'] = True
          (err_code, err_msg) = extract_module_status(batch_xml, SEND_CONF)
          node_report['errormessage'] = CLUNODE_CREATE_ERRORS[SEND_CONF] + err_msg
        elif creation_status == -(ENABLE_SVC_TASK):
          node_report['iserror'] = True
          (err_code, err_msg) = extract_module_status(batch_xml, DISABLE_SVC_TASK)
          node_report['errormessage'] = CLUNODE_CREATE_ERRORS[ENABLE_SVC_TASK] + err_msg
        elif creation_status == -(START_NODE):
          node_report['iserror'] = True
          (err_code, err_msg) = extract_module_status(batch_xml, START_NODE)
          node_report['errormessage'] = CLUNODE_CREATE_ERRORS[START_NODE]
        else:
          del_db_obj = True
          node_report['iserror'] = True
          node_report['errormessage'] = CLUNODE_CREATE_ERRORS[0]

        try:
          if del_db_obj is True:
            luci_log.debug_verbose('ICB13a: %s node creation failed for %s: %d: deleting DB entry' % (cluname, ricci[0], creation_status))
            clusterfolder.manage_delObjects([ricci[0]])
          clusterfolder.manage_delObjects([item[0]])
        except Exception, e:
          luci_log.debug_verbose('ICB14: delObjects: %s: %s' \
            % (item[0], str(e)))

        nodereports.append(node_report)
        continue
      else:  #either batch completed successfully, or still running
        if creation_status == total:  #finished...
          map['busy'] = "true"
          node_report['statusmessage'] = "Node created successfully" + REDIRECT_MSG
          node_report['statusindex'] = creation_status
          nodereports.append(node_report)
          try:
              clusterfolder.manage_delObjects([item[0]])
          except Exception, e:
              luci_log.info('ICB15: Unable to delete %s: %s' % (item[0], str(e)))
          continue
        else:
          map['busy'] = "true"
          isBusy = True
          node_report['statusmessage'] = "Node still being created"
          node_report['statusindex'] = creation_status
          nodereports.append(node_report)
          propslist = list()
          propslist.append(LAST_STATUS)
          try:
            item[1].manage_delProperties(propslist)
            item[1].manage_addProperty(LAST_STATUS, creation_status, "int")
          except Exception, e:
            luci_log.debug_verbose('ICB16: last_status err: %s %d: %s' \
              % (item[0], creation_status, str(e)))
          continue

    else:
      node_report = {}
      node_report['isnodecreation'] = False
      ricci = item[0].split("____") #This removes the 'flag' suffix

      try:
        rc = RicciCommunicator(ricci[0])
      except Exception, e:
        rc = None
        finished = -1
        err_msg = ''
        luci_log.debug_verbose('ICB15: ricci error: %s: %s' \
          % (ricci[0], str(e)))

      if rc is not None:
        batch_res = checkBatch(rc, item[1].getProperty(BATCH_ID))
        finished = batch_res[0]
        err_msg = batch_res[1]

      if finished == True or finished == -1:
        if finished == -1:
          flag_msg = err_msg
        else:
          flag_msg = ''
        flag_desc = item[1].getProperty(FLAG_DESC)
        if flag_desc is None:
          node_report['desc'] = flag_msg + REDIRECT_MSG
        else:
          node_report['desc'] = flag_msg + flag_desc + REDIRECT_MSG
        nodereports.append(node_report)
        try:
            clusterfolder.manage_delObjects([item[0]])
        except Exception, e:
            luci_log.info('ICB16: Unable to delete %s: %s' % (item[0], str(e)))
      else:
        node_report = {}
        map['busy'] = "true"
        isBusy = True
        node_report['desc'] = item[1].getProperty(FLAG_DESC)
        nodereports.append(node_report)

  if isBusy:
    part1 = req['ACTUAL_URL']
    part2 = req['QUERY_STRING']

    dex = part2.find("&busyfirst")
    if dex != (-1):
      tmpstr = part2[:dex] #This strips off busyfirst var
      part2 = tmpstr
      ###FIXME - The above assumes that the 'busyfirst' query var is at the
      ###end of the URL...
    wholeurl = part1 + "?" + part2
    map['refreshurl'] = "5; url=" + wholeurl
    req['specialpagetype'] = "1"
  else:
    try:
      query = req['QUERY_STRING'].replace('&busyfirst=true', '')
      map['refreshurl'] = '5; url=' + req['ACTUAL_URL'] + '?' + query
    except:
      map['refreshurl'] = '5; url=/luci/cluster?pagetype=3'
  return map

def getClusterOS(self, rc):
	map = {}

	try:
		os_str = resolveOSType(rc.os())
		map['os'] = os_str
		map['isVirtualized'] = rc.dom0()
	except:
		# default to rhel5 if something crazy happened.
		try:
			luci_log.debug('An error occurred while attempting to get OS/Virt info for %s -- defaulting to rhel5/False' % rc.hostname())
		except:
			# this can throw an exception if the original exception
			# is caused by rc being None or stale.
			pass
		map['os'] = 'rhel5'
		map['isVirtualized'] = False
	return map

def getResourcesInfo(model, request):
	resList = list()
	baseurl = request['URL']

	try:
		cluname = request['clustername']
	except:
		try:
			cluname = request.form['clustername']
		except:
			luci_log.debug_verbose('getResourcesInfo missing cluster name')
			return resList

	for item in model.getResources():
		itemmap = {}
		itemmap['name'] = item.getName()
		itemmap['attrs'] = item.attr_hash
		itemmap['type'] = item.resource_type
		itemmap['tag_name'] = item.TAG_NAME
		itemmap['cfgurl'] = baseurl + "?" + "clustername=" + cluname + "&resourcename=" + item.getName() + "&pagetype=" + RESOURCE_CONFIG
		itemmap['url'] = baseurl + "?" + "clustername=" + cluname + "&resourcename=" + item.getName() + "&pagetype=" + RESOURCE
		itemmap['delurl'] = baseurl + "?" + "clustername=" + cluname + "&resourcename=" + item.getName() + "&pagetype=" + RESOURCE_REMOVE
		resList.append(itemmap)
	return resList

def getResourceInfo(model, request):
	if not model:
		luci_log.debug_verbose('GRI0: no model object in session')
		return {}

	name = None
	try:
		name = request['resourcename']
	except:
		try:
			name = request.form['resourcename']
		except:
			pass

	if name is None:
		try:
			res_type = request.form['type']
			if res_type == 'ip':
				name = request.form['value'].strip()
		except:
			pass

	if name is None:
		luci_log.debug_verbose('getResourceInfo missing res name')
		return {}

	try:
		cluname = request['clustername']
	except:
		try:
			cluname = request.form['clustername']
		except:
			luci_log.debug_verbose('getResourceInfo missing cluster name')
			return {}

	try:
		baseurl = request['URL']
	except:
		luci_log.debug_verbose('getResourceInfo missing URL')
		return {}

	for res in model.getResources():
		if res.getName() == name:
			try:
				resMap = {}
				resMap['name'] = res.getName()
				resMap['type'] = res.resource_type
				resMap['tag_name'] = res.TAG_NAME
				resMap['attrs'] = res.attr_hash
				resMap['cfgurl'] = baseurl + "?" + "clustername=" + cluname + "&resourcename=" + res.getName() + "&pagetype=" + RESOURCE_CONFIG
				return resMap
			except:
				continue

def delService(self, request):
	errstr = 'An error occurred while attempting to set the new cluster.conf'

	try:
		model = request.SESSION.get('model')
	except Exception, e:
		luci_log.debug_verbose('delService0: no model: %s' % str(e))
		return (False, {'errors': [ errstr ] })

	name = None
	try:
		name = request['servicename']
	except:
		try:
			name = request.form['servicename']
		except:
			pass

	if name is None:
		luci_log.debug_verbose('delService1: no service name')
		return (False, {'errors': [ '%s: no service name was provided.' % errstr ]})

	clustername = None
	try:
		clustername = request['clustername']
	except:
		try:
			clustername = request.form['clustername']
		except:
			pass

	if clustername is None:
		luci_log.debug_verbose('delService2: no cluster name for %s' % name)
		return (False, {'errors': [ '%s: no cluster name was provided.' % errstr ]})

	rc = getRicciAgent(self, clustername)
	if not rc:
		luci_log.debug_verbose('delService3: unable to get ricci agent for cluster %s' % clustername)
		return (False, {'errors': [ '%s: unable to find a Ricci agent for this cluster.' % errstr ]})

	try:
		ragent = rc.hostname()
		if not ragent:
			raise Exception, 'unable to determine the hostname of the ricci agent'
	except Exception, e:
		luci_log.debug_verbose('delService4: %s: %s' % (errstr, str(e)))
		return (False, {'errors': [ '%s: unable to find a Ricci agent for this cluster.' % errstr ]})

	try:
		model.deleteService(name)
	except Exception, e:
		luci_log.debug_verbose('delService5: Unable to find a service named %s for cluster %s' % (name, clustername))
		return (False, {'errors': [ '%s: error removing service %s.' % (errstr, name) ]})

	try:
		model.setModified(True)
		conf = model.exportModelAsString()
		if not conf:
			raise Exception, 'model string is blank'
	except Exception, e:
		luci_log.debug_verbose('delService6: exportModelAsString failed: %s' \
			% str(e))
		return (False, {'errors': [ '%s: error removing service %s.' % (errstr, name) ]})

	batch_number, result = setClusterConf(rc, str(conf))
	if batch_number is None or result is None:
		luci_log.debug_verbose('delService7: missing batch and/or result')
		return (False, {'errors': [ '%s: error removing service %s.' % (errstr, name) ]})

	try:
		set_node_flag(self, clustername, ragent, str(batch_number), SERVICE_DELETE, "Removing service \'%s\'" % name)
	except Exception, e:
		luci_log.debug_verbose('delService8: failed to set flags: %s' % str(e))

	response = request.RESPONSE
	response.redirect(request['URL'] + "?pagetype=" + SERVICES + "&clustername=" + clustername + '&busyfirst=true')

def delResource(self, rc, request):
	errstr = 'An error occurred while attempting to set the new cluster.conf'

	try:
		model = request.SESSION.get('model')
	except Exception, e:
		luci_log.debug_verbose('delResource0: no model: %s' % str(e))
		return errstr

	name = None
	try:
		name = request['resourcename']
	except:
		try:
			name = request.form['resourcename']
		except:
			pass

	if name is None:
		luci_log.debug_verbose('delResource1: no resource name')
		return errstr + ': no resource name was provided.'

	clustername = None
	try:
		clustername = request['clustername']
	except:
		try:
			clustername = request.form['clustername']
		except:
			pass

	if clustername is None:
		luci_log.debug_verbose('delResource2: no cluster name for %s' % name)
		return errstr + ': could not determine the cluster name.'

	try:
		ragent = rc.hostname()
		if not ragent:
			raise Exception, 'unable to determine the hostname of the ricci agent'
	except Exception, e:
		luci_log.debug_verbose('delResource3: %s: %s' % (errstr, str(e)))
		return errstr + ': could not determine the ricci agent hostname'

	resPtr = model.getResourcesPtr()
	resources = resPtr.getChildren()

	found = 0
	for res in resources:
		if res.getName() == name:
			resPtr.removeChild(res)
			found = 1
			break

	if not found:
		luci_log.debug_verbose('delResource4: cant find res %s' % name)
		return errstr + ': the specified resource was not found.'

	try:
		model.setModified(True)
		conf = model.exportModelAsString()
		if not conf:
			raise Exception, 'model string is blank'
	except Exception, e:
		luci_log.debug_verbose('delResource5: exportModelAsString failed: %s' \
			% str(e))
		return errstr

	batch_number, result = setClusterConf(rc, str(conf))
	if batch_number is None or result is None:
		luci_log.debug_verbose('delResource6: missing batch and/or result')
		return errstr

	try:
		set_node_flag(self, clustername, ragent, str(batch_number), RESOURCE_REMOVE, "Removing resource \'%s\'" % request['resourcename'])
	except Exception, e:
		luci_log.debug_verbose('delResource7: failed to set flags: %s' % str(e))

	response = request.RESPONSE
	response.redirect(request['URL'] + "?pagetype=" + RESOURCES + "&clustername=" + clustername + '&busyfirst=true')

def addIp(request, form=None):
	errors = list()

	if form is None:
		form = request.form

	if not form:
		luci_log.debug_verbose('addIp0: form is missing')
		return None

	model = request.SESSION.get('model')
	if not model:
		luci_log.debug_verbose('addIp1: model is missing')
		return None

	res = None
	if form.has_key('edit'):
		try:
			oldname = form['oldname'].strip()
			if not oldname:
				raise Exception, 'oldname is blank.'

			try:
				res = getResourceForEdit(model, oldname)
			except KeyError, e1:
				errors.append('No IP resource named \"%s\" exists.' % oldname)
		except Exception, e:
			errors.append('No original name was found for this IP resource.')
	else:
		try:
			res = Ip()
			if not res:
				raise Exception, 'res is None'
		except Exception, e:
			errors.append('An error occurred while creating an IP resource.')
			luci_log.debug_verbose('addIp3: %s' % str(e))

	if not res:
		return [None, None, errors]

	try:
		addr = form['ip_address'].strip()
		if not addr:
			raise KeyError, 'ip_address is blank'
		# XXX: validate IP addr
		res.addAttribute('address', addr)
	except KeyError, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addIp4: %s' % err)

	if 'monitorLink' in form:
		res.addAttribute('monitor_link', '1')
	else:
		res.addAttribute('monitor_link', '0')

	if len(errors) > 1:
		return [None, None, errors]
	return [res, model, None]

def addFs(request, form=None):
	errors = list()

	if form is None:
		form = request.form

	if not form:
		luci_log.debug_verbose('addFs0: form is missing')
		return None

	model = request.SESSION.get('model')
	if not model:
		luci_log.debug_verbose('addFs1: model is missing')
		return None

	res = None
	if form.has_key('edit'):
		try:
			oldname = form['oldname'].strip()
			if not oldname:
				raise Exception, 'oldname is blank.'

			try:
				res = getResourceForEdit(model, oldname)
			except KeyError, e1:
				errors.append('No filesystem resource named \"%s\" exists.' % oldname)
		except Exception, e:
			errors.append('No original name was found for this filesystem resource.')
			luci_log.debug_verbose('addFs3: %s' % str(e))
	else:
		try:
			res = Fs()
			if not res:
				raise Exception, 'res is None'
		except Exception, e:
			errors.append('An error occurred while creating a filesystem resource.')
			luci_log.debug_verbose('addFs4: %s' % str(e))

	if not res:
		return [None, None, errors]

	# XXX: sanity check these fields
	try:
		name = form['resourceName'].strip()
		if not name:
			raise Exception, 'No name was given for this filesystem resource.'
		res.addAttribute('name', name)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addFs5: %s' % err)

	try:
		mountpoint = form['mountpoint'].strip()
		if not mountpoint:
			raise Exception, 'No mount point was given for this filesystem resource.'
		res.addAttribute('mountpoint', mountpoint)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addFs6: %s' % err)

	try:
		device = form['device'].strip()
		if not device:
			raise Exception, 'No device was given for this filesystem resource.'
		res.addAttribute('device', device)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addFs7: %s' % err)

	try:
		options = form['options'].strip()
		if not options:
			raise KeyError, 'no options'
		res.addAttribute('options', options)
	except KeyError, e:
		try:
			res.removeAttribute('options')
		except:
			pass
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addFs8: %s' % err)

	try:
		fstype = form['fstype'].strip()
		if not fstype:
			raise Exception, 'No filesystem type was given for this filesystem resource.'
		res.addAttribute('fstype', fstype)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addFs9: %s' % err)

	try:
		fsid = form['fsid'].strip()
		if not fsid:
			raise Exception, 'No filesystem ID was given for this filesystem resource.'
		fsid_int = int(fsid)
		if not fsid_is_unique(model, fsid_int):
			raise Exception, 'The filesystem ID provided is not unique.'
	except Exception, e:
		fsid = str(generate_fsid(model, name))
	res.addAttribute('fsid', fsid)

	if form.has_key('forceunmount'):
		res.addAttribute('force_unmount', '1')
	else:
		res.addAttribute('force_unmount', '0')

	if form.has_key('selffence'):
		res.addAttribute('self_fence', '1')
	else:
		res.addAttribute('self_fence', '0')

	if form.has_key('checkfs'):
		res.addAttribute('force_fsck', '1')
	else:
		res.addAttribute('force_fsck', '0')

	if len(errors) > 1:
		return [None, None, errors]
	return [res, model, None]

def addGfs(request, form=None):
	errors = list()

	if form is None:
		form = request.form

	if not form:
		luci_log.debug_verbose('addGfs0: form is missing')
		return None

	model = request.SESSION.get('model')
	if not model:
		luci_log.debug_verbose('addGfs1: model is missing')
		return None

	res = None
	if form.has_key('edit'):
		try:
			oldname = form['oldname'].strip()
			if not oldname:
				raise Exception, 'oldname is blank'

			try:
				res = getResourceForEdit(model, oldname)
			except KeyError, e1:
				errors.append('No filesystem resource named \"%s\" exists.' % oldname)
		except Exception, e:
			errors.append('No original name was found for this cluster filesystem resource.')
			luci_log.debug_verbose('addGfs2: %s' % str(e))
	else:
		try:
			res = Clusterfs()
			if not res:
				raise Exception, 'res is None'
		except Exception, e:
			errors.append('An error occurred while creating a cluster filesystem resource.')
			luci_log.debug('addGfs3: %s' % str(e))

	if not res:
		return [None, None, errors]

	# XXX: sanity check these fields
	try:
		name = form['resourceName'].strip()
		if not name:
			raise Exception, 'No name was given for this cluster filesystem resource.'
		res.addAttribute('name', name)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addGfs4: %s' % err)

	try:
		mountpoint = form['mountpoint'].strip()
		if not mountpoint:
			raise Exception, 'No mount point was given for this cluster filesystem resource.'
		res.addAttribute('mountpoint', mountpoint)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addGfs5: %s' % err)

	try:
		device = form['device'].strip()
		if not device:
			raise Exception, 'No device was given for this cluster filesystem resource.'
		res.addAttribute('device', device)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addGfs6: %s' % err)

	try:
		options = form['options'].strip()
		if not options:
			raise KeyError, 'no options'
		res.addAttribute('options', options)
	except KeyError, e:
		try:
			res.removeAttribute('options')
		except:
			pass
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addGfs7: %s' % err)

	try:
		fsid = form['fsid'].strip()
		if not fsid:
			raise Exception, 'No filesystem ID was given for this cluster filesystem resource.'
		fsid_int = int(fsid)
		if not fsid_is_unique(model, fsid_int):
			raise Exception, 'The filesystem ID provided is not unique.'
	except Exception, e:
		fsid = str(generate_fsid(model, name))
	res.addAttribute('fsid', fsid)

	if form.has_key('forceunmount'):
		res.addAttribute('force_unmount', '1')
	else:
		res.addAttribute('force_unmount', '0')

	if len(errors) > 1:
		return [None, None, errors]

	return [res, model, None]

def addNfsm(request, form=None):
	errors = list()

	if form is None:
		form = request.form

	if not form:
		luci_log.debug_verbose('addNfsm0: form is missing')
		return None

	model = request.SESSION.get('model')
	if not model:
		luci_log.debug_verbose('addNfsm1: model is missing')
		return None

	res = None
	if form.has_key('edit'):
		try:
			oldname = form['oldname'].strip()
			if not oldname:
				raise Exception, 'oldname is blank'

			try:
				res = getResourceForEdit(model, oldname)
			except KeyError, e1:
				errors.append('No NFS mount resource named \"%s\" exists.' % oldname)
		except Exception, e:
			errors.append('No original name was found for this NFS mount resource.')
			luci_log.debug_verbose('addNfsm2: %s' % str(e))
	else:
		try:
			res = Netfs()
			if not res:
				raise Exception, 'res is None'
		except Exception, e:
			errors.append('An error occurred while creating a NFS mount resource.')
			luci_log.debug_verbose('addNfsm3: %s' % str(e))

	if not res:
		return [None, None, errors]

	# XXX: sanity check these fields
	try:
		name = form['resourceName'].strip()
		if not name:
			raise Exception, 'No name was given for this NFS mount resource.'
		res.addAttribute('name', name)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addNfsm4: %s' % err)

	try:
		mountpoint = form['mountpoint'].strip()
		if not mountpoint:
			raise Exception, 'No mount point was given for NFS mount resource.'
		res.addAttribute('mountpoint', mountpoint)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addNfsm5: %s' % err)

	try:
		host = form['host'].strip()
		if not host:
			raise Exception, 'No host server was given for this NFS mount resource.'
		res.addAttribute('host', host)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addNfsm6 error: %s' % err)

	try:
		options = form['options'].strip()
		if not options:
			raise KeyError, 'no options'
		res.addAttribute('options', options)
	except KeyError, e:
		try:
			res.removeAttribute('options')
		except:
			pass
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addNfsm7: %s' % err)

	try:
		exportpath = form['exportpath'].strip()
		if not exportpath:
			raise Exception, 'No export path was given for this NFS mount resource.'
		res.addAttribute('exportpath', exportpath)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addNfsm8: %s' % err)

	try:
		nfstype = form['nfstype'].strip().lower()
		if nfstype != 'nfs' and nfstype != 'nfs4':
			raise Exception, 'An invalid NFS version \"%s\" was given.' % nfstype
		res.addAttribute('nfstype', nfstype)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addNfsm9: %s' % err)

	if form.has_key('forceunmount'):
		res.addAttribute('force_unmount', '1')
	else:
		res.addAttribute('force_unmount', '0')

	if len(errors) > 1:
		return [None, None, errors]
	return [res, model, None]

def addNfsc(request, form=None):
	errors = list()

	if form is None:
		form = request.form

	if not form:
		luci_log.debug_verbose('addNfsc0: form is missing')
		return None

	model = request.SESSION.get('model')
	if not model:
		luci_log.debug_verbose('addNfsc1: model is missing')
		return None

	res = None
	if form.has_key('edit'):
		try:
			oldname = form['oldname'].strip()
			if not oldname:
				raise Exception, 'oldname is blank'
			try:
				res = getResourceForEdit(model, oldname)
			except KeyError, e1:
				errors.append('No NFS client resource named \"%s\" exists.' % oldname)
		except Exception, e:
			errors.append('No original name was found for this NFS client resource.')
			luci_log.debug_verbose('addNfsc2: %s' % str(e))
	else:
		try:
			res = NFSClient()
			if not res:
				raise Exception, 'res is None'
		except Exception, e:
			errors.append('An error occurred while creating a NFS client resource.')
			luci_log.debug_verbose('addNfsc3: %s' % str(e))

	if not res:
		return [None, None, errors]

	# XXX: sanity check these fields
	try:
		name = form['resourceName'].strip()
		if not name:
			raise Exception, 'No name was given for this NFS client resource.'
		res.addAttribute('name', name)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addNfsc4: %s' % err)

	try:
		target = form['target'].strip()
		if not target:
			raise Exception, 'No target was given for NFS client resource.'
		res.addAttribute('target', target)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addNfsc5: %s' % err)

	try:
		options = form['options'].strip()
		if not options:
			raise KeyError, 'no options'
		res.addAttribute('options', options)
	except KeyError, e:
		try:
			res.removeAttribute('options')
		except:
			pass
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addNfsc6: %s' % err)

	if form.has_key('allow_recover'):
		res.addAttribute('allow_recover', '1')
	else:
		res.addAttribute('allow_recover', '0')

	if len(errors) > 1:
		return [None, None, errors]
	return [res, model, None]

def addNfsx(request, form=None):
	errors = list()

	if form is None:
		form = request.form

	if not form:
		luci_log.debug_verbose('addNfsx0: model is missing')
		return None

	model = request.SESSION.get('model')
	if not model:
		luci_log.debug_verbose('addNfsx0: model is missing')
		return None

	res = None
	if form.has_key('edit'):
		try:
			oldname = form['oldname'].strip()
			if not oldname:
				raise Exception, 'oldname is blank'
			try:
				res = getResourceForEdit(model, oldname)
			except KeyError, e1:
				errors.append('No NFS export resource named \"%s\" exists.' % oldname)
		except Exception, e:
			errors.append('No original name was found for this NFS export resource.')
			luci_log.debug_verbose('addNfsx2: %s', str(e))
	else:
		try:
			res = NFSExport()
			if not res:
				raise Exception, 'res is None'
		except Exception, e:
			errors.append('An error occurred while creating a NFS clientresource.')
			luci_log.debug_verbose('addNfsx3: %s', str(e))

	if not res:
		return [None, None, errors]

	try:
		name = form['resourceName'].strip()
		if not name:
			raise Exception, 'No name was given for this NFS export resource.'
		res.addAttribute('name', name)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addNfsx4: %s' % err)

	if len(errors) > 1:
		return [None, None, errors]
	return [res, model, None]

def addScr(request, form=None):
	errors = list()

	if form is None:
		form = request.form

	if not form:
		luci_log.debug_verbose('addScr0: form is missing')
		return None

	model = request.SESSION.get('model')
	if not model:
		luci_log.debug_verbose('addScr1: model is missing')
		return None

	res = None
	if form.has_key('edit'):
		try:
			oldname = form['oldname'].strip()
			if not oldname:
				raise Exception, 'oldname is blank'
			try:
				res = getResourceForEdit(model, oldname)
			except KeyError, e1:
				errors.append('No script resource named \"%s\" exists.' % oldname)
		except Exception, e:
			errors.append('No original name was found for this script resource.')
			luci_log.debug_verbose('addScr2: %s' % str(e))
	else:
		try:
			res = Script()
			if not res:
				raise Exception, 'res is None'
		except Exception, e:
			errors.append('An error occurred while creating a script resource.')
			luci_log.debug_verbose('addScr3: %s' % str(e))

	if not res:
		return [None, None, errors]

	try:
		name = form['resourceName'].strip()
		if not name:
			raise Exception, 'No name was given for this script resource.'
		res.addAttribute('name', name)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addScr4: %s' % err)

	try:
		path = form['file'].strip()
		if not path:
			raise Exception, 'No path to a script file was given for this script resource.'
		res.addAttribute('file', path)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addScr5: %s' % err)

	if len(errors) > 1:
		return [None, None, errors]
	return [res, model, None]

def addSmb(request, form=None):
	errors = list()

	if form is None:
		form = request.form

	if not form:
		luci_log.debug_verbose('addSmb0: form is missing')
		return None

	model = request.SESSION.get('model')
	if not model:
		luci_log.debug_verbose('addSmb1: model is missing')
		return None

	if form.has_key('edit'):
		try:
			oldname = form['oldname'].strip()
			if not oldname:
				raise Exception, 'oldname is blank'
			try:
				res = getResourceForEdit(model, oldname)
			except KeyError, e1:
				errors.append('No Samba resource named \"%s\" exists.' % oldname)
		except Exception, e:
			errors.append('No original name was found for this Samba resource.')
			luci_log.debug_verbose('addSmb2: %s' % str(e))
	else:
		try:
			res = Samba()
			if not res:
				raise Exception, 'res is None'
		except Exception, e:
			errors.append('An error occurred while creating a Samba resource.')
			luci_log.debug_verbose('addSmb3: %s' % str(e))

	if not res:
		return [None, None, errors]

	try:
		name = form['resourceName'].strip()
		if not name:
			raise Exception, 'No name was given for this Samba resource.'
		res.addAttribute('name', name)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addSmb4: %s' % err)

	try:
		workgroup = form['workgroup'].strip()
		if not workgroup:
			raise Exception, 'No workgroup was given for this Samba resource.'
		res.addAttribute('workgroup', workgroup)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addSmb5: %s' % err)

	if len(errors) > 1:
		return [None, None, errors]
	return [res, model, None]

def addApache(request, form=None):
	errors = list()

	if form is None:
		form = request.form

	if not form:
		luci_log.debug_verbose('addApache0: form is missing')
		return None

	model = request.SESSION.get('model')
	if not model:
		luci_log.debug_verbose('addApache1: model is missing')
		return None

	res = None
	if form.has_key('edit'):
		try:
			oldname = form['oldname'].strip()
			if not oldname:
				raise Exception, 'oldname is blank.'
			try:
				res = getResourceForEdit(model, oldname)
			except KeyError, e:
				errors.append('No Apache resource named \"%s\" exists.' % oldname)
		except Exception, e:
			errors.append('No original name was found for this Apache resource.')
			luci_log.debug_verbose('addApache2: %s' % str(e))
	else:
		try:
			res = Apache()
			if not res:
				raise Exception, 'could not create Apache object'
		except Exception, e:
			errors.append('An error occurred while creating an Apache resource.')
			luci_log.debug_verbose('addApache3: %s' % str(e))

	if not res:
		return [None, None, errors]

	try:
		name = form['resourceName'].strip()
		if not name:
			raise Exception, 'No name was given for this Apache resource.'
		res.addAttribute('name', name)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addApache4: %s' % err)

	try:
		server_root = form['server_root'].strip()
		if not server_root:
			raise KeyError, 'No server root was given for this Apache resource.'
		res.addAttribute('server_root', server_root)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addApache5: %s' % err)

	try:
		config_file = form['config_file'].strip()
		if not server_root:
			raise KeyError, 'No path to the Apache configuration file was given.'
		res.addAttribute('config_file', config_file)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addApache6: %s' % err)

	try:
		options = form['httpd_options'].strip()
		if not options:
			raise KeyError, 'no options'
		res.addAttribute('httpd_options', options)
	except KeyError, e:
		try:
			res.removeAttribute('httpd_options')
		except:
			pass
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addApache7: %s' % err)

	try:
		shutdown_wait = int(form['shutdown_wait'].strip())
		res.addAttribute('shutdown_wait', str(shutdown_wait))
	except KeyError, e:
		res.addAttribute('shutdown_wait', '0')
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addApache7: %s' % err)

	if len(errors) > 1:
		return [None, None, errors]
	return [res, model, None]

def addMySQL(request, form=None):
	errors = list()

	if form is None:
		form = request.form

	if not form:
		luci_log.debug_verbose('addMySQL0: form is missing')
		return None

	model = request.SESSION.get('model')
	if not model:
		luci_log.debug_verbose('addMySQL1: model is missing')
		return None

	res = None
	if form.has_key('edit'):
		try:
			oldname = form['oldname'].strip()
			if not oldname:
				raise Exception, 'oldname is blank.'
			try:
				res = getResourceForEdit(model, oldname)
			except KeyError, e:
				errors.append('No MySQL resource named \"%s\" exists.' % oldname)
		except Exception, e:
			errors.append('No original name was found for this MySQL resource.')
			luci_log.debug_verbose('addMySQL2: %s' % str(e))
	else:
		try:
			res = MySQL()
			if not res:
				raise Exception, 'could not create MySQL object'
		except Exception, e:
			errors.append('An error occurred while creating a MySQL resource.')
			luci_log.debug_verbose('addMySQL3: %s' % str(e))

	if not res:
		return [None, None, errors]

	try:
		name = form['resourceName'].strip()
		if not name:
			raise Exception, 'No name was given for this MySQL resource.'
		res.addAttribute('name', name)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addMySQL4: %s' % err)

	try:
		config_file = form['config_file'].strip()
		if not config_file:
			raise KeyError, 'No path to the MySQL configuration file was given.'
		res.addAttribute('config_file', config_file)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addMySQL5: %s' % err)

	try:
		listen_addr = form['listen_address'].strip()
		if not listen_addr:
			raise KeyError, 'No address was given for MySQL server to listen on.'
		res.addAttribute('listen_address', listen_addr)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addMySQL6: %s' % err)

	try:
		options = form['mysql_options'].strip()
		if not options:
			raise KeyError, 'no options'
		res.addAttribute('mysql_options', options)
	except KeyError, e:
		try:
			res.removeAttribute('mysql_options')
		except:
			pass
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addMySQL7: %s' % err)

	try:
		shutdown_wait = int(form['shutdown_wait'].strip())
		res.addAttribute('shutdown_wait', str(shutdown_wait))
	except KeyError, e:
		res.addAttribute('shutdown_wait', '0')
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addMySQL7: %s' % err)

	if len(errors) > 1:
		return [None, None, errors]
	return [res, model, None]

def addOpenLDAP(request, form=None):
	errors = list()

	if form is None:
		form = request.form

	if not form:
		luci_log.debug_verbose('addOpenLDAP0: form is missing')
		return None

	model = request.SESSION.get('model')
	if not model:
		luci_log.debug_verbose('addOpenLDAP1: model is missing')
		return None

	res = None
	if form.has_key('edit'):
		try:
			oldname = form['oldname'].strip()
			if not oldname:
				raise Exception, 'oldname is blank.'
			try:
				res = getResourceForEdit(model, oldname)
			except KeyError, e:
				errors.append('No OpenLDAP resource named \"%s\" exists.' % oldname)
		except Exception, e:
			errors.append('No original name was found for this OpenLDAP resource.')
			luci_log.debug_verbose('addOpenLDAP2: %s' % str(e))
	else:
		try:
			res = OpenLDAP()
			if not res:
				raise Exception, 'could not create OpenLDAP object'
		except Exception, e:
			errors.append('An error occurred while creating an OpenLDAP resource.')
			luci_log.debug_verbose('addOpenLDAP3: %s' % str(e))

	if not res:
		return [None, None, errors]

	try:
		name = form['resourceName'].strip()
		if not name:
			raise Exception, 'No name was given for this OpenLDAP resource.'
		res.addAttribute('name', name)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addOpenLDAP4: %s' % err)

	try:
		url_list = form['url_list'].strip()
		if not url_list:
			raise KeyError, 'No URL list was given for this OpenLDAP resource.'
		res.addAttribute('url_list', url_list)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addOpenLDAP5: %s' % err)

	try:
		config_file = form['config_file'].strip()
		if not config_file:
			raise KeyError, 'No path to the OpenLDAP configuration file was given.'
		res.addAttribute('config_file', config_file)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addOpenLDAP6: %s' % err)

	try:
		options = form['slapd_options'].strip()
		if not options:
			raise KeyError, 'no options'
		res.addAttribute('slapd_options', options)
	except KeyError, e:
		try:
			res.removeAttribute('slapd_options')
		except:
			pass
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addOpenLDAP7: %s' % err)

	try:
		shutdown_wait = int(form['shutdown_wait'].strip())
		res.addAttribute('shutdown_wait', str(shutdown_wait))
	except KeyError, e:
		res.addAttribute('shutdown_wait', '0')
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addOpenLDAP7: %s' % err)

	if len(errors) > 1:
		return [None, None, errors]
	return [res, model, None]

def addPostgres8(request, form=None):
	errors = list()

	if form is None:
		form = request.form

	if not form:
		luci_log.debug_verbose('addPostgreSQL80: form is missing')
		return None

	model = request.SESSION.get('model')
	if not model:
		luci_log.debug_verbose('addPostgreSQL81: model is missing')
		return None

	res = None
	if form.has_key('edit'):
		try:
			oldname = form['oldname'].strip()
			if not oldname:
				raise Exception, 'oldname is blank.'
			try:
				res = getResourceForEdit(model, oldname)
			except KeyError, e:
				errors.append('No PostgreSQL 8 resource named \"%s\" exists.' % oldname)
		except Exception, e:
			errors.append('No original name was found for this PostgreSQL 8 resource.')
			luci_log.debug_verbose('addPostgreSQL82: %s' % str(e))
	else:
		try:
			res = Postgres8()
			if not res:
				raise Exception, 'could not create PostgreSQL 8 object'
		except Exception, e:
			errors.append('An error occurred while creating a PostgreSQL 8 resource.')
			luci_log.debug_verbose('addPostgreSQL83: %s' % str(e))

	if not res:
		return [None, None, errors]

	try:
		name = form['resourceName'].strip()
		if not name:
			raise Exception, 'No name was given for this PostgreSQL 8 resource.'
		res.addAttribute('name', name)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addPostgreSQL84: %s' % err)

	try:
		user = form['postmaster_user'].strip()
		if not user:
			raise KeyError, 'No postmaster user was given for this PostgreSQL 8 resource.'
		res.addAttribute('postmaster_user', user)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addPostgreSQL85: %s' % err)

	try:
		config_file = form['config_file'].strip()
		if not config_file:
			raise KeyError, 'No path to the PostgreSQL 8 configuration file was given.'
		res.addAttribute('config_file', config_file)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addPostgreSQL86: %s' % err)

	try:
		options = form['postmaster_options'].strip()
		if not options:
			raise KeyError, 'no options'
		res.addAttribute('postmaster_options', options)
	except KeyError, e:
		try:
			res.removeAttribute('postmaster_options')
		except:
			pass
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addPostgreSQL87: %s' % err)

	try:
		shutdown_wait = int(form['shutdown_wait'].strip())
		res.addAttribute('shutdown_wait', str(shutdown_wait))
	except KeyError, e:
		res.addAttribute('shutdown_wait', '0')
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addPostgreSQL87: %s' % err)

	if len(errors) > 1:
		return [None, None, errors]
	return [res, model, None]

def addTomcat5(request, form=None):
	errors = list()

	if form is None:
		form = request.form

	if not form:
		luci_log.debug_verbose('addTomcat50: form is missing')
		return None

	model = request.SESSION.get('model')
	if not model:
		luci_log.debug_verbose('addTomcat51: model is missing')
		return None

	res = None
	if form.has_key('edit'):
		try:
			oldname = form['oldname'].strip()
			if not oldname:
				raise Exception, 'oldname is blank.'
			try:
				res = getResourceForEdit(model, oldname)
			except KeyError, e:
				errors.append('No Tomcat 5 resource named \"%s\" exists.' % oldname)
		except Exception, e:
			errors.append('No original name was found for this Tomcat 5 resource.')
			luci_log.debug_verbose('addTomcat52: %s' % str(e))
	else:
		try:
			res = Tomcat5()
			if not res:
				raise Exception, 'could not create Tomcat5 object'
		except Exception, e:
			errors.append('An error occurred while creating a Tomcat 5 resource.')
			luci_log.debug_verbose('addTomcat53: %s' % str(e))

	if not res:
		return [None, None, errors]

	try:
		name = form['resourceName'].strip()
		if not name:
			raise Exception, 'No name was given for this Tomcat 5 resource.'
		res.addAttribute('name', name)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addTomcat54: %s' % err)

	try:
		user = form['tomcat_user'].strip()
		if not user:
			raise KeyError, 'No user was given for this Tomcat 5 resource.'
		res.addAttribute('tomcat_user', user)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addTomcat55: %s' % err)

	try:
		config_file = form['config_file'].strip()
		if not config_file:
			raise KeyError, 'No path to the Tomcat 5 configuration file was given.'
		res.addAttribute('config_file', config_file)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addTomcat56: %s' % err)

	try:
		options = form['catalina_options'].strip()
		if not options:
			raise KeyError, 'no options'
		res.addAttribute('catalina_options', options)
	except KeyError, e:
		try:
			res.removeAttribute('catalina_options')
		except:
			pass
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addTomcat57: %s' % err)

	try:
		catalina_base = form['catalina_base'].strip()
		if not catalina_base:
			raise KeyError, 'No cataliny base directory was given for this Tomcat 5 resource.'
		res.addAttribute('catalina_base', catalina_base)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addTomcat58: %s' % err)

	try:
		shutdown_wait = int(form['shutdown_wait'].strip())
		res.addAttribute('shutdown_wait', str(shutdown_wait))
	except KeyError, e:
		res.addAttribute('shutdown_wait', '0')
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addTomcat59: %s' % err)

	if len(errors) > 1:
		return [None, None, errors]
	return [res, model, None]

def addLVM(request, form=None):
	errors = list()

	if form is None:
		form = request.form

	if not form:
		luci_log.debug_verbose('addLVM0: form is missing')
		return None

	model = request.SESSION.get('model')
	if not model:
		luci_log.debug_verbose('addLVM1: model is missing')
		return None

	res = None
	if form.has_key('edit'):
		try:
			oldname = form['oldname'].strip()
			if not oldname:
				raise Exception, 'oldname is blank.'
			try:
				res = getResourceForEdit(model, oldname)
			except KeyError, e:
				errors.append('No LVM resource named \"%s\" exists.' % oldname)
		except Exception, e:
			errors.append('No original name was found for this LVM resource.')
			luci_log.debug_verbose('addLVM2: %s' % str(e))
	else:
		try:
			res = LVM()
			if not res:
				raise Exception, 'could not create LVM object'
		except Exception, e:
			errors.append('An error occurred while creating a LVM resource.')
			luci_log.debug_verbose('addLVM3: %s' % str(e))

	if not res:
		return [None, None, errors]

	try:
		name = form['resourceName'].strip()
		if not name:
			raise Exception, 'No name was given for this LVM resource.'
		res.addAttribute('name', name)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addLVM4: %s' % err)

	try:
		vg_name = form['vg_name'].strip()
		if not vg_name:
			raise KeyError, 'No volume group name was given.'
		res.addAttribute('vg_name', vg_name)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addLVM5: %s' % err)

	try:
		lv_name = form['lv_name'].strip()
		if not lv_name:
			raise KeyError, 'No logical volume name was given.'
		res.addAttribute('lv_name', lv_name)
	except Exception, e:
		err = str(e)
		errors.append(err)
		luci_log.debug_verbose('addLVM6: %s' % err)

	if len(errors) > 1:
		return [None, None, errors]
	return [res, model, None]

resourceAddHandler = {
	'ip': addIp,
	'fs': addFs,
	'gfs': addGfs,
	'nfsm': addNfsm,
	'nfsx': addNfsx,
	'nfsc': addNfsc,
	'scr': addScr,
	'smb': addSmb,
	'tomcat-5': addTomcat5,
	'postgres-8': addPostgres8,
	'apache': addApache,
	'openldap': addOpenLDAP,
	'lvm': addLVM,
	'mysql': addMySQL
}

def resolveClusterChanges(self, clusterName, model):
	try:
		mb_nodes = model.getNodes()
		if not mb_nodes or not len(mb_nodes):
			raise Exception, 'node list is empty'
	except Exception, e:
		luci_log.debug_verbose('RCC0: no model builder nodes found for %s: %s' \
				% (str(e), clusterName))
		return 'Unable to find cluster nodes for %s' % clusterName

	try:
		cluster_node = self.restrictedTraverse(PLONE_ROOT + '/systems/cluster/' + clusterName)
		if not cluster_node:
			raise Exception, 'cluster node is none'
	except Exception, e:
		luci_log.debug('RCC1: cant find cluster node for %s: %s'
			% (clusterName, str(e)))
		return 'Unable to find an entry for %s in the Luci database.' % clusterName

	try:
		db_nodes = map(lambda x: x[0], cluster_node.objectItems('Folder'))
		if not db_nodes or not len(db_nodes):
			raise Exception, 'no database nodes'
	except Exception, e:
		# Should we just create them all? Can this even happen?
		luci_log.debug('RCC2: error: %s' % str(e))
		return 'Unable to find database entries for any nodes in %s' % clusterName

	same_host = lambda x, y: x == y or x[:len(y) + 1] == y + '.' or y[:len(x) + 1] == x + '.'

	# this is a really great algorithm.
	missing_list = list()
	new_list = list()
	for i in mb_nodes:
		for j in db_nodes:
			f = 0
			if same_host(i, j):
				f = 1
				break
		if not f:
			new_list.append(i)

	for i in db_nodes:
		for j in mb_nodes:
			f = 0
			if same_host(i, j):
				f = 1
				break
		if not f:
			missing_list.append(i)

	messages = list()
	for i in missing_list:
		try:
			## or alternately
			##new_node = cluster_node.restrictedTraverse(i)
			##setNodeFlag(self, new_node, CLUSTER_NODE_NOT_MEMBER)
			cluster_node.delObjects([i])
			messages.append('Node \"%s\" is no longer in a member of cluster \"%s\." It has been deleted from the management interface for this cluster.' % (i, clusterName))
			luci_log.debug_verbose('VCC3: deleted node %s' % i)
		except Exception, e:
			luci_log.debug_verbose('VCC4: delObjects: %s: %s' % (i, str(e)))

	new_flags = CLUSTER_NODE_NEED_AUTH | CLUSTER_NODE_ADDED
	for i in new_list:
		try:
			cluster_node.manage_addFolder(i, '__luci__:csystem:' + clusterName)
			new_node = cluster_node.restrictedTraverse(i)
			setNodeFlag(self, new_node, new_flags)
			messages.append('A new cluster node, \"%s,\" is now a member of cluster \"%s.\" It has been added to the management interface for this cluster, but you must authenticate to it in order for it to be fully functional.' % (i, clusterName))
		except Exception, e:
			messages.append('A new cluster node, \"%s,\" is now a member of cluster \"%s,\". but it has not been added to the management interface for this cluster as a result of an error creating a database entry for it.' % (i, clusterName))
			luci_log.debug_verbose('VCC5: addFolder: %s/%s: %s' \
				% (clusterName, i, str(e)))

	return messages

def addResource(self, request, model, res, res_type):
	clustername = model.getClusterName()
	if not clustername:
		luci_log.debug_verbose('addResource0: no cluname from mb')
		return 'Unable to determine cluster name'

	rc = getRicciAgent(self, clustername)
	if not rc:
		luci_log.debug_verbose('addResource1: unable to find a ricci agent for cluster %s' % clustername)
		return 'Unable to find a ricci agent for the %s cluster' % clustername

	try:
		model.getResourcesPtr().addChild(res)
	except Exception, e:
		luci_log.debug_verbose('addResource2: adding the new resource failed: %s' % str(e))
		return 'Unable to add the new resource'

	try:
		model.setModified(True)
		conf = model.exportModelAsString()
		if not conf:
			raise Exception, 'model string for %s is blank' % clustername
	except Exception, e:
		luci_log.debug_verbose('addResource3: exportModelAsString : %s' \
			% str(e))
		return 'An error occurred while adding this resource'

	try:
		ragent = rc.hostname()
		if not ragent:
			luci_log.debug_verbose('addResource4: missing ricci hostname')
			raise Exception, 'unknown ricci agent hostname'

		batch_number, result = setClusterConf(rc, str(conf))
		if batch_number is None or result is None:
			luci_log.debug_verbose('addResource5: missing batch_number or result')
			raise Exception, 'unable to save the new cluster configuration.'
	except Exception, e:
		luci_log.debug_verbose('addResource6: %s' % str(e))
		return 'An error occurred while propagating the new cluster.conf: %s' % str(e)

	if res_type != 'ip':
		res_name = res.attr_hash['name']
	else:
		res_name = res.attr_hash['address']

	try:
		try:
			if request.form.has_key('edit'):
				action_type = RESOURCE_CONFIG
				action_str = 'Configuring resource \"%s\"' % res_name
			else:
				raise Exception, 'new'
		except Exception, e:
			action_type = RESOURCE_ADD
			action_str = 'Creating new resource \"%s\"' % res_name

		set_node_flag(self, clustername, ragent, str(batch_number), action_type, action_str)
	except Exception, e:
		luci_log.debug_verbose('addResource7: failed to set flags: %s' % str(e))

	response = request.RESPONSE
	response.redirect(request['URL'] + "?pagetype=" + RESOURCES + "&clustername=" + clustername + '&busyfirst=true')

def getResource(model, name):
	resPtr = model.getResourcesPtr()
	resources = resPtr.getChildren()

	for res in resources:
		if res.getName() == name:
			return res

	luci_log.debug_verbose('getResource: unable to find resource \"%s\"' % name)
	raise KeyError, name

def getResourceForEdit(model, name):
	resPtr = model.getResourcesPtr()
	resources = resPtr.getChildren()

	for res in resources:
		if res.getName() == name:
			resPtr.removeChild(res)
			return res

	luci_log.debug_verbose('GRFE0: unable to find resource \"%s\"' % name)
	raise KeyError, name

def appendModel(request, model):
	try:
		request.SESSION.set('model', model)
	except:
		luci_log.debug_verbose('Appending model to request failed')
		return 'An error occurred while storing the cluster model.'

def resolve_nodename(self, clustername, nodename):
	path = str(CLUSTER_FOLDER_PATH + clustername)

	try:
		clusterfolder = self.restrictedTraverse(path)
		objs = clusterfolder.objectItems('Folder')
	except Exception, e:
		luci_log.debug_verbose('RNN0: error for %s/%s: %s' \
			% (nodename, clustername, str(e)))
		return nodename

	for obj in objs:
		try:
			if obj[0].find(nodename) != (-1):
				return obj[0]
		except:
			continue

	luci_log.debug_verbose('RNN1: failed for %s/%s: nothing found' \
		% (nodename, clustername))
	return nodename

def noNodeFlagsPresent(self, nodefolder, flagname, hostname):
	try:
		items = nodefolder.objectItems('ManagedSystem')
	except:
		luci_log.debug('NNFP0: error getting flags for %s' % nodefolder[0])
		return None

	for item in items:
		if item[0] != flagname:
			continue

		#a flag already exists... try to delete it
		try:
			# hostname must be a FQDN
			rc = RicciCommunicator(hostname)
		except Exception, e:
			luci_log.info('NNFP1: ricci error %s: %s' % (hostname, str(e)))
			return None

		if not rc.authed():
			try:
				snode = getStorageNode(self, hostname)
				setNodeFlag(snode, CLUSTER_NODE_NEED_AUTH)
			except:
				pass
			luci_log.info('NNFP2: %s not authenticated' % item[0])

		batch_ret = checkBatch(rc, item[1].getProperty(BATCH_ID))
		finished = batch_ret[0]
		if finished == True or finished == -1:
			if finished == -1:
				luci_log.debug_verbose('NNFP2: batch error: %s' % batch_ret[1])
			try:
				nodefolder.manage_delObjects([item[0]])
			except Exception, e:
				luci_log.info('NNFP3: manage_delObjects for %s failed: %s' \
					% (item[0], str(e)))
				return None
			return True
		else:
			#Not finished, so cannot remove flag
			return False

	return True

def getModelBuilder(self, rc, isVirtualized):
	try:
		cluster_conf_node = getClusterConf(rc)
		if not cluster_conf_node:
			raise Exception, 'getClusterConf returned None'
	except Exception, e:
		luci_log.debug_verbose('GMB0: unable to get cluster_conf_node in getModelBuilder: %s' % str(e))
		return None

	try:
		model = ModelBuilder(0, None, None, cluster_conf_node)
		if not model:
			raise Exception, 'ModelBuilder returned None'
	except Exception, e:
		try:
			luci_log.debug_verbose('GMB1: An error occurred while trying to get model for conf \"%s\": %s' % (cluster_conf_node.toxml(), str(e)))
		except:
			luci_log.debug_verbose('GMB1: ModelBuilder failed')

	if model:
		model.setIsVirtualized(isVirtualized)
	return model

def getModelForCluster(self, clustername):
	rc = getRicciAgent(self, clustername)
	if not rc:
		luci_log.debug_verbose('GMFC0: unable to find a ricci agent for %s' \
			% clustername)
		return None

	try:
		model = getModelBuilder(None, rc, rc.dom0())
		if not model:
			raise Exception, 'model is none'
	except Exception, e:
		luci_log.debug_verbose('GMFC1: unable to get model builder for %s: %s' \
			 % (clustername, str(e)))
		return None

	return model

def set_node_flag(self, cluname, agent, batchid, task, desc):
	path = str(CLUSTER_FOLDER_PATH + cluname)
	batch_id = str(batchid)
	objname = str(agent + '____flag')

	objpath = ''
	try:
		clusterfolder = self.restrictedTraverse(path)
		clusterfolder.manage_addProduct['ManagedSystem'].addManagedSystem(objname)
		objpath = str(path + '/' + objname)
		flag = self.restrictedTraverse(objpath)
		flag.manage_addProperty(BATCH_ID, batch_id, 'string')
		flag.manage_addProperty(TASKTYPE, task, 'string')
		flag.manage_addProperty(FLAG_DESC, desc, 'string')
	except Exception, e:
		errmsg = 'SNF0: error creating flag (%s,%s,%s) at %s: %s' \
					% (batch_id, task, desc, objpath, str(e))
		luci_log.debug_verbose(errmsg)
		raise Exception, errmsg











def process_cluster_conf_editor(self, req):
	clustername = req['clustername']
	msg = '\n'
	cc = ''
	if 'new_cluster_conf' in req:
		cc = req['new_cluster_conf']
		msg += 'Checking if valid XML - '
		cc_xml = None
		try:
			cc_xml = minidom.parseString(cc)
		except:
			pass
		if cc_xml == None:
			msg += 'FAILED\n'
			msg += 'Fix the error and try again:\n'
		else:
			msg += 'PASSED\n'

			msg += 'Making sure no clustername change has accured - '
			new_name = cc_xml.firstChild.getAttribute('name')
			if new_name != clustername:
				msg += 'FAILED\n'
				msg += 'Fix the error and try again:\n'
			else:
				msg += 'PASSED\n'

				msg += 'Increasing cluster version number - '
				version = cc_xml.firstChild.getAttribute('config_version')
				version = int(version) + 1
				cc_xml.firstChild.setAttribute('config_version', str(version))
				msg += 'DONE\n'

				msg += 'Propagating new cluster.conf'
				rc = getRicciAgent(self, clustername)
				if not rc:
					luci_log.debug_verbose('VFA: unable to find a ricci agent for the %s cluster' % clustername)
					msg += '\nUnable to contact a ricci agent for cluster ' + clustername + '\n\n'
				else:
					batch_id, result = setClusterConf(rc, cc_xml.toxml())
					if batch_id is None or result is None:
						luci_log.debug_verbose('VFA: setClusterConf: batchid or result is None')
						msg += '\nUnable to propagate the new cluster configuration for ' + clustername + '\n\n'
					else:
						msg += ' - DONE\n'
						cc = cc_xml.toxml()
						msg += '\n\nALL DONE\n\n'
	else:
		if getClusterInfo(self, None, req) == {}:
			msg = 'invalid cluster'
		else:
			model = req.SESSION.get('model')
			cc = model.exportModelAsString()
	return {'msg'              : msg,
		'cluster_conf'     : cc}
